@@ -35,10 +35,10 @@
 #ifdef HAVE_STDLIB_H
 #include <stdlib.h>
 #endif
-
+#include <strings.h>
 #include <string.h>
 #include <ctype.h>
-
+#include <limits.h>
 
 #ifdef HAVE_UNAME
 #include <sys/utsname.h>
@@ -87,7 +87,12 @@ struct utsname
 #define WTERMSIG(s) ((s) & 0)
 #endif
 
-
+#include <openssl/err.h>
+#include <openssl/pem.h>
+#include <openssl/evp.h>
+#include <openssl/rsa.h>
+#include <openssl/rand.h>
+#include <openssl/bn.h>
 #include <errno.h>
 
 #ifdef HAVE_DIRENT_H
@@ -183,20 +188,18 @@ extern int errno;
 # include <sys/sockio.h>
 #endif
 
-#ifdef INET
-# include <sys/socket.h>
-# include <sys/ioctl.h>
-# include <net/if.h>
-# include <netinet/in.h>
-# ifndef AOS
-#  include <arpa/inet.h>
-# endif
-# include <netdb.h>
-# if !defined LINUX && !defined NT
-# include <sys/protosw.h>
-# undef sgi
-# include <net/route.h>
-# endif
+#include <sys/socket.h>
+#include <sys/ioctl.h>
+#include <net/if.h>
+#include <netinet/in.h>
+#ifndef AOS
+# include <arpa/inet.h>
+#endif
+#include <netdb.h>
+#if !defined LINUX && !defined NT
+#include <sys/protosw.h>
+#undef sgi
+#include <net/route.h>
 #endif
 
 #ifdef LINUX
@@ -217,6 +220,11 @@ extern int errno;
 # include "../pub/gnuregex.h"
 #endif
 
+#ifndef HAVE_SNPRINTF
+#include "../pub/snprintf.h"
+#endif
+
+
 /*******************************************************************/
 /* Various defines                                                 */
 /*******************************************************************/
@@ -229,10 +237,11 @@ extern int errno;
 #define maxlinksize 256
 #define maxlinklevel 4
 #define maxargs 31
+#define cfmaxiplen 64       /* numerical ip length */
 #define noproccols 16
 #define hashtablesize 1021  /* prime number */
 #define macroalphabet 61    /* a-z, A-Z plus a bit */
-#define maxshellargs 10
+#define maxshellargs 30
 #define samemode 0
 #define sameowner -1
 #define cfnosize    -1
@@ -241,6 +250,11 @@ extern int errno;
 #define ticksperday 86400 /* 60 * 60 *24 */
 #define cf_not_connected -1
 #define recursion_limit 100
+#define cf_monday_morning 342000
+
+#define exec_ifelapsed 0
+#define exec_expireafter 10
+
 
 /* Need this to to avoid conflict with solaris 2.6 and db.h */
 
@@ -256,14 +270,20 @@ extern int errno;
 /* This is the only place you ever need to edit anything           */
 /*******************************************************************/
 
-#define clssattr 33         /* increase this for each new class added */
+#define CLSSATTR 35         /* increase this for each new class added */
                             /* It defines the array size for class data */
-#define attrdim 3           /* Only used in CLASSATTRUBUTES[][] defn */
+#define ATTRDIM 3           /* Only used in CLASSATTRUBUTES[][] defn */
 
    /* end class array limits */
 
 /*******************************************************************/
 
+#define AVDB_FILE     "av.db"
+#define DISTDB_FILE   "dist.db"
+#define STATELOG_FILE "state_log"
+#define ENV_NEW_FILE  "env_data.new"
+#define ENV_FILE      "env_data"
+
 #define CFINPUTSVAR "CFINPUTS"          /* default name for file path var */
 #define CFALLCLASSESVAR "CFALLCLASSES"  /* default name for CFALLCLASSES env */
 #define INFINITERECURSE -99             /* code used to signify inf in recursion */
@@ -277,13 +297,23 @@ extern int errno;
 #define CFD_TERMINATOR "---cfXen/gine/cfXen/gine---"
 #define CFD_TRUE "CFD_TRUE"
 #define CFD_FALSE "CFD_FALSE"
+#define CF_ANYCLASS "any"
+#define CF_NOCLASS "XX_CF_opposite_any_XX"
 #define CF_NOUSER -99
+#define CF_RSA_PROTO_OFFSET 24
+#define CF_PROTO_OFFSET 16
+#define CF_SMALL_OFFSET 2
+#define CF_MD5_LEN 16
+#define CF_SHA_LEN 20
 
-#define CFFAILEDSTR "BAD: Host authentication failed. Did you forget the domain name in cfengine.conf or cfd.conf?"
+#define CF_DONE 't'
+#define CF_MORE 'm'
+
+#define CFFAILEDSTR "BAD: Host authentication failed. Did you forget the domain name?"
 
 #define CF_START_DOMAIN "undefined.domain"
 
-#define CFLOGSIZE 16384                  /* Size of lock-log before rotation */
+#define CFLOGSIZE 1048576                  /* Size of lock-log before rotation */
 
 /* Output control defines */
 
@@ -298,11 +328,34 @@ extern int errno;
 #define DaemonOnly if (ISCFENGINE) yyerror("This belongs in cfd.conf")
 #define CfengineOnly if (! ISCFENGINE) yyerror("This belongs in cfengine.conf")
 
-
 /* GNU REGEX */
 
 #define BYTEWIDTH 8
 
+
+#define ATTR   11
+#define CFWEEK (7.0*24.0*3600.0)
+#define MEASURE_INTERVAL (5.0*60.0)
+
+/*****************************************************************************/
+
+struct Averages
+   {
+   double expect_number_of_users;
+   double expect_rootprocs;
+   double expect_otherprocs;
+   double expect_diskfree;
+   double expect_incoming[ATTR];
+   double expect_outgoing[ATTR];
+
+   double var_number_of_users;
+   double var_rootprocs;
+   double var_otherprocs;
+   double var_diskfree;
+   double var_incoming[ATTR];
+   double var_outgoing[ATTR];
+   };
+
 /*******************************************************************/
 /* Copy file defines                                               */
 /*******************************************************************/
@@ -357,6 +410,23 @@ extern int errno;
 
 #define CFENGINE_SERVICE "cfengine"
 
+enum PROTOS
+   {
+   cfd_exec,
+   cfd_auth,
+   cfd_get,
+   cfd_opendir,
+   cfd_synch,
+   cfd_classes,
+   cfd_md5,
+   cfd_smd5,
+   cfd_cauth,
+   cfd_sauth,
+   cfd_ssynch,
+   cfd_sget,
+   cfd_bad
+   };
+
 #define CF_WORDSIZE 8 /* Number of bytes in a word */
 
 enum cf_filetype
@@ -414,24 +484,23 @@ enum cfsizes
    cfpercent
    };
 
-/*******************************************************************/
-/* These are to reuse some of the GLOBAL buffers used in parsing   */
+
 /*******************************************************************/
 
-/* If you don't understand why global variables are important
-   don't mess with these */
-
-#define EXPR            LINKFROM       /* buff */
-#define CURRENTAUTHPATH ALLCLASSBUFFER /* buff */
-#define RESTART         LINKTO         /* buff */
-#define FILTERDATA      LINKTO         /* buff */
-#define PROSIGNAL       ROTATE         /* short*/
-
-#define PROACTION       LINKTYPE       /* char starts as 's' */
-#define PROCOMP         COPYTYPE       /* char */
-#define IMGCOMP         DISCOMP        /* char */
-
-#define IMGSIZE         DISABLESIZE    /* int */
+enum builtin
+   {
+   nofn,
+   fn_randomint,
+   fn_newerthan,
+   fn_accessedbefore,
+   fn_changedbefore,
+   fn_fileexists,
+   fn_isdir,
+   fn_islink,
+   fn_isplain,
+   fn_execresult,
+   fn_returnszero
+   };
 
 /*******************************************************************/
 
@@ -449,6 +518,7 @@ enum actions
    binservers,
    mailserver,
    required,
+   disks,
    mountables,
    links,
    import,
@@ -465,7 +535,8 @@ enum actions
    deny,
    acls,
    interfaces,
-   filters
+   filters,
+   strategies
    };
 
 /*******************************************************************/
@@ -501,6 +572,8 @@ enum classes
    cfnt,
    unix_sv,
    cfsco,
+   darwin,
+   ux4800,
    unused1,
    unused2,
    unused3
@@ -539,6 +612,8 @@ enum fileattr  /* See COMMATTRIBUTES[] in globals.c  for matching entry */
    cftype,
    cfdest,
    cfforce,
+   cfforcedirs,
+   cfforceipv4,
    cfbackup,
    cfrotate,
    cfsize,
@@ -571,7 +646,8 @@ enum fileattr  /* See COMMATTRIBUTES[] in globals.c  for matching entry */
    cfstealth,
    cfchksum,
    cfflags,
-   cfsecure,
+   cfencryp,
+   cfverify,
    cfroot,
    cftypecheck,
    cfumask,
@@ -583,6 +659,8 @@ enum fileattr  /* See COMMATTRIBUTES[] in globals.c  for matching entry */
    cfpreview,
    cfrepository,
    cftimestamps,
+   cftrustkey,
+   cfcompat,
    cfbad
    };
 
@@ -622,9 +700,11 @@ enum vnames
    cfhomepat,
    cfaddclass,
    cfinstallclass,
+   cfschedule,
    cfaccess,
    cfclass,
    cfarch,
+   cfarch2,
    cfdate,
    cfyear,
    cfmonth,
@@ -653,9 +733,12 @@ enum vnames
    cfifelapsed,
    cfextension,
    cfsuspicious,
+   cfspooldirs,
    cfnonattackers,
    cfattackers,
    cfmulticonn,
+   cftrustkeys,
+   cfallowusers,
    cfskipverify,
    cfdefcopy,
    nonexistentvar
@@ -683,10 +766,9 @@ enum aseq
    {
    mkpaths,
    lnks,
-   simplelnks,
-   childlnks,
    chkmail,
    requir,
+   diskreq,
    tidyf,
    shellcom,
    chkfiles,
@@ -777,6 +859,8 @@ enum editnames
    BeginGroupIfFileIsNewer,
    BeginGroupIfFileExists,
    BeginGroupIfNoLineContaining,
+   BeginGroupIfDefined,
+   BeginGroupIfNotDefined,
    AutoCreate,
    ForEachLineIn,
    EndLoop,
@@ -799,6 +883,8 @@ enum editnames
    EditInclude,
    EditRepos,
    EditUmask,
+   EditUseShell,
+   EditFilter,
    };
 
 enum RegExpTypes
@@ -865,16 +951,37 @@ enum cfoutputlevel
 
 enum modestate
    {
+   wild,
    who,
    which
    };
 
+enum modesort
+   {
+   unknown,
+   numeric,
+   symbolic
+   };
+
 /*******************************************************************/
 
 typedef char flag;
 
 /*******************************************************************/
 
+struct cfagent_connection
+   {
+   int sd;
+   int trust;
+   int family;     /* AF_INET or AF_INET6 */
+   char localip[cfmaxiplen];
+   char remoteip[cfmaxiplen];
+   unsigned char *session_key;
+   short error;
+   };
+
+/*******************************************************************/
+
 struct CompressedArray
    {
    int key;
@@ -886,6 +993,7 @@ struct CompressedArray
 
 struct Interface
    {
+   char done;
    char *ifdev;
    char *netmask;
    char *broadcast;
@@ -897,6 +1005,7 @@ struct Interface
 
 struct Item
    {
+   char done;
    char *name;
    char *classes;
    struct Item *next;
@@ -918,6 +1027,7 @@ struct TwoDimList
 
 struct Process
    {
+   char done;
    char           *expr;          /* search regex */
    char           *restart;       /* shell comm to be done after */
    char           *chdir;
@@ -953,8 +1063,11 @@ struct Mountables
 
 struct Tidy
    {
-   int                recurse;
-   char               *path;
+   int          maxrecurse;              /* sets maxval */
+   char         done;       /* Too intensive in Tidy Pattern */
+   char        *path;
+   struct Item *exclusions;
+   struct Item *ignores;      
    struct TidyPattern *tidylist;
    struct Tidy        *next;   
    };
@@ -966,7 +1079,7 @@ struct Tidy
          int                recurse;
          short              age;
          int                size;              /* in bytes */
-         char               *pattern;
+         char               *pattern;        /* synonym for pattern */
          struct Item        *filters;
 	 char               *classes;
 	 char               *defines;
@@ -996,6 +1109,7 @@ struct Mounted
 
 struct MiscMount
    {
+   char done;
    char *from;
    char *onto;
    char *options;
@@ -1007,9 +1121,10 @@ struct MiscMount
 
 struct UnMount
    {
+   char done;
    char *name;
    char *classes;
-   char deletedir;  /* t/f - true false */
+   char deletedir;  /* y/n - true false */
    char deletefstab;
    char force;
    struct UnMount *next;
@@ -1019,6 +1134,7 @@ struct UnMount
 
 struct File
    {
+   char   done;
    char   *path;
    char   *defines;
    char   *elsedef;
@@ -1048,6 +1164,7 @@ struct File
 
 struct Disk
    {
+   char  done;
    char  *name;
    char  *classes;
    char  *define;
@@ -1062,6 +1179,7 @@ struct Disk
 
 struct Disable
    {
+   char  done;
    char  *name;
    char  *classes;
    char  *type;
@@ -1069,6 +1187,7 @@ struct Disable
    short  rotate;
    int    size;
    char   comp;
+   char   action;   /* d=delete,w=warn */
    char   *defines;
    char   *elsedef;
    struct Item  *filters;
@@ -1081,6 +1200,7 @@ struct Disable
 
 struct Image
    {
+   char   done;
    char   *path;
    char   *destination;
    char   *server;
@@ -1094,14 +1214,16 @@ struct Image
    char   *defines;
    char   *elsedef;
    char   force;                                     /* true false */
+   char   forcedirs;
+   char   forceipv4;
    char   type;                         /* checksum, ctime, binary */
    char   linktype;         /* if file is linked instead of copied */
    char   stealth;               /* preserve times on source files */
    char   preservetimes;                 /* preserve times in copy */
-   short  backup;
+   char   backup;
    int    recurse;
    int    makeholes;
-   off_t    size;
+   off_t  size;
    char   comp;
    char   purge;
    struct Item *exclusions;
@@ -1114,13 +1236,17 @@ struct Image
    struct CompressedArray *inode_cache;              /* inode cache */
  
    struct in_addr *dns;                      /* Cache gethostbyname */
+   int    addrfamily;
    
    struct Image *next;
    struct Item *acl_aliases;
    char   log;
    char   inform;
    char   typecheck;
-   short  secure;
+   char   trustkey;
+   char   encrypt;
+   char   verify;
+   char   compat;
    u_long plus_flags;    /* for *BSD chflags */
    u_long minus_flags;    /* for *BSD chflags */      
    };
@@ -1185,12 +1311,13 @@ struct CFACE
 
 struct Link
    {
+   char   done;
    char   *from;
    char   *to;
    char   *classes;
    char   *defines;
    char   *elsedef;
-   short  force;
+   char   force;
    short  nofile;
    short  silent;
    char   type;
@@ -1199,6 +1326,7 @@ struct Link
    struct Item *exclusions;
    struct Item *inclusions;
    struct Item *ignores;            
+   struct Item *filters;      
    struct Item *copy;
    struct Link *next;
    char   log;
@@ -1209,13 +1337,15 @@ struct Link
 
 struct Edit
    {
+   char done;     /* Have this here, too dangerous in Edlist */
    char *fname;
    char *defines;
    char *elsedef;
    mode_t umask;
+   char useshell;
    char *repository;
    int   recurse;
-   char  binary;   /* t/f */
+   char  binary;   /* y/n */
    struct Item *ignores;
    struct Item *exclusions;
    struct Item *inclusions;      
@@ -1227,7 +1357,7 @@ struct Edit
    /**** SUB-CLASS ********************************************/
 
       struct Edlist
-         {
+         {	 
          enum editnames code;
          char *data;
          struct Edlist *next;
@@ -1292,6 +1422,7 @@ struct Filter
 
 struct ShellComm
    {
+   char              done;
    char              *name;
    char              *classes;
    char              *chdir;
@@ -1304,7 +1435,7 @@ struct ShellComm
    struct ShellComm  *next;
    char              log;
    char              inform;
-   char              fork;      /* t,f */
+   char              fork;
    char              *defines;
    char              *elsedef;
    char              preview;
@@ -1317,10 +1448,21 @@ struct Auth
    char *path;
    struct Item *accesslist;
    struct Item *maproot;    /* which hosts should have root read access */
-   int secure;              /* which files HAVE to be transmitted securely */
+   int encrypt;              /* which files HAVE to be transmitted securely */
    struct Auth *next;
    };
 
+/*******************************************************************/
+
+struct Strategy
+   {
+   char   *name;
+   char   *classes;
+   char   type;                 /* default r=random */
+   struct Item *strategies;
+   struct Strategy *next;
+   };
+
 /*******************************************************************/
 /* Ultrix/BSD don't have all these from sys/stat.h                 */
 /*******************************************************************/
@@ -1429,6 +1571,10 @@ struct Auth
 #  define O_BINARY 0
 #endif
 
+/*******************************************************************/
+/* File path manipulation primitives				   */
+/*******************************************************************/
+
 /* Defined maximum length of a filename. */
 
 #ifdef NT
@@ -1437,6 +1583,20 @@ struct Auth
 #  define MAX_FILENAME 254
 #endif
 
+/* File node separator (cygwin can use \ or / but prefer \ for communicating
+ * with native windows commands). */
+
+#ifdef NT
+#  define IsFileSep(c) ((c) == '\\' || (c) == '/')
+#  define FILE_SEPARATOR '\\'
+#  define FILE_SEPARATOR_STR "\\"
+#else
+#  define IsFileSep(c) ((c) == '/')
+#  define FILE_SEPARATOR '/'
+#  define FILE_SEPARATOR_STR "/"
+#endif
+
+
 /********************************************************************/
 /* All prototypes                                                   */
 /********************************************************************/
