@@ -28,17 +28,20 @@
 /*                                                                 */
 /*******************************************************************/
 
-/* TCP wrappers */
-
-extern int     allow_severity;
-extern int     deny_severity;
+extern RSA *PRIVKEY, *PUBKEY;
 
 /* cfengine */
 
 extern char PADCHAR;
-extern int SD;
+extern struct cfagent_connection *CONN;
 extern int AUTHENTICATED;
 
+extern char CFLOCK[bufsize];
+extern char CFLOG[bufsize];
+extern char CFLAST[bufsize];
+extern char LOCKDB[bufsize];
+
+
 extern char *tzname[2];
 extern char *optarg;
 extern int optind;
@@ -48,27 +51,34 @@ extern char CFDES1[8];
 extern char CFDES2[8];
 extern char CFDES3[8];
 
+extern char CFPUBKEYFILE[bufsize];
+extern char CFPRIVKEYFILE[bufsize];
+extern char AVDB[1024];
+
+
 extern char *VPRECONFIG;
 extern char *VRCFILE;
 
 extern char *VARCH;
+extern char *VARCH2;
 extern char VYEAR[];
 extern char VDAY[];
 extern char VMONTH[];
 extern char VHR[];
 extern char VMINUTE[];
 extern char VSEC[];
-extern char *ACTIONTEXT[]; 
+extern char *ACTIONTEXT[];
 extern char *ACTIONID[];
+extern char *BUILTINS[];
 extern char *CLASSTEXT[];
-extern char *CLASSATTRIBUTES[clssattr][attrdim];
+extern char *CLASSATTRIBUTES[CLSSATTR][ATTRDIM];
 extern char *FILEACTIONTEXT[];
 extern char *COMMATTRIBUTES[];
 extern char VINPUTFILE[];
 extern char *VCANONICALFILE;
 extern char VCURRENTFILE[];
 extern char VLOGFILE[];
-extern char CHDIR[];
+extern char *CHDIR;
 extern char *VSETUIDLOG;
 extern FILE *VLOGFP;
 extern char VEDITABORT[];
@@ -92,6 +102,7 @@ extern struct tm TM2;
 extern int ERRORCOUNT;
 extern int NUMBEROFEDITS;
 extern time_t CFSTARTTIME;
+extern time_t CFINITSTARTTIME;
 extern int CF_TIMEOUT;
 
 extern struct utsname VSYSNAME;
@@ -100,9 +111,10 @@ extern int LINENUMBER;
 extern mode_t DEFAULTMODE;
 extern mode_t DEFAULTSYSTEMMODE;
 extern int HAVEUID;
-extern char VUIDNAME[maxvarsize];
-extern char VGIDNAME[maxvarsize];
-extern char CFSERVER[maxvarsize];
+extern char *VUIDNAME;
+extern char *VGIDNAME;
+extern char CFSERVER[];
+extern char *PROTOCOL[];
 extern char VIPADDRESS[];
 extern char VPREFIX[];
 extern int VRECURSE;
@@ -112,10 +124,15 @@ extern char MOUNTMODE;
 extern char DELETEDIR;
 extern char DELETEFSTAB;
 extern char FORCE;
+extern char FORCEIPV4;
+extern char FORCELINK;
+extern char FORCEDIRS;
 extern char STEALTH;
 extern char PRESERVETIMES;
+extern char TRUSTKEY;
 extern char FORK;
 
+extern short COMPATIBILITY_MODE;
 extern short LINKSILENT;
 extern char  LINKTYPE;
 extern char  AGETYPE;
@@ -125,23 +142,39 @@ extern char  LINKDIRS;
 extern char  LOGP;
 extern char  INFORMP;
 
-extern char      FILTERNAME[bufsize];
-extern char     CURRENTITEM[bufsize];
-extern char     CURRENTPATH[bufsize];
-extern char       GROUPBUFF[bufsize];
-extern char      ACTIONBUFF[bufsize];
-extern char       CLASSBUFF[bufsize];
-extern char  ALLCLASSBUFFER[bufsize];
-extern char          CHROOT[bufsize];
+extern char *FILTERNAME;
+extern char *STRATEGYNAME;
+extern char *CURRENTITEM;
+extern char *CURRENTPATH;
+extern char *GROUPBUFF;
+extern char *ACTIONBUFF;
+extern char *CLASSBUFF;
+extern char ALLCLASSBUFFER[bufsize];
+extern char CHROOT[bufsize];
 extern char ELSECLASSBUFFER[bufsize];
-extern char        LINKFROM[bufsize];
-extern char          LINKTO[bufsize];
-extern char           ERROR[bufsize];
-extern char       MOUNTFROM[bufsize];
-extern char       MOUNTONTO[bufsize];
-extern char       MOUNTOPTS[bufsize];
-extern char     DESTINATION[bufsize];
-extern char     IMAGEACTION[bufsize];
+extern char *LINKFROM;
+extern char *LINKTO;
+extern char *ERROR;
+extern char *MOUNTFROM;
+extern char *MOUNTONTO;
+extern char *MOUNTOPTS;
+extern char *DESTINATION;
+extern char *IMAGEACTION;
+
+extern char *EXPR;
+extern char *CURRENTAUTHPATH;
+extern char *RESTART;
+extern char *FILTERDATA;
+extern char *STRATEGYDATA;
+
+extern short PROSIGNAL;
+extern char  PROACTION;
+extern char PROCOMP;
+extern char IMGCOMP;
+
+extern int IMGSIZE;
+
+
 extern char *CHECKSUMDB;
 extern char *COMPRESSCOMMAND;
 
@@ -160,7 +193,7 @@ extern struct Item *VTIMEZONE;
 extern char VDEFAULTROUTE[maxvarsize];
 extern char VNFSTYPE[maxvarsize];
 extern char *VREPOSITORY;
-extern char LOCALREPOS[bufsize];
+extern char *LOCALREPOS;
 extern char VIFNAME[16];
 extern char VIFNAMEOVERRIDE[16];
 extern enum classes VSYSTEMHARDCLASS;
@@ -178,6 +211,7 @@ extern struct Item *VINCLUDEPARSE;
 extern struct Item *VIGNOREPARSE;
 extern struct Item *VACLBUILD;
 extern struct Item *VFILTERBUILD;
+extern struct Item *VSTRATEGYBUILD;
 
 extern struct Item *VMOUNTLIST;
 extern struct Item *VHEAP;      /* Points to the base of the attribute heap */
@@ -207,8 +241,12 @@ extern struct Item *VIGNORE;
 extern struct Item *VHOMEPATLIST;
 extern struct Item *EXTENSIONLIST;
 extern struct Item *SUSPICIOUSLIST;
+extern struct Item *SCHEDULE;
+extern struct Item *SPOOLDIRLIST;
 extern struct Item *NONATTACKERLIST;
 extern struct Item *MULTICONNLIST;
+extern struct Item *TRUSTKEYLIST;
+extern struct Item *ALLOWUSERLIST;
 extern struct Item *SKIPVERIFY;
 extern struct Item *ATTACKERLIST;
 extern struct Item *MOUNTOPTLIST;
@@ -224,6 +262,9 @@ extern struct Edit *VEDITLIST;
 extern struct Edit *VEDITLISTTOP;
 extern struct Filter *VFILTERLIST;
 extern struct Filter *VFILTERLISTTOP;
+extern struct Strategy *VSTRATEGYLIST;
+extern struct Strategy *VSTRATEGYLISTTOP;
+
 extern struct CFACL  *VACLLIST;
 extern struct CFACL  *VACLLISTTOP;
 extern struct UnMount *VUNMOUNT;
@@ -260,12 +301,15 @@ extern short PARSING;
 extern short ISCFENGINE;
 
 extern short VERBOSE;
+extern short EXCLAIM;
 extern short INFORM;
+
 extern short LOGGING;
 extern short INFORM_save;
 extern short LOGGING_save;
 extern short CFPARANOID;
 extern short SHOWACTIONS;
+extern short LOGTIDYHOMEFILES;
 
 extern short TIDYDIRS;
 extern short TRAVLINKS;
@@ -282,6 +326,7 @@ extern short NOFILECHECK;
 extern short NOTIDY;
 extern short NOSCRIPTS;
 extern short PRSYSADM;
+extern short PRMAILSERVER;
 extern short MOUNTCHECK;
 extern short NOEDITS;
 extern short KILLOLDLINKS;
@@ -292,13 +337,11 @@ extern short NONALPHAFILES;
 extern short MINUSF;
 extern short NOLINKS;
 extern short ENFORCELINKS;
-extern short FORCELINK;
 extern short NOCOPY;
 extern short FORCENETCOPY;
 extern short SILENT;
 extern short EDITVERBOSE;
-extern short FORCECOPY;
-extern short IMAGEBACKUP;
+extern char IMAGEBACKUP;
 extern short ROTATE;
 extern int   TIDYSIZE;
 extern short USEENVIRON;
@@ -326,7 +369,9 @@ extern flag  MOUNT_FROM;
 extern flag  HAVE_RESTART;
 extern flag  ACTIONPENDING;
 extern flag  HOMECOPY;
-extern short  SECURE;
+extern char ENCRYPT;
+extern char VERIFY;
+extern char COMPATIBILITY;
 
 extern char *VPSCOMM[];
 extern char *VPSOPTS[];
