@@ -1,6 +1,6 @@
 /* 
 
-        Copyright (C) 1995
+        Copyright (C) 1995-
         Free Software Foundation, Inc.
 
    This file is part of GNU cfengine - written and maintained 
@@ -31,59 +31,18 @@
 /*                                                                 */
 /*******************************************************************/
 
-#define INET 1
-
 #include "cf.defs.h"
 #include "cf.extern.h"
 
-#ifdef HAVE_LIBCRYPTO
-# include <openssl/des.h>
-#endif
-
-/* Protocol specs:
-
-   Pre authorization: CAUTH (clear text auth) client-name server-name user-name
-
-   Stat file:  SYNCH long-time-string STAT filename
-	       reply with OK: <stat-reply-string>
-	       keep open channel
-
-   md5 file    Checksum compare
-
-   Get file:   GET filename
-
-	       reply with <multiple buffers>, break on zero length buffer
-	       or BAD: <message>
-	       keep open channel for more stat/get
-
-   Opendir:    OPENDIR dir-name
-
-	       reply with <multiple buffers>, break on zero length buffer
-
-   Exec:       EXEC option-string
-   	       reply with <multiple buffers>, break on zero length buffer
-               Close after this.
-
-Replies:
-
-  OK: message (succeeded)
-
-  BAD: message (failed)
-   
- */
-
 /*********************************************************************/
 /* Level 1                                                           */
 /*********************************************************************/
 
-int OpenServerConnection(serverlist,ip)
+int OpenServerConnection(ip)
 
-struct Item *serverlist;
 struct Image *ip;
 
-{ int err;
-  struct sockaddr_in cin;
-
+{
 if (strcmp(ip->server,"localhost") == 0)
    {
    AUTHENTICATED = true;
@@ -92,44 +51,41 @@ if (strcmp(ip->server,"localhost") == 0)
 
 AUTHENTICATED = false; 
  
-if (SD == cf_not_connected)
+if (CONN->sd == cf_not_connected)
    {
    Debug("Opening server connnection to %s\n",ip->server);
-   bzero(&cin,sizeof(cin));
 
-   cin.sin_port = PORTNUMBER;
-   cin.sin_addr.s_addr = (ip->dns)->s_addr;
-   cin.sin_family = AF_INET; 
-   
-   Debug("Trying to connect to %s = %s, port h=%d\n",ip->server,inet_ntoa(cin.sin_addr),PORTNUMBER);
-  
-   if ((SD = socket(AF_INET,SOCK_STREAM,0)) == -1)
+   if (!RemoteConnect(ip->server,ip->forceipv4))
       {
       CfLog(cferror,"Couldn't open a socket","socket");
+      if (CONN->sd != cf_not_connected)
+	 {
+	 CloseServerConnection();
+	 }
       AUTHENTICATED = false;
       return false;
       }
    
-   signal(SIGALRM,(void *)TimeOut);
-   alarm(CF_TIMEOUT);
-   
-   if (err=connect(SD,(void *) &cin,sizeof(cin)) == -1)
+   if (!IdentifyForVerification(CONN->sd,CONN->localip,CONN->family))
       {
-      sprintf(OUTPUT,"Couldn't connect to host %s\n",ip->server);
-      CfLog(cfinform,OUTPUT,"connect");
+      snprintf(OUTPUT,bufsize,"Id-authentication for %s failed\n",VFQNAME);
+      CfLog(cferror,OUTPUT,"");
+      errno = EPERM;
       CloseServerConnection();
       AUTHENTICATED = false;
       return false;
       }
-   
-   alarm(0);
-   signal(SIGALRM,SIG_DFL);
-   
-   Debug("Sending on socket...%d\n",SD);
 
-   if (! IdentifyForVerification(SD,VFQNAME))
+   if (ip->compat == 'y')
       {
-      CfLog(cferror,"Server registration procedure failed","");
+      CfLog(cfinform,"WARNING: the connection to %s is not offering key authentication\n","");
+      CfLog(cfinform,"WARNING: oldserver=true is a TEMPORARY measure only\n","");
+      }
+   
+   else if (!KeyAuthentication(ip))
+      {
+      snprintf(OUTPUT,bufsize,"Key-authentication for %s failed\n",VFQNAME);
+      CfLog(cferror,OUTPUT,"");
       errno = EPERM;
       CloseServerConnection();
       AUTHENTICATED = false;
@@ -141,7 +97,7 @@ if (SD == cf_not_connected)
    }
 else
    {
-   Debug("Server connection to %s already open on %d\n",ip->server,SD);
+   Debug("Server connection to %s already open on %d\n",ip->server,CONN->sd);
    }
 
 AUTHENTICATED = true; 
@@ -154,8 +110,16 @@ void CloseServerConnection()
 
 {
 Debug("Closing current connection\n");
-close(SD);
-SD = cf_not_connected;
+
+close(CONN->sd);
+
+CONN->sd = cf_not_connected;
+
+if (CONN->session_key != NULL)
+   {
+   free(CONN->session_key);
+   CONN->session_key = NULL;
+   } 
 }
 
 /*********************************************************************/
@@ -174,9 +138,8 @@ char *stattype;
   char recvbuffer[bufsize];
   char in[bufsize],out[bufsize];
   struct cfstat cfst;
-  int i,ret;
+  int ret,tosend,cipherlen;
   time_t tloc;
-  struct hostent *hp;
 
 Debug("cf_rstat(%s)\n",file);
 bzero(recvbuffer,bufsize); 
@@ -201,31 +164,34 @@ if ((tloc = time((time_t *)NULL)) == -1)
 
 sendbuffer[0] = '\0';
  
-if (ip->secure)
+if (ip->encrypt == 'y')
    {
-#ifdef HAVE_LIBCRYPTO
-   bzero(in,bufsize);
-   bzero(out,bufsize);
-   bzero(sendbuffer,bufsize);
-   sprintf(in,"SYNCH %d STAT %s",tloc,file);
-   cfencrypt(in,out,CFDES1,CFDES2,CFDES3,strlen(in));
-   sprintf(sendbuffer,"SSYNCH %d",strlen(in));
-   bcopy(out,sendbuffer+23,bufsize-24);
-#endif   
+   if (CONN->session_key == NULL)
+      {
+      CfLog(cferror,"Cannot do encrypted copy without keys (use cfkey)","");
+      return -1;
+      }
+   
+   snprintf(in,bufsize,"SYNCH %d STAT %s",tloc,file);
+   cipherlen = EncryptString(in,out,CONN->session_key,strlen(in)+1);
+   snprintf(sendbuffer,bufsize,"SSYNCH %d",cipherlen);
+   bcopy(out,sendbuffer+CF_PROTO_OFFSET,cipherlen);
+   tosend = cipherlen+CF_PROTO_OFFSET;
    }
 else
    {
-   sprintf(sendbuffer,"SYNCH %d STAT %s",tloc,file);
+   snprintf(sendbuffer,bufsize,"SYNCH %d STAT %s",tloc,file);
+   tosend = strlen(sendbuffer);
    }
 
-if (send(SD,sendbuffer,bufsize,0) == -1)
+if (SendTransaction(CONN->sd,sendbuffer,tosend,CF_DONE) == -1)
    {
-   sprintf(OUTPUT,"Transmission failed/refused talking to %.255s:%.255s in stat",ip->server,file);
+   snprintf(OUTPUT,bufsize*2,"Transmission failed/refused talking to %.255s:%.255s in stat",ip->server,file);
    CfLog(cfinform,OUTPUT,"send");
    return -1;
    }
 
-if (RecvSocketStream(SD,recvbuffer,bufsize,0) == -1)
+if (ReceiveTransaction(CONN->sd,recvbuffer,NULL) == -1)
    {
    return -1;
    }
@@ -237,15 +203,15 @@ if (strstr(recvbuffer,"unsynchronized"))
    return -1;
    }
 
-if (strncmp(recvbuffer,"BAD:",4) == 0)
+if (BadProtoReply(recvbuffer))
    {
-   sprintf(OUTPUT,"Server returned error: %s\n",recvbuffer+4);
+   snprintf(OUTPUT,bufsize*2,"Server returned error: %s\n",recvbuffer+4);
    CfLog(cfverbose,OUTPUT,"");
    errno = EPERM;
    return -1;
    }
 
-if (strncmp(recvbuffer,"OK:",3) == 0)
+if (OKProtoReply(recvbuffer))
    {
    long d1,d2,d3,d4,d5,d6,d7,d8,d9,d10,d11,d12=0;
    
@@ -276,7 +242,7 @@ if (strncmp(recvbuffer,"OK:",3) == 0)
 
    bzero(recvbuffer,bufsize);
    
-   if (RecvSocketStream(SD,recvbuffer,bufsize,0) == -1)
+   if (ReceiveTransaction(CONN->sd,recvbuffer,NULL) == -1)
       {
       return -1;
       }
@@ -306,6 +272,8 @@ if (strncmp(recvbuffer,"OK:",3) == 0)
 	             break;
       case cf_block: cfst.cf_mode |= (mode_t) S_IFBLK;
 	             break;
+      case cf_link:  cfst.cf_mode |= (mode_t) S_IFLNK;
+	             break;
       }
 
 
@@ -348,7 +316,7 @@ if (strncmp(recvbuffer,"OK:",3) == 0)
    }
 
 
-sprintf(OUTPUT,"Transmission refused or failed statting %s\nGot: %s\n",file,recvbuffer); 
+snprintf(OUTPUT,bufsize*2,"Transmission refused or failed statting %s\nGot: %s\n",file,recvbuffer); 
 CfLog(cferror,OUTPUT,"");
 errno = EPERM;
 
@@ -364,7 +332,7 @@ struct Image *ip;
 
 { char sendbuffer[bufsize];
   char recvbuffer[bufsize];
-  int i,offset,n, done=false;
+  int n, done=false;
   CFDIR *cfdirh;
   char *sp;
 
@@ -386,16 +354,16 @@ cfdirh->cf_list = NULL;
 cfdirh->cf_listpos = NULL;
 cfdirh->cf_dirh = NULL;
 
-sprintf(sendbuffer,"OPENDIR %s",dirname);
+snprintf(sendbuffer,bufsize,"OPENDIR %s",dirname);
 
-if (send(SD,sendbuffer,bufsize,0) == -1)
+if (SendTransaction(CONN->sd,sendbuffer,0,CF_DONE) == -1)
    {
    return NULL;
    }
 
 while (!done)
    {
-   if ((n = RecvSocketStream(SD,recvbuffer,bufsize,0)) == -1)
+   if ((n = ReceiveTransaction(CONN->sd,recvbuffer,NULL)) == -1)
       {
       if (errno == EINTR) 
          {
@@ -409,16 +377,16 @@ while (!done)
       break;
       }
 
-   if (strncmp(recvbuffer,CFFAILEDSTR,strlen(CFFAILEDSTR)) == 0)
+   if (FailedProtoReply(recvbuffer))
       {
-      sprintf(OUTPUT,"Network access to %s:%s denied\n",ip->server,dirname);
+      snprintf(OUTPUT,bufsize*2,"Network access to %s:%s denied\n",ip->server,dirname);
       CfLog(cfinform,OUTPUT,"");
       return false;      
       }
 
-   if (strncmp(recvbuffer,"BAD:",4) == 0)
+   if (BadProtoReply(recvbuffer))
       {
-      sprintf(OUTPUT,"%s\n",recvbuffer+4);
+      snprintf(OUTPUT,bufsize*2,"%s\n",recvbuffer+4);
       CfLog(cfinform,OUTPUT,"");
       return false;      
       }
@@ -465,50 +433,52 @@ int CompareMD5Net(file1,file2,ip)
 char *file1, *file2;
 struct Image *ip;
 
-{ static unsigned char d[16];
-  char sendbuffer[bufsize];
-  char recvbuffer[bufsize],in[bufsize],out[bufsize];
-  int i,offset;
+{ static unsigned char d[CF_MD5_LEN];
+  char *sp,sendbuffer[bufsize],recvbuffer[bufsize],in[bufsize],out[bufsize];
+  int i,tosend,cipherlen;
 
-Debug("Send digest of %s to server\n",file2);
-cfMDFile(file2,d);   /* send this to the server for comparison */
+
+ChecksumFile(file2,d,'m');   /* send md5 to the server for comparison */
+Debug("Send digest of %s to server, %s\n",file2,ChecksumPrint('m',d));
 
 bzero(recvbuffer,bufsize);
-bzero(sendbuffer,bufsize);
 
-if (20 + 3*16 + strlen(file1) > bufsize-1)
+if (ip->encrypt == 'y')
    {
-   CfLog(cferror,"MD5 transfer overflow...updating for safety\n","");
-   return true;
-   }
+   snprintf(in,bufsize,"MD5 %s",file1);
 
+   sp = in + strlen(in) + CF_SMALL_OFFSET;
 
-if (ip->secure)
-   {
-#ifdef HAVE_LIBCRYPTO
-   bzero(in,bufsize);
-   bzero(out,bufsize);
-   bzero(sendbuffer,bufsize);
-   sprintf(in,"MD5 %s\n %d %d %d %d %d %d %d %d %d %d %d %d %d %d %d %d \0",file1,
-	   d[0],d[1],d[2],d[3],d[4],d[5],d[6],d[7],d[8],d[9],d[10],d[11],d[12],d[13],d[14],d[15]);
-   cfencrypt(in,out,CFDES1,CFDES2,CFDES3,strlen(in));
-   sprintf(sendbuffer,"SMD5 %d",strlen(in));
-   bcopy(out,sendbuffer+23,bufsize-24);
-#endif   
+   for (i = 0; i < CF_MD5_LEN; i++)
+      {
+      *sp++ = d[i];
+      }
+   
+   cipherlen = EncryptString(in,out,CONN->session_key,strlen(in)+CF_SMALL_OFFSET+CF_MD5_LEN);
+   snprintf(sendbuffer,bufsize,"SMD5 %d",cipherlen);
+   bcopy(out,sendbuffer+CF_PROTO_OFFSET,cipherlen);
+   tosend = cipherlen + CF_PROTO_OFFSET;
    }
 else
    {
-   sprintf(sendbuffer,"MD5 %s\n %d %d %d %d %d %d %d %d %d %d %d %d %d %d %d %d \0",file1,
-	   d[0],d[1],d[2],d[3],d[4],d[5],d[6],d[7],d[8],d[9],d[10],d[11],d[12],d[13],d[14],d[15]);
+   snprintf(sendbuffer,bufsize,"MD5 %s",file1);
+   sp = sendbuffer + strlen(sendbuffer) + CF_SMALL_OFFSET;
+
+   for (i = 0; i < CF_MD5_LEN; i++)
+      {
+      *sp++ = d[i];
+      }
+   
+   tosend = strlen(sendbuffer)+CF_SMALL_OFFSET+CF_MD5_LEN;
    } 
  
-if (send(SD,sendbuffer,bufsize,0) == -1)
+if (SendTransaction(CONN->sd,sendbuffer,tosend,CF_DONE) == -1)
    {
    CfLog(cferror,"","send");
    return false;
    }
 
-if (RecvSocketStream(SD,recvbuffer,bufsize,0) == -1)
+if (ReceiveTransaction(CONN->sd,recvbuffer,NULL) == -1)
    {
    Verbose("No answer from host, assuming checksum ok to avoid remote copy for now...\n");
    return false;
@@ -516,12 +486,12 @@ if (RecvSocketStream(SD,recvbuffer,bufsize,0) == -1)
 
 if (strcmp(CFD_TRUE,recvbuffer) == 0)
    {
-   Debug("MD5 mismatch: %s\n",recvbuffer);
+   Debug("MD5 mismatch: (reply - %s)\n",recvbuffer);
    return true; /* mismatch */
    }
 else
    {
-   Debug("MD5 match: %s\n",recvbuffer);
+   Debug("MD5 matched ok: (reply - %s)\n",recvbuffer);
    return false;
    }
  
@@ -536,14 +506,14 @@ char *source, *new;
 struct Image *ip;
 off_t size;
 
-{ int dd, buf_size, i=0;
-  char *buf,*cp,*sp,in[bufsize],out[bufsize],sendbuffer[bufsize];
-  unsigned char key[8];
-  int n_read, *intp,err,toget,towrite,len;
-  long n_read_total = 0;
-  int last_write_made_hole = 0, done = false;
-
-Debug("CopyRegNet(%s,%s,%d) to socket %d\n",source,new,size,SD);
+{ int dd, buf_size,n_read = 0,toget,towrite,plainlen,more = true;
+  int last_write_made_hole = 0, done = false,tosend,cipherlen=0;
+  char *buf,in[bufsize],out[bufsize],sendbuffer[bufsize];
+  unsigned char iv[] = {1,2,3,4,5,6,7,8};
+  long n_read_total = 0;  
+  EVP_CIPHER_CTX ctx;
+  
+EVP_CIPHER_CTX_init(&ctx);  
 
 if ((strlen(new) > bufsize-20))
    {
@@ -555,42 +525,44 @@ unlink(new);  /* To avoid link attacks */
   
 if ((dd = open(new,O_WRONLY|O_CREAT|O_TRUNC|O_EXCL|O_BINARY, 0600)) == -1)
    {
-   sprintf(OUTPUT,"Copy %s:%s security - failed attempt to exploit a race? (Not copied)\n",ip->server,new);
+   snprintf(OUTPUT,bufsize*2,"Copy %s:%s security - failed attempt to exploit a race? (Not copied)\n",ip->server,new);
    CfLog(cferror,OUTPUT,"open");
    unlink(new);
    return false;
    }
 
 sendbuffer[0] = '\0';
- 
-if (ip->secure)
+
+buf_size = ST_BLKSIZE(dstat);
+
+if (buf_size < 2048)
    {
-#ifdef HAVE_LIBCRYPTO
-   des_random_key(&key);
-   bzero(in,bufsize);
-   bzero(out,bufsize);
-   bzero(sendbuffer,bufsize);
-   len = cfprintf(in,8,"GET",key,source);
-   cfencrypt(in,out,CFDES1,CFDES2,CFDES3,len);
-   sprintf(sendbuffer,"SGET %4d %4d",len,ST_BLKSIZE(dstat));
-   bcopy(out,sendbuffer+31,bufsize-32);
-#endif
+   buf_size = 2048;
+   } 
+ 
+if (ip->encrypt == 'y')
+   {
+   snprintf(in,bufsize-CF_PROTO_OFFSET,"GET dummykey %s",source);
+   cipherlen = EncryptString(in,out,CONN->session_key,strlen(in));
+   snprintf(sendbuffer,bufsize,"SGET %4d %4d",cipherlen,buf_size);
+   bcopy(out,sendbuffer+CF_PROTO_OFFSET,cipherlen);
+   tosend=cipherlen+CF_PROTO_OFFSET;   
+   EVP_DecryptInit(&ctx,EVP_bf_cbc(),CONN->session_key,iv);
    }
 else
    {
-   sprintf(sendbuffer,"GET %d %s",ST_BLKSIZE(dstat),source);
+   snprintf(sendbuffer,bufsize,"GET %d %s",buf_size,source);
+   tosend=strlen(sendbuffer);
    }
 
-if (send(SD,sendbuffer,bufsize,0) == -1)
+if (SendTransaction(CONN->sd,sendbuffer,tosend,CF_DONE) == -1)
    {
    CfLog(cferror,"Couldn't send","send");
    close(dd);
    return false;
    }
 
-buf_size = ST_BLKSIZE(dstat);
-buf = (char *) malloc(bufsize + sizeof(int)); /* Note bufsize not buf_size */
-
+buf = (char *) malloc(bufsize + sizeof(int)); /* Note bufsize not buf_size !! */
 n_read_total = 0;
 
 while (!done)
@@ -602,53 +574,70 @@ while (!done)
    else if (size != 0)
       {
       towrite = (size - n_read_total);
-      toget = buf_size; /* padding required for encryption chaining */
+      
+      if (ip->compat == 'y')
+	 {
+	 toget = buf_size;
+	 }
+      else
+	 {
+	 toget = towrite;
+	 }
       }
    else
       {
       toget = towrite = 0;
       }
 
-   if ((n_read = RecvSocketStream(SD,buf,toget,0)) == -1)
+   if (ip->encrypt == 'y')
       {
-      if (errno == EINTR) 
-         {
-         continue;
-         }
-
-      CfLog(cferror,"Error in socket stream","recv");
-      close(dd);
-      free(buf);
-      return false;
+      if (more)
+	 {
+	 cipherlen = ReceiveTransaction(CONN->sd,buf,&more);
+	 }
+      else
+	 {
+	 break;	 /* Already written last encrypted buffer */
+	 }
       }
-
-   if (strncmp(buf,CFFAILEDSTR,strlen(CFFAILEDSTR)) == 0)
+   else
       {
-      sprintf(OUTPUT,"Network access to %s:%s denied\n",ip->server,source);
-      RecvSocketStream(SD,buf,bufsize-toget,0); /* flush rest of transaction */
+      if ((n_read = RecvSocketStream(CONN->sd,buf,toget,0)) == -1)
+	 {
+	 if (errno == EINTR) 
+	    {
+	    continue;
+	    }
+	 
+	 CfLog(cferror,"Error in socket stream","recv");
+	 close(dd);
+	 free(buf);
+	 return false;
+	 }
+      }
+   
+   /* If the first thing we get is an error message, break. */
+   
+   if (n_read_total == 0 && strncmp(buf,CFFAILEDSTR,strlen(CFFAILEDSTR)) == 0)
+      {
+      snprintf(OUTPUT,bufsize*2,"Network access to %s:%s denied\n",ip->server,source);
+      RecvSocketStream(CONN->sd,buf,buf_size-n_read,0); /* flush rest of transaction */
       CfLog(cfinform,OUTPUT,"");
       close(dd);
       free(buf);
       return false;      
       }
 
-   if (strncmp(buf,"BAD:File",8) == 0) /* error */
+   if (ip->encrypt == 'y')
       {
-      if (strncmp(buf,"BAD:File %s",11) != 0) /* Code copies itself! */
+      if (!EVP_DecryptUpdate(&ctx,sendbuffer,&plainlen,buf,cipherlen))
 	 {
-	 CfLog(cfinform,buf+4,"");
-	 close(dd);
-	 free(buf);
-	 CfLog(cferror,buf,"");
+	 Debug("Decryption failed\n");
 	 return false;
 	 }
-      }
-
-   if (ip->secure)
-      {
-      bzero(sendbuffer,bufsize);
-      cfdecrypt(buf,sendbuffer,CFDES1,key,CFDES3,buf_size);
-      bcopy(sendbuffer,buf,buf_size);
+      
+      bcopy(sendbuffer,buf,plainlen);
+      n_read = towrite = plainlen;
       }
 
    if (n_read == 0)
@@ -658,96 +647,71 @@ while (!done)
 
    n_read_total += towrite; /*n_read;*/
 
-   if (n_read_total >= (long)size)  /* Handle EOF without closing socket */
-      {                                 /* signature is 8 bytes */
-      done = true;
+   if (ip->encrypt == 'n')
+      {
+      if (n_read_total >= (long)size)  /* Handle EOF without closing socket */
+	 {
+	 done = true;
+	 }
       }
 
-   intp = 0;
-
-   if (ip->makeholes)
+   if (!EmbeddedWrite(new,dd,buf,ip,towrite,&last_write_made_hole,n_read))
       {
-      buf[n_read] = 1;	                   /* Sentinel to stop loop.  */
-
-      /* Find first non-zero *word*, or the word with the sentinel.  */
-
-      intp = (int *) buf;
-
-      while (*intp++ == 0)
-         {
-         }
-
-      /* Find the first non-zero *byte*, or the sentinel.  */
-
-      cp = (char *) (intp - 1);
-
-      while (*cp++ == 0)
-         {
-         }
-
-      /* If we found the sentinel, the whole input block was zero,
-         and we can make a hole.  */
-
-      if (cp > buf + n_read)
-         {
-         /* Make a hole.  */
-         if (lseek (dd,(off_t)n_read,SEEK_CUR) < 0L)
-            {
-            sprintf (OUTPUT,"lseek in CopyReg, dest=%s\n", new);
-            CfLog(cferror,OUTPUT,"lseek");
-            free(buf);
-	    unlink(new);
-	    close(dd);
-	    FlushToEnd(SD,size - n_read_total);
-            return false;
-            }
-         last_write_made_hole = 1;
-         }
-      else
-         {
-         /* Clear to indicate that a normal write is needed. */
-         intp = 0;
-         }
-      }
-
-   if (intp == 0)
-      {
-      if (cf_full_write (dd,buf,towrite) < 0)
-         {
-	 sprintf(OUTPUT,"CopyNetReg(%.256s:%.256s,%.256s) failed\n",ip->server,source,new);
-         CfLog(cferror,OUTPUT,"write");
-	 FlushToEnd(SD,size - n_read_total);
-         close(dd);
-         free(buf);
-	 unlink(new);
-         return false;
-         }
-      last_write_made_hole = 0;
+      snprintf(OUTPUT,bufsize,"Local disk write failed copying %s:%s to %s\n",ip->server,source,new);
+      CfLog(cferror,OUTPUT,"");
+      free(buf);
+      unlink(new);
+      close(dd);
+      FlushToEnd(CONN->sd,size - n_read_total);
+      EVP_CIPHER_CTX_cleanup(&ctx);
+      return false;
       }
    }
 
+if (ip->encrypt == 'y') /* final crypto cleanup */
+   {
+   if (!EVP_DecryptFinal(&ctx,buf,&plainlen))
+      {
+      Debug("Final decrypt failed\n");
+      return false;
+      }
+
+   if (!EmbeddedWrite(new,dd,buf,ip,plainlen,&last_write_made_hole,n_read))
+      {
+      snprintf(OUTPUT,bufsize,"Local disk write failed copying %s:%s to %s\n",ip->server,source,new);
+      CfLog(cferror,OUTPUT,"");
+      free(buf);
+      unlink(new);
+      close(dd);
+      FlushToEnd(CONN->sd,size - n_read_total);
+      EVP_CIPHER_CTX_cleanup(&ctx);
+      return false;
+      }
+   }
+ 
   /* If the file ends with a `hole', something needs to be written at
      the end.  Otherwise the kernel would truncate the file at the end
-     of the last write operation.  */
+     of the last write operation. Write a null character and truncate
+     it again.  */
 
-  if (last_write_made_hole)
-    {
-    /* Write a null character and truncate it again.  */
-
-    if (cf_full_write (dd,"",1) < 0 || ftruncate (dd,n_read_total) < 0)
-       {
-       CfLog(cferror,"cfengine: full_write or ftruncate error in CopyReg\n","");
-       free(buf);
-       unlink(new);
-       close(dd);
-       FlushToEnd(SD,size - n_read_total);
-       return false;
-       }
-    }
-
-Debug("End of file\n");
+if (last_write_made_hole)   
+   {
+   if (cf_full_write (dd,"",1) < 0 || ftruncate (dd,n_read_total) < 0)
+      {
+      CfLog(cferror,"cfengine: full_write or ftruncate error in CopyReg\n","");
+      free(buf);
+      unlink(new);
+      close(dd);
+      FlushToEnd(CONN->sd,size - n_read_total);
+      EVP_CIPHER_CTX_cleanup(&ctx);
+      return false;
+      }
+   }
+ 
+Debug("End of CopyNetReg\n");
 close(dd);
 free(buf);
+EVP_CIPHER_CTX_cleanup(&ctx);
 return true;
 }
 
@@ -834,7 +798,7 @@ int sd,toget;
 { int i;
   char buffer[2]; 
 
-sprintf(OUTPUT,"Flushing rest of file...%d bytes\n",toget);
+snprintf(OUTPUT,bufsize*2,"Flushing rest of file...%d bytes\n",toget);
 CfLog(cfinform,OUTPUT,""); 
  
 for (i = 0; i < toget; i++)
@@ -843,36 +807,43 @@ for (i = 0; i < toget; i++)
    }
 }
 
-
-/*********************************************************************/
-/* ERS                                                               */
 /*********************************************************************/
 
-int cfprintf(out,len2,in1,in2,in3)
+struct cfagent_connection *NewAgentConn()
 
-char *out,*in1,*in2,*in3;
-int len2;
+{ struct cfagent_connection *ap;
+
+if ((ap = (struct cfagent_connection *)malloc(sizeof(struct cfagent_connection))) == NULL)
+   {
+   return NULL;
+   }
+
+Debug("New server connection...\n");
+ap->sd = cf_not_connected;
+ap->family = AF_INET; 
+ap->trust = false;
+ap->localip[0] = '\0';
+ap->remoteip[0] = '\0';
+ap->session_key = NULL;
+ap->error = false; 
+return ap;
+};
+
+/*********************************************************************/
+
+void DeleteAgentConn(ap)
+
+struct cfagent_connection *ap;
 
 {
-int len1, len3;
-char *sp;
+if (ap->session_key != NULL)
+   {
+   free(ap->session_key);
+   }
+
+free(ap);
+ap = NULL; 
+}
+
 
-len1=strlen(in1);
-len3=strlen(in3);
-   
-sp = out;
-bcopy(in1,sp,len1);
-sp += len1;
-   
-*sp++ = ' ';
-bcopy(in2,sp,len2);
-sp += len2;
-   
-*sp++ = ' ';
-bcopy(in3,sp,len3);
-sp += len3;
-*sp = '\0';
-   
-return (len1 + len2 + len3 + 2);
-}  
 
