@@ -33,38 +33,33 @@
 #include "cf.defs.h"
 #include "cf.extern.h"
 
+char EDITBUFF[bufsize];
+
 /********************************************************************/
 /* EDIT Data structure routines                                     */
 /********************************************************************/
 
-int DoRecursiveEditFiles(name,level,ptr,sb)
+void DoRecursiveEditFiles(name,level,ptr)
 
 char *name;
 int level;
 struct Edit *ptr;
-struct stat *sb;
 
 { DIR *dirh;
   struct dirent *dirp;
   char pcwd[bufsize];
   struct stat statbuf;
-  int goback;
 
 if (level == -1)
    {
-   return false;
+   return;
    }
 
-Debug("RecursiveEditFiles(%s)\n",name);
+printf("RecursiveEditFiles(%s)\n",name);
 
-if (!DirPush(name,sb))
+if ((dirh = opendir(name)) == NULL)
    {
-   return false;
-   }
- 
-if ((dirh = opendir(".")) == NULL)
-   {
-   return true;
+   return;
    }
 
 for (dirp = readdir(dirh); dirp != NULL; dirp = readdir(dirh))
@@ -81,33 +76,34 @@ for (dirp = readdir(dirh); dirp != NULL; dirp = readdir(dirh))
 
    strcpy(pcwd,name);                                   /* Assemble pathname */
    AddSlash(pcwd);
-   
-   if (!FileObjectFilter(pcwd,&statbuf,ptr->filters,editfiles))
-      {
-      continue;
-      }
-      
+
    if (BufferOverflow(pcwd,dirp->d_name))
       {
-      return true;
+      return;
       }
 
    strcat(pcwd,dirp->d_name);
 
+   if (!FileObjectFilter(pcwd,&statbuf,ptr->filters,editfiles))
+      {
+      Verbose("Skipping filtered file %s\n",pcwd);
+      continue;
+      }
+      
    if (TRAVLINKS)
       {
-      if (stat(dirp->d_name,&statbuf) == -1)
+      if (stat(pcwd,&statbuf) == -1)
          {
-         sprintf(OUTPUT,"RecursiveCheck was working on %s when this happened:\n",pcwd);
+         snprintf(OUTPUT,bufsize*2,"RecursiveCheck was working on %s when this happened:\n",pcwd);
 	 CfLog(cferror,OUTPUT,"stat");
          continue;
          }
       }
    else
       {
-      if (lstat(dirp->d_name,&statbuf) == -1)
+      if (lstat(pcwd,&statbuf) == -1)
          {
-         sprintf(OUTPUT,"RecursiveCheck was working in %s when this happened:\n",pcwd);
+         snprintf(OUTPUT,bufsize*2,"RecursiveCheck was working in %s when this happened:\n",pcwd);
          CfLog(cferror,OUTPUT,"lstat");
          continue;
          }
@@ -123,9 +119,7 @@ for (dirp = readdir(dirh); dirp != NULL; dirp = readdir(dirh))
          {
          if ((ptr->recurse > 1) || (ptr->recurse == INFINITERECURSE))
             {
-            goback = DoRecursiveEditFiles(pcwd,level-1,ptr,&statbuf);
-
-	    DirPop(goback,name,sb);
+            DoRecursiveEditFiles(pcwd,level-1,ptr);
             }
          else
             {
@@ -140,7 +134,6 @@ for (dirp = readdir(dirh); dirp != NULL; dirp = readdir(dirh))
    }
 
 closedir(dirh);
-return true; 
 }
 
 /********************************************************************/
@@ -172,7 +165,7 @@ for (ip = VMOUNTLIST; ip != NULL; ip=ip->next)
    
    if ((dirh = opendir(ip->name)) == NULL)
       {
-      sprintf(OUTPUT,"Can't open directory %s\n",ip->name);
+      snprintf(OUTPUT,bufsize*2,"Can't open directory %s\n",ip->name);
       CfLog(cferror,OUTPUT,"opendir");
       return;
       }
@@ -195,7 +188,7 @@ for (ip = VMOUNTLIST; ip != NULL; ip=ip->next)
 
       if ((dirh2 = opendir(homedir)) == NULL)
          {
-         sprintf(OUTPUT,"Can't open directory%s\n",homedir);
+         snprintf(OUTPUT,bufsize*2,"Can't open directory%s\n",homedir);
          CfLog(cferror,OUTPUT,"opendir");
          return;
          }
@@ -216,7 +209,7 @@ for (ip = VMOUNTLIST; ip != NULL; ip=ip->next)
 
          if (stat(dest,&statbuf))
             {
-            Verbose("%s: file %s doesn't exist for editing, skipping\n",VPREFIX,dest);
+            EditVerbose("File %s doesn't exist for editing, skipping\n",dest);
             continue;
             }
       
@@ -259,14 +252,14 @@ if (lstat(filename,&statbuf) != -1)
    {
    if (S_ISLNK(statbuf.st_mode))
       {
-      Verbose("%s: File %s is a link, editing real file instead\n",VPREFIX,filename);
+      EditVerbose("File %s is a link, editing real file instead\n",filename);
       
       bzero(linkname,bufsize);
       bzero(realname,bufsize);
       
       if (readlink(filename,linkname,bufsize-1) == -1)
 	 {
-	 sprintf(OUTPUT,"Cannot read link %s\n",filename);
+	 snprintf(OUTPUT,bufsize*2,"Cannot read link %s\n",filename);
 	 CfLog(cferror,OUTPUT,"readlink");
 	 return;
 	 }
@@ -280,7 +273,7 @@ if (lstat(filename,&statbuf) != -1)
       
       if (BufferOverflow(realname,linkname))
 	 {
-	 sprintf(OUTPUT,"(culprit %s in editfiles)\n",filename);
+	 snprintf(OUTPUT,bufsize*2,"(culprit %s in editfiles)\n",filename);
 	 CfLog(cferror,OUTPUT,"");
 	 return;
 	 }
@@ -290,7 +283,7 @@ if (lstat(filename,&statbuf) != -1)
 	 if (statbuf2.st_uid != statbuf.st_uid)
 	    {
 	    /* Link to /etc/passwd? ouch! */
-	    sprintf(OUTPUT,"Forbidden to edit a link to another user's file with privilege (%s)",filename);
+	    snprintf(OUTPUT,bufsize*2,"Forbidden to edit a link to another user's file with privilege (%s)",filename);
 	    CfLog(cfinform,OUTPUT,"");
 	    return;
 	    }
@@ -300,26 +293,28 @@ if (lstat(filename,&statbuf) != -1)
 
       if (!FileObjectFilter(realname,&statbuf2,ptr->filters,editfiles))
 	 {
-	 Debug("Skipping filtered editfile %s\n",files);
+	 Debug("Skipping filtered editfile %s\n",filename);
 	 return;
 	 }
       DoEditFile(ptr,realname);
+      return;
       }
    else
       {
       if (!FileObjectFilter(filename,&statbuf,ptr->filters,editfiles))
 	 {
-	 Debug("Skipping filtered editfile %s\n",files);
+	 Debug("Skipping filtered editfile %s\n",filename);
 	 return;
 	 }
       DoEditFile(ptr,filename);
+      return;
       }
    }
 else
    {
    if (!FileObjectFilter(filename,&statbuf,ptr->filters,editfiles))
       {
-      Debug("Skipping filtered editfile %s\n",files);
+      Debug("Skipping filtered editfile %s\n",filename);
       return;
       }
    DoEditFile(ptr,filename);
@@ -339,19 +334,18 @@ char *filename;
 
 { struct Edlist *ep, *loopstart, *loopend, *ThrowAbort();
   struct Item *filestart = NULL, *newlineptr;
-  char *currenteditscript, searchstr[bufsize], expdata[bufsize];
+  char currenteditscript[bufsize], searchstr[bufsize], expdata[bufsize];
   struct stat tmpstat;
   char spliton = ':';
   mode_t maskval;
   int todo = 0;
-  int foreachloop = false;
-  FILE *loop_fp;
+  FILE *loop_fp = NULL;
 
 Debug("DoEditFile(%s)\n",filename);
 filestart = NULL;
-currenteditscript = NULL;
+currenteditscript[0] = '\0';
 searchstr[0] = '\0';
-bzero(ACTIONBUFF,bufsize);
+bzero(EDITBUFF,bufsize);
 AUTOCREATED = false;
 IMAGEBACKUP = true;
 
@@ -360,7 +354,7 @@ if (IgnoredOrExcluded(editfiles,filename,ptr->inclusions,ptr->exclusions))
    Debug("Skipping excluded file %s\n",filename);
    return;
    }
- 
+
 for (ep = ptr->actions; ep != NULL; ep=ep->next)
    {
    if (!IsExcluded(ep->classes))
@@ -374,12 +368,12 @@ if (todo == 0)   /* Because classes are stored per edit, not per file */
    return;
    }
 
-if (!GetLock(ASUniqueName("editfile"),CanonifyName(ptr->fname),VIFELAPSED,VEXPIREAFTER,VUQNAME,CFSTARTTIME))
+if (!GetLock(ASUniqueName("editfile"),CanonifyName(filename),VIFELAPSED,VEXPIREAFTER,VUQNAME,CFSTARTTIME))
    {
    return;
    }
-
-if (ptr->binary == 't')
+ 
+if (ptr->binary == 'y')
    {
    BinaryEditFile(ptr,filename);
    ReleaseCurrentLock();
@@ -409,7 +403,7 @@ SEARCHREPLACELEVEL = 0;
 FOREACHLEVEL = 0;
 loopstart = NULL;
 
-Verbose("%s: Begin editing %s\n",VPREFIX,filename);
+Verbose("Begin editing %s\n",filename);
 ep = ptr->actions;
 
 while (ep != NULL)
@@ -419,7 +413,7 @@ while (ep != NULL)
       ep = ep->next;
       continue;
       }
-
+   
    ExpandVarstring(ep->data,expdata,NULL);
    
    Debug2("Edit action: %s\n",VEDITNAMES[ep->code]);
@@ -428,17 +422,25 @@ while (ep != NULL)
       {
       case NoEdit:
       case EditInform:
+      case EditBackup:
       case EditUmask:
       case AutoCreate:
                break;
 
+      case EditUseShell:
+	       if (strcmp(expdata,"false") == 0)
+		  {
+		  ptr->useshell = 'n';
+		  }
+	       break;
+
       case CatchAbort:
-	          EditVerbose("%s: Caught Exception\n",VPREFIX);
+	          EditVerbose("Caught Exception\n");
 	          break;
 
       case SplitOn:
 	          spliton = *(expdata);
-		  EditVerbose("%s: Split lines by %c\n",VPREFIX,spliton);
+		  EditVerbose("Split lines by %c\n",spliton);
 		  break;
 
       case DeleteLinesStarting:
@@ -485,22 +487,22 @@ while (ep != NULL)
                break;
 
       case SetLine:
-               strcpy(ACTIONBUFF,expdata);
-               EditVerbose("%s: Set current line to %s\n",VPREFIX,ACTIONBUFF);
+               strcpy(EDITBUFF,expdata);
+               EditVerbose("Set current line to %s\n",EDITBUFF);
                break;
 
       case AppendIfNoLineMatching:
 
-               if (strcmp(ACTIONBUFF,"") == 0)
+               if (strcmp(EDITBUFF,"") == 0)
                   {
-                  sprintf(OUTPUT,"%s: SetLine not set when calling AppendIfNoLineMatching %s\n",VPREFIX,expdata);
+                  snprintf(OUTPUT,bufsize*2,"SetLine not set when calling AppendIfNoLineMatching %s\n",expdata);
 		  CfLog(cferror,OUTPUT,"");
                   continue;
                   }
 	       
                if (LocateNextItemMatching(filestart,expdata) == NULL)
 	          {
-                  AppendItem(&filestart,ACTIONBUFF,NULL);
+                  AppendItem(&filestart,EDITBUFF,NULL);
                   }
                break;
 
@@ -517,64 +519,65 @@ while (ep != NULL)
 
       case PrependIfNoLineMatching:
 
-               if (strcmp(ACTIONBUFF,"") == 0)
+               if (strcmp(EDITBUFF,"") == 0)
                   {
-                  printf("%s: SetLine not set when calling PrependIfNoLineMatching %s\n",VPREFIX,expdata);
+                  snprintf(OUTPUT,bufsize,"SetLine not set when calling PrependIfNoLineMatching %s\n",expdata);
+		  CfLog(cferror,OUTPUT,"");
                   continue;
                   }
 
                if (LocateNextItemMatching(filestart,expdata) == NULL)
                   {
-                  PrependItem(&filestart,ACTIONBUFF,NULL);
+                  PrependItem(&filestart,EDITBUFF,NULL);
                   }
                break;
 
       case WarnIfNoSuchLine:
                if (LocateNextItemMatching(filestart,expdata) == NULL)
                   {
-                  printf("%s: Warning, file %s has no line matching %s\n",VPREFIX,filename,expdata);
+                  printf("Warning, file %s has no line matching %s\n",filename,expdata);
                   }
                break;
 
       case WarnIfLineMatching:
                if (LocateNextItemMatching(filestart,expdata) != NULL)
                   {
-                  printf("%s: Warning, file %s has a line matching %s\n",VPREFIX,filename,expdata);
+                  printf("Warning, file %s has a line matching %s\n",filename,expdata);
                   }
                break;
 
       case WarnIfNoLineMatching:
                if (LocateNextItemMatching(filestart,expdata) == NULL)
                   {
-                  printf("%s: Warning, file %s has a no line matching %s\n",VPREFIX,filename,expdata);
+                  printf("Warning, file %s has a no line matching %s\n",filename,expdata);
                   }
                break;
 
       case WarnIfLineStarting:
                if (LocateNextItemStarting(filestart,expdata) != NULL)
                   {
-                  printf("%s: Warning, file %s has a line starting %s\n",VPREFIX,filename,expdata);
+                  printf("Warning, file %s has a line starting %s\n",filename,expdata);
                   }
                break;
 
       case WarnIfNoLineStarting:
                if (LocateNextItemStarting(filestart,expdata) == NULL)
                   {
-                  printf("%s: Warning, file %s has no line starting %s\n",VPREFIX,filename,expdata);
+                  printf("Warning, file %s has no line starting %s\n",filename,expdata);
                   }
                break;
 
       case WarnIfLineContaining:
                if (LocateNextItemContaining(filestart,expdata) != NULL)
                   {
-                  printf("%s: Warning, file %s has a line containing %s\n",VPREFIX,filename,expdata);
+                  printf("Warning, file %s has a line containing %s\n",filename,expdata);
                   }
                break;
 
       case WarnIfNoLineContaining:
                if (LocateNextItemContaining(filestart,expdata) == NULL)
                   {
-                  printf("%s: Warning, file %s has no line containing %s\n",VPREFIX,filename,expdata);
+                  printf("Warning, file %s has no line containing %s\n",filename,expdata);
                   }
                break;
 
@@ -663,7 +666,7 @@ while (ep != NULL)
       case ResetSearch:
                if (!ResetEditSearch(expdata,filestart))
                   {
-                  printf("%s: ResetSearch Failed in %s, aborting editing\n",VPREFIX,filename);
+                  printf("ResetSearch Failed in %s, aborting editing\n",filename);
                   goto abort;
                   }
                break;
@@ -673,7 +676,7 @@ while (ep != NULL)
 
                if (newlineptr == NULL)
                   {
-                  EditVerbose("%s: LocateLineMatchingRegexp failed in %s, aborting editing\n",VPREFIX,filename);
+                  EditVerbose("LocateLineMatchingRegexp failed in %s, aborting editing\n",filename);
                   ep = ThrowAbort(ep);
                   }
                break;
@@ -696,7 +699,7 @@ while (ep != NULL)
       case IncrementPointer:
                if (! IncrementEditPointer(expdata,filestart))     /* edittools */
                   {
-                  printf ("%s: IncrementPointer failed in %s, aborting editing\n",VPREFIX,filename);
+                  printf ("IncrementPointer failed in %s, aborting editing\n",filename);
   	          ep = ThrowAbort(ep);
                   }
 
@@ -705,7 +708,7 @@ while (ep != NULL)
      case ReplaceLineWith:
                if (!ReplaceEditLineWith(expdata))
                   {
-                  printf("%s: aborting edit of file %s\n",VPREFIX,filename);
+                  printf("Aborting edit of file %s\n",filename);
                   continue;
                   }
                break;
@@ -713,8 +716,8 @@ while (ep != NULL)
       case DeleteToLineMatching:
                if (! DeleteToRegExp(&filestart,expdata))
                   {
-                  EditVerbose("%s: Nothing matched DeleteToLineMatching regular expression\n",VPREFIX);
-                  EditVerbose("%s: Aborting file editing of %s.\n",VPREFIX ,filename);
+                  EditVerbose("Nothing matched DeleteToLineMatching regular expression\n");
+                  EditVerbose("Aborting file editing of %s.\n" ,filename);
                   ep = ThrowAbort(ep);
                   }
                break;
@@ -722,8 +725,8 @@ while (ep != NULL)
       case DeleteNLines:
                if (! DeleteSeveralLines(&filestart,expdata))
                   {
-                  EditVerbose("%s: Could not delete %s lines from file\n",VPREFIX,expdata);
-                  EditVerbose("%s: Aborting file editing of %s.\n",VPREFIX,filename);
+                  EditVerbose("Could not delete %s lines from file\n",expdata);
+                  EditVerbose("Aborting file editing of %s.\n",filename);
                   ep = ThrowAbort(ep);
                   }
                break;
@@ -731,8 +734,8 @@ while (ep != NULL)
       case HashCommentToLineMatching:
                if (! CommentToRegExp(&filestart,expdata,"#",""))
                   {
-                  EditVerbose("%s: Nothing matched HashCommentToLineMatching regular expression\n",VPREFIX);
-                  EditVerbose("%s: Aborting file editing of %s.\n",VPREFIX,filename);
+                  EditVerbose("Nothing matched HashCommentToLineMatching regular expression\n");
+                  EditVerbose("Aborting file editing of %s.\n",filename);
                   ep = ThrowAbort(ep);
                   }
                break;
@@ -740,8 +743,8 @@ while (ep != NULL)
       case PercentCommentToLineMatching:
                if (! CommentToRegExp(&filestart,expdata,"%",""))
                   {
-                  EditVerbose("%s: Nothing matched PercentCommentToLineMatching regular expression\n",VPREFIX);
-                  EditVerbose("%s: Aborting file editing of %s.\n",VPREFIX,filename);
+                  EditVerbose("Nothing matched PercentCommentToLineMatching regular expression\n");
+                  EditVerbose("Aborting file editing of %s.\n",filename);
                   ep = ThrowAbort(ep);
                   }
                break;
@@ -749,8 +752,8 @@ while (ep != NULL)
       case CommentToLineMatching:
                if (! CommentToRegExp(&filestart,expdata,COMMENTSTART,COMMENTEND))
                   {
-                  EditVerbose("%s: Nothing matched CommentToLineMatching regular expression\n",VPREFIX);
-                  EditVerbose("%s: Aborting file editing of %s.\n",VPREFIX,filename);
+                  EditVerbose("Nothing matched CommentToLineMatching regular expression\n");
+                  EditVerbose("Aborting file editing of %s.\n",filename);
                   ep = ThrowAbort(ep);
                   }
                break;
@@ -758,8 +761,8 @@ while (ep != NULL)
       case CommentNLines:
                if (! CommentSeveralLines(&filestart,expdata,COMMENTSTART,COMMENTEND))
                   {
-                  EditVerbose("%s: Could not comment %s lines from file\n",VPREFIX,expdata);
-                  EditVerbose("%s: Aborting file editing of %s.\n",VPREFIX,filename);
+                  EditVerbose("Could not comment %s lines from file\n",expdata);
+                  EditVerbose("Aborting file editing of %s.\n",filename);
                   ep = ThrowAbort(ep);
                   }
                break;
@@ -767,8 +770,8 @@ while (ep != NULL)
       case UnCommentNLines:
                if (! UnCommentSeveralLines(&filestart,expdata,COMMENTSTART,COMMENTEND))
                   {
-                  EditVerbose("%s: Could not comment %s lines from file\n",VPREFIX,expdata);
-                  EditVerbose("%s: Aborting file editing of %s.\n",VPREFIX,filename);
+                  EditVerbose("Could not comment %s lines from file\n",expdata);
+                  EditVerbose("Aborting file editing of %s.\n",filename);
                   ep = ThrowAbort(ep);
                   }
                break;
@@ -786,13 +789,14 @@ while (ep != NULL)
                break;
 	       
       case SetScript:
-               currenteditscript = expdata;
-               break;
+                strncpy(currenteditscript, expdata, bufsize);
+                currenteditscript[bufsize-1] = '\0';
+                break;
 
       case RunScript:
                if (! RunEditScript(expdata,filename,&filestart,ptr))
                   {
-                  printf("%s: Aborting further edits to %s\n",VPREFIX,filename);
+                  printf("Aborting further edits to %s\n",filename);
                   ep = ThrowAbort(ep);
                   }
                break;
@@ -802,7 +806,7 @@ while (ep != NULL)
                   {
                   if (! RunEditScript(currenteditscript,filename,&filestart,ptr))
                      {
-                     printf("%s: Aborting further edits to %s\n",VPREFIX,filename);
+                     printf("Aborting further edits to %s\n",filename);
                      ep = ThrowAbort(ep);
                      }
                   }
@@ -813,7 +817,7 @@ while (ep != NULL)
                   {
                   if (! RunEditScript(currenteditscript,filename,&filestart,ptr))
                      {
-                     printf("%s: Aborting further edits to %s\n",VPREFIX,filename);
+                     printf("Aborting further edits to %s\n",filename);
                      ep = ThrowAbort(ep);
                      }
                   }
@@ -910,6 +914,31 @@ while (ep != NULL)
                   }
                break;
 
+
+      case BeginGroupIfDefined:
+               if (!IsExcluded(expdata))
+                 {
+                 EditVerbose("(Begin Group - class %s defined)\n", expdata);
+                 }
+               else
+                 {
+                 EditVerbose("(Begin Group - class %s not defined - skipping)\n", expdata);
+                 ep = SkipToEndGroup(ep,filename);
+                 }
+               break;
+
+      case BeginGroupIfNotDefined:
+               if (IsExcluded(expdata))
+                 {
+                 EditVerbose("(Begin Group - class %s not defined)\n", expdata);
+                 }
+               else
+                 {
+                 EditVerbose("(Begin Group - class %s defined - skipping)\n", expdata);
+                 ep = SkipToEndGroup(ep,filename);
+                 }
+               break;
+	       
      case BeginGroupIfFileExists:
                if (stat(expdata,&tmpstat) == -1)
                   {
@@ -930,7 +959,11 @@ while (ep != NULL)
                break;
 
       case With:
-               GlobalReplace(&filestart,searchstr,expdata);
+               if (!GlobalReplace(&filestart,searchstr,expdata))
+		  {
+		  snprintf(OUTPUT,bufsize*2,"Error editing file %s",filename);
+		  CfLog(cferror,OUTPUT,"");
+		  }
                break;
 
       case FixEndOfLine:
@@ -972,12 +1005,32 @@ while (ep != NULL)
 		  {
 	          if (!feof(loop_fp))
 		     {
-		     bzero(ACTIONBUFF,bufsize);
-		     ReadLine(ACTIONBUFF,bufsize,loop_fp); /* Like SetLine */
+		     bzero(EDITBUFF,bufsize);
+
+		     while (ReadLine(EDITBUFF,bufsize,loop_fp)) /* Like SetLine */
+			{
+			if (strlen(ACTIONBUFF) == 0)
+			   {
+			   EditVerbose("ForEachLineIn skipping blank line");
+			   continue;
+			   }
+			break;
+			}
+		     if (strlen(ACTIONBUFF) == 0)
+			{
+			EditVerbose("EndForEachLineIn");
+			fclose(loop_fp);
+			loopstart = NULL;
+			while(ep->code != EndLoop)
+			   {
+			   ep = ep->next;
+			   }
+			EditVerbose("EndForEachLineIn, set current line to: %s\n",ACTIONBUFF);
+			}
 		     }
 		  else
 		     {
-		     EditVerbose("EndForeach\n");
+		     EditVerbose("EndForEachLineIn");
 		     
 		     fclose(loop_fp);
 		     loopstart = NULL;
@@ -997,20 +1050,19 @@ while (ep != NULL)
 	       continue;
 
       case ReplaceLinesMatchingField:
-	       ReplaceWithFieldMatch(&filestart,expdata,ACTIONBUFF,spliton,filename);
+	       ReplaceWithFieldMatch(&filestart,expdata,EDITBUFF,spliton,filename);
 	       break;
 
       case AppendToLineIfNotContains:
 	       AppendToLine(CURRENTLINEPTR,expdata,filename);
 	       break;
 
+      case EditFilter:
       case DefineClasses:
-	       break;
-
       case ElseDefineClasses:
 	       break;
 
-      default: sprintf(OUTPUT,"Unknown action in editing of file %s\n",filename);
+      default: snprintf(OUTPUT,bufsize*2,"Unknown action in editing of file %s\n",filename);
 	       CfLog(cferror,OUTPUT,"");
                break;
       }
@@ -1020,7 +1072,7 @@ while (ep != NULL)
 
 abort :  
 
-EditVerbose("%s: End editing %s\n",VPREFIX,filename);
+EditVerbose("End editing %s\n",filename);
 EditVerbose(".....................................................................\n");
 
 EDITVERBOSE = false;
@@ -1045,8 +1097,9 @@ ResetOutputRoute('d','d');
 ReleaseCurrentLock();
 
 umask(maskval);
- 
+
 DeleteItemList(filestart);
+ptr->done = 'y';
 }
 
 /********************************************************************/
@@ -1063,7 +1116,7 @@ sscanf(str,"%d", &n);
 
 if (n == 0)
    {
-   printf("%s: Illegal increment value: %s\n",VPREFIX,str);
+   printf("Illegal increment value: %s\n",str);
    return false;
    }
 
@@ -1088,8 +1141,8 @@ if (n < 0)
    {
    if (CURRENTLINENUMBER + n < 1)
       {
-      EditVerbose("%s: pointer decrements to before start of file!\n",VPREFIX);
-      EditVerbose("%s: pointer stopped at start of file!\n",VPREFIX);
+      EditVerbose("pointer decrements to before start of file!\n");
+      EditVerbose("pointer stopped at start of file!\n");
       CURRENTLINEPTR=liststart;
       CURRENTLINENUMBER=1;      
       return true;
@@ -1144,7 +1197,7 @@ sscanf(str,"%d", &n);
 
 if (n < 1)
    {
-   printf("%s: Illegal reset value: %s\n",VPREFIX,str);
+   printf("Illegal reset value: %s\n",str);
    return false;
    }
 
@@ -1154,7 +1207,7 @@ for (ip = list; (i < n) && (ip != NULL); ip=ip->next, i++)
 
 if (i < n || ip == NULL)
    {
-   printf("%s: Search for (%s) begins after end of file!!\n",VPREFIX,str);
+   printf("Search for (%s) begins after end of file!!\n",str);
    return false;
    }
 
@@ -1182,7 +1235,7 @@ if (strcmp(string,CURRENTLINEPTR->name) == 0)
 
 if ((sp = malloc(strlen(string)+1)) == NULL)
    {
-   printf("%s: Memory allocation failed in ReplaceEditLineWith, aborting edit.\n",VPREFIX);
+   printf("Memory allocation failed in ReplaceEditLineWith, aborting edit.\n");
    return false;
    }
 
@@ -1207,7 +1260,7 @@ struct Edit *ptr;
 
 if (script == NULL)
    {
-   printf("%s: No script defined for with SetScript\n",VPREFIX);
+   printf("No script defined for with SetScript\n");
    return false;
    }
 
@@ -1228,13 +1281,22 @@ else
 
 DeleteItemList(*filestart);
 
-sprintf (buffer,"%s %s %s  2>&1",script,fname,CLASSTEXT[VSYSTEMHARDCLASS]);
+snprintf(buffer,bufsize,"%s %s %s  2>&1",script,fname,CLASSTEXT[VSYSTEMHARDCLASS]);
 
 EditVerbose("Running command: %s\n",buffer);
 
-if ((pp = cfpopen(buffer,"r")) == NULL)
+      
+switch (ptr->useshell)
    {
-   printf("%s: edit script %s failed to open.\n",VPREFIX,fname);
+   case 'y':  pp = cfpopen_sh(buffer,"r");
+              break;
+   default:   pp = cfpopen(buffer,"r");
+              break;	     
+   }
+  
+if (pp == NULL)
+   {
+   printf("Edit script %s failed to open.\n",fname);
    return false;
    }
 
@@ -1254,7 +1316,7 @@ cfpclose(pp);
 
 if (! LoadItemList(filestart,fname))
    {
-   printf("          File was marked for editing\n");
+   EditVerbose("File was marked for editing\n");
    return false;
    }
 
@@ -1320,7 +1382,7 @@ if (strcmp("dos",type) == 0 || strcmp("DOS",type) == 0)
    return;
    }
 
-printf("%s: Unknown file format: %s\n",VPREFIX,type);
+printf("Unknown file format: %s\n",type);
 }
 
 /**************************************************************/
@@ -1341,7 +1403,7 @@ for (ip = VMOUNTABLES; ip != NULL; ip=ip->next)
       }
 
    sp++;
-   sprintf(buffer,"%s\t%s\t%s",sp,opts,ip->name);
+   snprintf(buffer,bufsize,"%s\t%s\t%s",sp,opts,ip->name);
 
    if (LocateNextItemContaining(*filestart,sp) == NULL)
       {
@@ -1368,6 +1430,8 @@ struct Edlist *actions;
   char expdata[bufsize];
   int fd;
 
+PARSING = true;
+ 
 for (ep = actions; ep != NULL; ep=ep->next)
    {
    if (IsExcluded(ep->classes))
@@ -1388,7 +1452,7 @@ for (ep = actions; ep != NULL; ep=ep->next)
 			     {
 			     if ((fd = creat(filename,0644)) == -1)
 				{
-				sprintf(OUTPUT,"Unable to create file %s\n",filename);
+				snprintf(OUTPUT,bufsize*2,"Unable to create file %s\n",filename);
 				CfLog(cfinform,OUTPUT,"creat");
 				}
 			     else
@@ -1396,7 +1460,7 @@ for (ep = actions; ep != NULL; ep=ep->next)
 				AUTOCREATED = true;
 				close(fd);
 				}
-			     sprintf(OUTPUT,"Creating file %s, mode 644\n",filename);
+			     snprintf(OUTPUT,bufsize*2,"Creating file %s, mode 644\n",filename);
 			     CfLog(cfinform,OUTPUT,"");
 			     umask(mask);
 			     return;
@@ -1415,25 +1479,25 @@ for (ep = actions; ep != NULL; ep=ep->next)
 		       
       case EditLog:    if (strcmp(ToLowerStr(expdata),"true") == 0 || strcmp(ToLowerStr(expdata),"on") == 0)
 	                  {
-			  log = 't';
+			  log = 'y';
 			  break;
 			  }
 
                        if (strcmp(ToLowerStr(expdata),"false") == 0 || strcmp(ToLowerStr(expdata),"off") == 0)
                           {
-			  log = 'f';
+			  log = 'n';
 			  break;
 			  }
 
       case EditInform: if (strcmp(ToLowerStr(expdata),"true") == 0 || strcmp(ToLowerStr(expdata),"on") == 0)
 	                  {
-			  inform = 't';
+			  inform = 'y';
 			  break;
 			  }
 
                        if (strcmp(ToLowerStr(expdata),"false") == 0 || strcmp(ToLowerStr(expdata),"off") == 0)
                           {
-			  inform = 'f';
+			  inform = 'n';
 			  break;
 			  }
 
@@ -1441,6 +1505,7 @@ for (ep = actions; ep != NULL; ep=ep->next)
       }
    }
 
+PARSING = false; 
 ResetOutputRoute(log,inform);
 }
 
@@ -1461,6 +1526,11 @@ if (editsdone)
    {
    for (ep = list->actions; ep != NULL; ep=ep->next)
       {
+      if (IsExcluded(ep->classes))
+	 {
+	 continue;
+	 }
+
       if (ep->code == DefineClasses)
 	 {
 	 break;
@@ -1471,6 +1541,11 @@ else
    {
    for (ep = list->actions; ep != NULL; ep=ep->next)
       {
+      if (IsExcluded(ep->classes))
+	 {
+	 continue;
+	 }
+      
       if (ep->code == ElseDefineClasses)
 	 {
 	 break;
@@ -1503,7 +1578,7 @@ struct Edlist *ThrowAbort(from)
 
 struct Edlist *from;
 
-{ struct Edlist *ep, *last;
+{ struct Edlist *ep, *last = NULL;
  
 for (ep = from; ep != NULL; ep=ep->next)
    {
@@ -1537,6 +1612,8 @@ while(ep != NULL)
       case BeginGroupIfFileIsNewer:
       case BeginGroupIfFileExists:
       case BeginGroupIfNoLineContaining:
+      case BeginGroupIfDefined:
+      case BeginGroupIfNotDefined:
 	  level ++;
       }
 
@@ -1553,7 +1630,7 @@ while(ep != NULL)
    
    if (ep->next == NULL)
       {
-      sprintf(OUTPUT,"Missing EndGroup in %s",filename);
+      snprintf(OUTPUT,bufsize*2,"Missing EndGroup in %s",filename);
       CfLog(cferror,OUTPUT,"");
       break;
       }
@@ -1576,35 +1653,35 @@ char *filename;
   struct stat statbuf;
   void *memseg;
 
-EditVerbose("%s: Begin (binary) editing %s\n",VPREFIX,filename);
+EditVerbose("Begin (binary) editing %s\n",filename);
 
 NUMBEROFEDITS = 0;
 search[0] = '\0'; 
  
 if (stat(filename,&statbuf) == -1)
    {
-   sprintf(OUTPUT,"Couldn't stat %s\n",filename);
+   snprintf(OUTPUT,bufsize*2,"Couldn't stat %s\n",filename);
    CfLog(cfverbose,OUTPUT,"stat");
    return false;
    }
 
 if ((EDITBINFILESIZE != 0) &&(statbuf.st_size > EDITBINFILESIZE))
    {
-   sprintf(OUTPUT,"File %s is bigger than the limit <editbinaryfilesize>\n",filename);
+   snprintf(OUTPUT,bufsize*2,"File %s is bigger than the limit <editbinaryfilesize>\n",filename);
    CfLog(cfinform,OUTPUT,"");
    return(false);
    }
 
 if (! S_ISREG(statbuf.st_mode))
    {
-   sprintf(OUTPUT,"%s is not a plain file\n",filename);
+   snprintf(OUTPUT,bufsize*2,"%s is not a plain file\n",filename);
    CfLog(cfinform,OUTPUT,"");
    return false;
    }
 
 if ((memseg = malloc(statbuf.st_size+buffer_margin)) == NULL)
    {
-   sprintf(OUTPUT,"Unable to load file %s into memory",filename);
+   snprintf(OUTPUT,bufsize*2,"Unable to load file %s into memory",filename);
    CfLog(cferror,OUTPUT,"malloc");
    return false;
    }
@@ -1635,6 +1712,9 @@ while (ep != NULL)
 	  WarnIfContainsFilePattern(memseg,statbuf.st_size,expdata,filename);
 	  break;
 
+      case EditMode:
+	  break;
+
       case ReplaceAll:
 	  Debug("Replace %s\n",expdata);
 	  strcpy(search,expdata);
@@ -1653,16 +1733,8 @@ while (ep != NULL)
 	     }
 	  break;
 
-      case EditMode:
-          if (!DONTDO)
-	     {
-	     mode_t plus,minus;
-	     ParseModeString(expdata,&plus,&minus);
-	     chmod(filename,plus);
-	     }
-	  break;
       default:
-	  sprintf(OUTPUT,"Cannot use %s in a binary edit (%s)",VEDITNAMES[ep->code],filename);
+	  snprintf(OUTPUT,bufsize*2,"Cannot use %s in a binary edit (%s)",VEDITNAMES[ep->code],filename);
 	  CfLog(cferror,OUTPUT,"");
       }
 
@@ -1680,7 +1752,7 @@ else
    }
 
 free(memseg);
-EditVerbose("%s: End editing %s\n",VPREFIX,filename);
+EditVerbose("End editing %s\n",filename);
 EditVerbose(".....................................................................\n"); 
 return true;
 }
@@ -1704,7 +1776,7 @@ Debug("LoadBinaryFile(%s,%d)\n",source,size);
   
 if ((sd = open(source,O_RDONLY)) == -1)
    {
-   sprintf(OUTPUT,"Can't copy %s!\n",source);
+   snprintf(OUTPUT,bufsize*2,"Can't copy %s!\n",source);
    CfLog(cfinform,OUTPUT,"open");
    return false;
    }
@@ -1765,7 +1837,7 @@ unlink(new);  /* To avoid link attacks */
  
 if ((dd = open(new,O_WRONLY|O_CREAT|O_TRUNC|O_EXCL, 0600)) == -1)
    {
-   sprintf(OUTPUT,"Copy %s security - failed attempt to exploit a race? (Not copied)\n",file);
+   snprintf(OUTPUT,bufsize*2,"Copy %s security - failed attempt to exploit a race? (Not copied)\n",file);
    CfLog(cfinform,OUTPUT,"open");
    return false;
    }
@@ -1778,7 +1850,7 @@ if (! IsItemIn(VREPOSLIST,new))
    {
    if (rename(file,backup) == -1)
       {
-      sprintf(OUTPUT,"Error while renaming backup %s\n",file);
+      snprintf(OUTPUT,bufsize*2,"Error while renaming backup %s\n",file);
       CfLog(cferror,OUTPUT,"rename ");
       unlink(new);
       return false;
@@ -1787,7 +1859,7 @@ if (! IsItemIn(VREPOSLIST,new))
       {
       if (rename(new,file) == -1)
 	 {
-	 sprintf(OUTPUT,"Error while renaming %s\n",file);
+	 snprintf(OUTPUT,bufsize*2,"Error while renaming %s\n",file);
 	 CfLog(cferror,OUTPUT,"rename");
 	 return false;
 	 }       
@@ -1798,7 +1870,7 @@ if (! IsItemIn(VREPOSLIST,new))
 
 if (rename(new,file) == -1)
    {
-   sprintf(OUTPUT,"Error while renaming %s\n",file);
+   snprintf(OUTPUT,bufsize*2,"Error while renaming %s\n",file);
    CfLog(cferror,OUTPUT,"rename");
    return false;
    }       
@@ -1817,7 +1889,6 @@ char *data,*filename;
 { off_t sp;
   regex_t rx,rxcache;
   regmatch_t pmatch;
-  size_t nmatch = 1;
 
 Debug("WarnIfContainsRegex(%s)\n",data); 
 
@@ -1825,7 +1896,7 @@ for (sp = 0; sp < (off_t)(size-strlen(data)); sp++)
    {
    if (bcmp((char *) memseg+sp,data,strlen(data)) == 0)
       {
-      sprintf(OUTPUT,"WARNING! File %s contains literal string %.255s",filename,data);
+      snprintf(OUTPUT,bufsize*2,"WARNING! File %s contains literal string %.255s",filename,data);
       CfLog(cferror,OUTPUT,"");
       return;
       }
@@ -1842,7 +1913,7 @@ for (sp = 0; sp < (off_t)(size-strlen(data)); sp++)
 
    if (regexec(&rx,(char *)memseg+sp,1,&pmatch,0) == 0)
       {
-      sprintf(OUTPUT,"WARNING! File %s contains regular expression %.255s",filename,data);
+      snprintf(OUTPUT,bufsize*2,"WARNING! File %s contains regular expression %.255s",filename,data);
       CfLog(cferror,OUTPUT,"");
       regfree(&rx);
       return;
@@ -1866,35 +1937,35 @@ Debug("WarnIfContainsFile(%s)\n",data);
 
 if (stat(data,&statbuf) == -1)
    {
-   sprintf(OUTPUT,"File %s cannot be opened",data);
+   snprintf(OUTPUT,bufsize*2,"File %s cannot be opened",data);
    CfLog(cferror,OUTPUT,"stat");
    return;
    }
 
 if (! S_ISREG(statbuf.st_mode))
    {
-   sprintf(OUTPUT,"File %s cannot be used as a binary pattern",data);
+   snprintf(OUTPUT,bufsize*2,"File %s cannot be used as a binary pattern",data);
    CfLog(cferror,OUTPUT,"");
    return;
    }
 
 if (statbuf.st_size > size)
    {
-   sprintf(OUTPUT,"File %s is larger than the search file, ignoring",data);
+   snprintf(OUTPUT,bufsize*2,"File %s is larger than the search file, ignoring",data);
    CfLog(cfinform,OUTPUT,"");
    return;
    }
 
 if ((pattern = malloc(statbuf.st_size+buffer_margin)) == NULL)
    {
-   sprintf(OUTPUT,"File %s cannot be loaded",data);
+   snprintf(OUTPUT,bufsize*2,"File %s cannot be loaded",data);
    CfLog(cferror,OUTPUT,"");
    return;
    }
 
 if (!LoadBinaryFile(data,statbuf.st_size,pattern))
    {
-   sprintf(OUTPUT,"File %s cannot be opened",data);
+   snprintf(OUTPUT,bufsize*2,"File %s cannot be opened",data);
    CfLog(cferror,OUTPUT,"stat");
    return;
    }
@@ -1903,7 +1974,7 @@ for (sp = 0; sp < (off_t)(size-statbuf.st_size); sp++)
    {
    if (bcmp((char *) memseg+sp,pattern,statbuf.st_size-1) == 0)
       {
-      sprintf(OUTPUT,"WARNING! File %s contains the contents of reference file %s",filename,data);
+      snprintf(OUTPUT,bufsize*2,"WARNING! File %s contains the contents of reference file %s",filename,data);
       CfLog(cferror,OUTPUT,"");
       free(pattern);
       return;
@@ -1942,7 +2013,7 @@ for (sp = 0; sp < (off_t)(size-strlen(replace)); sp++)
       {
       if (pmatch.rm_eo-pmatch.rm_so < strlen(replace))
 	 {
-	 sprintf(OUTPUT,"Cannot perform binary replacement: string doesn't fit in %s",filename);
+	 snprintf(OUTPUT,bufsize*2,"Cannot perform binary replacement: string doesn't fit in %s",filename);
 	 CfLog(cfverbose,OUTPUT,"");
 	 }
       else
