@@ -73,6 +73,25 @@ char *MONTHTEXT[] =
 /* Level 1                                                           */
 /*********************************************************************/
 
+int Day2Number(datestring)
+
+char *datestring;
+
+{ int i = 0;
+
+for (i = 0; i < 7; i++)
+   {
+   if (strncmp(datestring,DAYTEXT[i],3) == 0)
+      {
+      return i;
+      }
+   }
+
+return -1;
+}
+
+/*********************************************************************/
+
 void AddInstallable(classlist)
 
 char *classlist;
@@ -123,9 +142,9 @@ Debug("AddMultipleClasses(%s)\n",classlist);
  
 for (sp = classlist; *sp != '\0'; sp++)
    {
-   currentitem[0] = '\0';
+   bzero(currentitem,maxvarsize);
 
-   sscanf(sp,"%[^,:.]",currentitem);
+   sscanf(sp,"%250[^.:,]",currentitem);
 
    sp += strlen(currentitem);
 
@@ -325,11 +344,14 @@ if (strncmp(class,"IfElapsed",strlen("IfElapsed")) == 0)
       Silent("%s: silly IfElapsed parameter in action sequence, using default...\n",VPREFIX);
       return true;
       }
-   
-   VIFELAPSED = value;
 
-   Verbose("                  IfElapsed time: %d minutes\n",VIFELAPSED);
-   return true;
+   if (!PARSING)
+      {
+      VIFELAPSED = value;
+      
+      Verbose("                  IfElapsed time: %d minutes\n",VIFELAPSED);
+      return true;
+      }
    }
 
 if (strncmp(class,"ExpireAfter",strlen("ExpireAfter")) == 0)
@@ -341,10 +363,13 @@ if (strncmp(class,"ExpireAfter",strlen("ExpireAfter")) == 0)
       Silent("%s: silly ExpireAter parameter in action sequence, using default...\n",VPREFIX);
       return true;
       }
-   
-   VEXPIREAFTER = value;
-   Verbose("\n                  ExpireAfter time: %d minutes\n",VEXPIREAFTER); 
-   return true;
+
+   if (!PARSING)
+      {
+      VEXPIREAFTER = value;
+      Verbose("\n                  ExpireAfter time: %d minutes\n",VEXPIREAFTER); 
+      return true;
+      }
    }
 
 return false;
@@ -546,7 +571,7 @@ struct Item *list;
 
 { char *sp, *atom;
   char cbuff[bufsize];
-  int count = 1, brackets = 0;
+  int count = 1;
   int negation = false;
 
 count = CountEvalAtoms(class);
@@ -780,24 +805,29 @@ char *str;
 
 { char *sp;
   int i;
+  enum actions action;
 
-ACTION = none;
+action = none;
 
 for (sp = str; *sp != '\0'; sp++)
    {
    *sp = ToLower(*sp);
+   if (*sp == ':')
+      {
+      *sp = '\0';
+      }
    }
 
 for (i = 1; ACTIONID[i] != '\0'; i++)
    {
    if (strcmp(ACTIONID[i],str) == 0)
       {
-      ACTION = (enum actions) i;
+      action = (enum actions) i;
       break;
       }
    }
 
-if (ACTION == none)
+if (action == none)
   {
   yyerror("Indexed macro specified no action");
   FatalError("Could not compile action");
