@@ -98,7 +98,7 @@ Debug2("cfengine: EmptyDir(%s)\n",path);
 
 if ((dirh = opendir(path)) == NULL)
    {
-   sprintf(OUTPUT,"Can't open directory %s\n",path);
+   snprintf(OUTPUT,bufsize*2,"Can't open directory %s\n",path);
    CfLog(cferror,OUTPUT,"opendir");
    return true;
    }
@@ -120,7 +120,7 @@ return (!count);
 
 /*********************************************************************/
 
-int RecursiveCheck(name,plus,minus,action,uidlist,gidlist,recurse,rlevel,ptr,sb)
+void RecursiveCheck(name,plus,minus,action,uidlist,gidlist,recurse,rlevel,ptr)
 
 char *name;
 mode_t plus,minus;
@@ -130,42 +130,35 @@ enum fileactions action;
 int recurse;
 int rlevel;
 struct File *ptr;
-struct stat *sb;
 
 { DIR *dirh;
   struct dirent *dirp;
   char pcwd[bufsize];
   struct stat statbuf;
-  int goback;
 
 if (recurse == -1)
    {
-   return false;
+   return;
    }
 
 if (rlevel > recursion_limit)
    {
-   sprintf(OUTPUT,"WARNING: Very deep nesting of directories (>%d deep): %s (Aborting files)",rlevel,name);
+   snprintf(OUTPUT,bufsize*2,"WARNING: Very deep nesting of directories (>%d deep): %s (Aborting files)",rlevel,name);
    CfLog(cferror,OUTPUT,"");
-   return false;
+   return;
    }
  
 bzero(pcwd,bufsize); 
 
 Debug("RecursiveCheck(%s,+%o,-%o)\n",name,plus,minus);
 
-if (!DirPush(name,sb))
-   {
-   return false;
-   }
-
-if ((dirh = opendir(".")) == NULL)
+if ((dirh = opendir(name)) == NULL)
    {
    if (lstat(name,&statbuf) != -1)
       {
       CheckExistingFile(name,plus,minus,action,uidlist,gidlist,&statbuf,ptr,ptr->acl_aliases);
       }
-   return true;
+   return;
    }
 
 for (dirp = readdir(dirh); dirp != NULL; dirp = readdir(dirh))
@@ -185,24 +178,24 @@ for (dirp = readdir(dirh); dirp != NULL; dirp = readdir(dirh))
 
    if (BufferOverflow(pcwd,dirp->d_name))
       {
-      return true;
+      return;
       }
 
    strcat(pcwd,dirp->d_name);
    
    
-   if (lstat(dirp->d_name,&statbuf) == -1)
+   if (lstat(pcwd,&statbuf) == -1)
       {
-      sprintf(OUTPUT,"RecursiveCheck was working in %s when this happened:\n",pcwd);
+      snprintf(OUTPUT,bufsize*2,"RecursiveCheck was working in %s when this happened:\n",pcwd);
       CfLog(cferror,OUTPUT,"lstat");
       continue;
       }
    
    if (TRAVLINKS)
       {
-      if (stat(dirp->d_name,&statbuf) == -1)
+      if (stat(pcwd,&statbuf) == -1)
          {
-         sprintf(OUTPUT,"RecursiveCheck was working on %s when this happened:\n",pcwd);
+         snprintf(OUTPUT,bufsize*2,"RecursiveCheck was working on %s when this happened:\n",pcwd);
 	 CfLog(cferror,OUTPUT,"stat");
          continue;
          }
@@ -226,9 +219,7 @@ for (dirp = readdir(dirh); dirp != NULL; dirp = readdir(dirh))
          if ((ptr->recurse > 1) || (ptr->recurse == INFINITERECURSE))
             {
             CheckExistingFile(pcwd,plus,minus,action,uidlist,gidlist,&statbuf,ptr,ptr->acl_aliases);
-            goback = RecursiveCheck(pcwd,plus,minus,action,uidlist,gidlist,recurse-1,rlevel+1,ptr,&statbuf);
-
-	    DirPop(goback,name,sb);
+            RecursiveCheck(pcwd,plus,minus,action,uidlist,gidlist,recurse-1,rlevel+1,ptr);
             }
          else
             {
@@ -243,7 +234,6 @@ for (dirp = readdir(dirh); dirp != NULL; dirp = readdir(dirh))
    }
 
 closedir(dirh);
-return true; 
 }
 
 
@@ -262,7 +252,8 @@ struct Item *acl_aliases;
  
 { mode_t newperm = dstat->st_mode, maskvalue;
   int amroot = true, fixmode = true, docompress=false;
-  char digest[17];
+  unsigned char digest[EVP_MAX_MD_SIZE+1];
+
 #if defined HAVE_CHFLAGS
   u_long newflags;
 #endif
@@ -312,7 +303,7 @@ if (ptr != NULL)
 
    if (action == alert)
       {
-      sprintf(OUTPUT,"Alert specified on file %s (m=%o,o=%d,g=%d)",file,(dstat->st_mode & 07777),dstat->st_uid,dstat->st_gid);
+      snprintf(OUTPUT,bufsize*2,"Alert specified on file %s (m=%o,o=%d,g=%d)",file,(dstat->st_mode & 07777),dstat->st_uid,dstat->st_gid);
       CfLog(cferror,OUTPUT,"");
       return;
       }
@@ -371,7 +362,7 @@ if (dstat->st_uid == 0 && (dstat->st_mode & S_ISUID))
          {
          if (amroot)
 	    {
-	    sprintf(OUTPUT,"NEW SETUID root PROGRAM %s\n",file);
+	    snprintf(OUTPUT,bufsize*2,"NEW SETUID root PROGRAM %s\n",file);
 	    CfLog(cfinform,OUTPUT,"");
 	    }
          PrependItem(&VSETUIDLIST,file,NULL);
@@ -383,7 +374,7 @@ if (dstat->st_uid == 0 && (dstat->st_mode & S_ISUID))
          {
          case fixall:
          case fixdirs:
-         case fixplain:  sprintf(OUTPUT,"Removing setuid (root) flag from %s...\n\n",file);
+         case fixplain:  snprintf(OUTPUT,bufsize*2,"Removing setuid (root) flag from %s...\n\n",file);
 	                 CfLog(cfinform,OUTPUT,"");
 			 
 	                 if (ptr != NULL)
@@ -395,7 +386,7 @@ if (dstat->st_uid == 0 && (dstat->st_mode & S_ISUID))
          case warndirs:
          case warnplain: if (amroot)
 	                    {
-      	                    sprintf(OUTPUT,"WARNING setuid (root) flag on %s...\n\n",file);
+      	                    snprintf(OUTPUT,bufsize*2,"WARNING setuid (root) flag on %s...\n\n",file);
 			    CfLog(cferror,OUTPUT,"");
 	                    }
                          break;
@@ -418,7 +409,7 @@ if (dstat->st_uid == 0 && (dstat->st_mode & S_ISGID))
             {
             if (amroot)
 	       {
-	       sprintf(OUTPUT,"NEW SETGID root PROGRAM %s\n",file);
+	       snprintf(OUTPUT,bufsize*2,"NEW SETGID root PROGRAM %s\n",file);
 	       CfLog(cferror,OUTPUT,"");
 	       }
             PrependItem(&VSETUIDLIST,file,NULL);
@@ -431,7 +422,7 @@ if (dstat->st_uid == 0 && (dstat->st_mode & S_ISGID))
          {
          case fixall:
          case fixdirs:
-         case fixplain: sprintf(OUTPUT,"Removing setgid (root) flag from %s...\n\n",file);
+         case fixplain: snprintf(OUTPUT,bufsize*2,"Removing setgid (root) flag from %s...\n\n",file);
 	                CfLog(cfinform,OUTPUT,"");
 	                if (ptr != NULL)
 			   {
@@ -440,7 +431,7 @@ if (dstat->st_uid == 0 && (dstat->st_mode & S_ISGID))
 	                break;
          case warnall:
          case warndirs:
-         case warnplain: sprintf(OUTPUT,"WARNING setgid (root) flag on %s...\n\n",file);
+         case warnplain: snprintf(OUTPUT,bufsize*2,"WARNING setgid (root) flag on %s...\n\n",file);
 	                 CfLog(cferror,OUTPUT,"");
                          break;
 
@@ -457,24 +448,27 @@ if (CheckOwner(file,action,uidlist,gidlist,dstat))
       }
    }
 
-if ((ptr != NULL) && S_ISREG(dstat->st_mode) && (ptr->checksum == 'm'))
+if ((ptr != NULL) && S_ISREG(dstat->st_mode) && (ptr->checksum != 'n'))
     {
-    Debug("Checking MD5 integrity of %s\n",file);
-    cfMDFile(file,digest);
-    digest[16] = '\0';
+    Debug("Checking checksum integrity of %s\n",file);
+
+    bzero(digest,EVP_MAX_MD_SIZE+1);
+    ChecksumFile(file,digest,ptr->checksum);
     
-    if (ChecksumChanged(file,digest,cferror,false))
+    if (ChecksumChanged(file,digest,cferror,false,ptr->checksum))
        {
        }
     }
- 
+
 if (S_ISLNK(dstat->st_mode))             /* No point in checking permission on a link */
    {
-   KillOldLink(file);
-
    if (ptr != NULL)
       {
-      AddMultipleClasses(ptr->defines);
+      KillOldLink(file,ptr->defines);
+      }
+   else
+      {
+      KillOldLink(file,NULL);
       }
    umask(maskvalue);
    return;
@@ -482,7 +476,7 @@ if (S_ISLNK(dstat->st_mode))             /* No point in checking permission on a
 
 if (stat(file,dstat) == -1)
    {
-   sprintf(OUTPUT,"Can't stat file %s while looking at permissions (was not copied?)\n",file);
+   snprintf(OUTPUT,bufsize*2,"Can't stat file %s while looking at permissions (was not copied?)\n",file);
    CfLog(cfverbose,OUTPUT,"stat");
    umask(maskvalue);
    return;
@@ -509,6 +503,7 @@ if (((newperm & 07777) == (dstat->st_mode & 07777)) && (action != touch))    /*
       {
       AddMultipleClasses(ptr->elsedef);
       }
+
    umask(maskvalue);
    return;
    }
@@ -531,25 +526,25 @@ if (fixmode)
       case warnplain:
 	  if (S_ISREG(dstat->st_mode))
 	     {
-	     sprintf(OUTPUT,"%s has permission %o\n",file,dstat->st_mode & 07777);
+	     snprintf(OUTPUT,bufsize*2,"%s has permission %o\n",file,dstat->st_mode & 07777);
 	     CfLog(cferror,OUTPUT,"");
-	     sprintf(OUTPUT,"[should be %o]\n",newperm & 07777);
+	     snprintf(OUTPUT,bufsize*2,"[should be %o]\n",newperm & 07777);
 	     CfLog(cferror,OUTPUT,"");
 	     }
 	  break;
       case warndirs:
 	  if (S_ISDIR(dstat->st_mode))
 	     {
-	     sprintf(OUTPUT,"%s has permission %o\n",file,dstat->st_mode & 07777);
+	     snprintf(OUTPUT,bufsize*2,"%s has permission %o\n",file,dstat->st_mode & 07777);
 	     CfLog(cferror,OUTPUT,"");
-	     sprintf(OUTPUT,"[should be %o]\n",newperm & 07777);
+	     snprintf(OUTPUT,bufsize*2,"[should be %o]\n",newperm & 07777);
 	     CfLog(cferror,OUTPUT,"");
 	     }
 	  break;
       case warnall:   
-	  sprintf(OUTPUT,"%s has permission %o\n",file,dstat->st_mode & 07777);
+	  snprintf(OUTPUT,bufsize*2,"%s has permission %o\n",file,dstat->st_mode & 07777);
 	  CfLog(cferror,OUTPUT,"");
-	  sprintf(OUTPUT,"[should be %o]\n",newperm & 07777);
+	  snprintf(OUTPUT,bufsize*2,"[should be %o]\n",newperm & 07777);
 	  CfLog(cferror,OUTPUT,"");
 	  break;
 	  
@@ -560,13 +555,13 @@ if (fixmode)
 		{
 		if (chmod (file,newperm & 07777) == -1)
 		   {
-		   sprintf(OUTPUT,"chmod failed on %s\n",file);
+		   snprintf(OUTPUT,bufsize*2,"chmod failed on %s\n",file);
 		   CfLog(cferror,OUTPUT,"chmod");
 		   break;
 		   }
 		}
 	     
-	     sprintf(OUTPUT,"%s had permission %o, changed it to %o\n",
+	     snprintf(OUTPUT,bufsize*2,"%s had permission %o, changed it to %o\n",
 		     file,dstat->st_mode & 07777,newperm & 07777);
 	     CfLog(cfinform,OUTPUT,"");
 	     
@@ -584,13 +579,13 @@ if (fixmode)
 		{
 		if (chmod (file,newperm & 07777) == -1)
 		   {
-		   sprintf(OUTPUT,"chmod failed on %s\n",file);
+		   snprintf(OUTPUT,bufsize*2,"chmod failed on %s\n",file);
 		   CfLog(cferror,OUTPUT,"chmod");
 		   break;
 		   }
 		}
 	     
-	     sprintf(OUTPUT,"%s had permission %o, changed it to %o\n",
+	     snprintf(OUTPUT,bufsize*2,"%s had permission %o, changed it to %o\n",
 		     file,dstat->st_mode & 07777,newperm & 07777);
 	     CfLog(cfinform,OUTPUT,"");
 	     
@@ -606,13 +601,13 @@ if (fixmode)
 	     {
 	     if (chmod (file,newperm & 07777) == -1)
 		{
-		sprintf(OUTPUT,"chmod failed on %s\n",file);
+		snprintf(OUTPUT,bufsize*2,"chmod failed on %s\n",file);
 		CfLog(cferror,OUTPUT,"chmod");
 		break;
 		}
 	     }
 	  
-	  sprintf(OUTPUT,"%s had permission %o, changed it to %o\n",
+	  snprintf(OUTPUT,bufsize*2,"%s had permission %o, changed it to %o\n",
 		  file,dstat->st_mode & 07777,newperm & 07777);
 	  CfLog(cfinform,OUTPUT,"");
 	  
@@ -627,7 +622,7 @@ if (fixmode)
 	     {
 	     if (chmod (file,newperm & 07777) == -1)
 		{
-		sprintf(OUTPUT,"chmod failed on %s\n",file);
+		snprintf(OUTPUT,bufsize*2,"chmod failed on %s\n",file);
 		CfLog(cferror,OUTPUT,"chmod");
 		break;
 		}
@@ -643,6 +638,7 @@ if (fixmode)
       }
    }
 
+ 
 #if defined HAVE_CHFLAGS  /* BSD special flags */
 
 if (ptr != NULL)
@@ -666,25 +662,25 @@ if (ptr != NULL)
 	 case warnplain:
 	     if (S_ISREG(dstat->st_mode))
 		{
-		sprintf(OUTPUT,"%s has flags %o\n",file,dstat->st_flags & CHFLAGS_MASK);
+		snprintf(OUTPUT,bufsize*2,"%s has flags %o\n",file,dstat->st_flags & CHFLAGS_MASK);
 		CfLog(cferror,OUTPUT,"");
-		sprintf(OUTPUT,"[should be %o]\n",newflags & CHFLAGS_MASK);
+		snprintf(OUTPUT,bufsize*2,"[should be %o]\n",newflags & CHFLAGS_MASK);
 		CfLog(cferror,OUTPUT,"");
 		}
 	     break;
 	 case warndirs:
 	     if (S_ISDIR(dstat->st_mode))
 		{
-		sprintf(OUTPUT,"%s has flags %o\n",file,dstat->st_mode & CHFLAGS_MASK);
+		snprintf(OUTPUT,bufsize*2,"%s has flags %o\n",file,dstat->st_mode & CHFLAGS_MASK);
 		CfLog(cferror,OUTPUT,"");
-		sprintf(OUTPUT,"[should be %o]\n",newflags & CHFLAGS_MASK);
+		snprintf(OUTPUT,bufsize*2,"[should be %o]\n",newflags & CHFLAGS_MASK);
 		CfLog(cferror,OUTPUT,"");
 		}
 	     break;
 	 case warnall:
-	     sprintf(OUTPUT,"%s has flags %o\n",file,dstat->st_mode & CHFLAGS_MASK);
+	     snprintf(OUTPUT,bufsize*2,"%s has flags %o\n",file,dstat->st_mode & CHFLAGS_MASK);
 	     CfLog(cferror,OUTPUT,"");
-	     sprintf(OUTPUT,"[should be %o]\n",newflags & CHFLAGS_MASK);
+	     snprintf(OUTPUT,bufsize*2,"[should be %o]\n",newflags & CHFLAGS_MASK);
 	     CfLog(cferror,OUTPUT,"");
 	     break;
 	     
@@ -696,13 +692,13 @@ if (ptr != NULL)
 		   {
 		   if (chflags (file,newflags & CHFLAGS_MASK) == -1)
 		      {
-		      sprintf(OUTPUT,"chflags failed on %s\n",file);
+		      snprintf(OUTPUT,bufsize*2,"chflags failed on %s\n",file);
 		      CfLog(cferror,OUTPUT,"chflags");
 		      break;
 		      }
 		   }
 		
-		sprintf(OUTPUT,"%s had flags %o, changed it to %o\n",
+		snprintf(OUTPUT,bufsize*2,"%s had flags %o, changed it to %o\n",
 			file,dstat->st_flags & CHFLAGS_MASK,newflags & CHFLAGS_MASK);
 		CfLog(cfinform,OUTPUT,"");
 		
@@ -720,13 +716,13 @@ if (ptr != NULL)
 		   {
 		   if (chflags (file,newflags & CHFLAGS_MASK) == -1)
 		      {
-		      sprintf(OUTPUT,"chflags failed on %s\n",file);
+		      snprintf(OUTPUT,bufsize*2,"chflags failed on %s\n",file);
 		      CfLog(cferror,OUTPUT,"chflags");
 		      break;
 		      }
 		   }
 		
-		sprintf(OUTPUT,"%s had flags %o, changed it to %o\n",
+		snprintf(OUTPUT,bufsize*2,"%s had flags %o, changed it to %o\n",
 			file,dstat->st_flags & CHFLAGS_MASK,newflags & CHFLAGS_MASK);
 		CfLog(cfinform,OUTPUT,"");
 		
@@ -742,13 +738,13 @@ if (ptr != NULL)
 	     {
 	     if (chflags (file,newflags & CHFLAGS_MASK) == -1)
 		{
-		sprintf(OUTPUT,"chflags failed on %s\n",file);
+		snprintf(OUTPUT,bufsize*2,"chflags failed on %s\n",file);
 		CfLog(cferror,OUTPUT,"chflags");
 		break;
 		}
 	     }
 	  
-	  sprintf(OUTPUT,"%s had flags %o, changed it to %o\n",
+	  snprintf(OUTPUT,bufsize*2,"%s had flags %o, changed it to %o\n",
 		  file,dstat->st_flags & CHFLAGS_MASK,newflags & CHFLAGS_MASK);
 	  CfLog(cfinform,OUTPUT,"");
 	  
@@ -763,7 +759,7 @@ if (ptr != NULL)
 	     {
 	     if (chflags (file,newflags & CHFLAGS_MASK) == -1)
 		{
-		sprintf(OUTPUT,"chflags failed on %s\n",file);
+		snprintf(OUTPUT,bufsize*2,"chflags failed on %s\n",file);
 		CfLog(cferror,OUTPUT,"chflags");
 		break;
 		}
@@ -914,13 +910,13 @@ else
                         {
                         if (uid != (uid_t) -1)
                            {
-                           sprintf(OUTPUT,"  (Change owner to uid %d if possible)\n",uid);
+                           snprintf(OUTPUT,bufsize*2,"  (Change owner to uid %d if possible)\n",uid);
                            CfLog(cfinform,OUTPUT,"");
                            }
 
                         if (gid != (gid_t) -1)
                            {
-                           sprintf(OUTPUT,"  (Change group to gid %d if possible)\n",gid);
+                           snprintf(OUTPUT,bufsize*2,"  (Change group to gid %d if possible)\n",gid);
                            CfLog(cfinform,OUTPUT,"");
                            }
                         }
@@ -932,7 +928,7 @@ else
 		     Debug("Using LCHOWN function\n");
                      if (lchown(file,uid,gid) == -1)
                         {
-                        sprintf(OUTPUT,"Cannot set ownership on link %s!\n",file);
+                        snprintf(OUTPUT,bufsize*2,"Cannot set ownership on link %s!\n",file);
                         CfLog(cflogonly,OUTPUT,"lchown");
                         }
 		     else
@@ -945,7 +941,7 @@ else
                      {
                      if (chown(file,uid,gid) == -1)
                         {
-                        sprintf(OUTPUT,"Cannot set ownership on file %s!\n",file);
+                        snprintf(OUTPUT,bufsize*2,"Cannot set ownership on file %s!\n",file);
                         CfLog(cflogonly,OUTPUT,"chown");
                         }
 		     else
@@ -961,21 +957,21 @@ else
       case warnplain:
                   if ((pw = getpwuid(statbuf->st_uid)) == NULL)
                      {
-                     sprintf(OUTPUT,"File %s is not owned by anybody in the passwd database\n",file);
+                     snprintf(OUTPUT,bufsize*2,"File %s is not owned by anybody in the passwd database\n",file);
 		     CfLog(cferror,OUTPUT,"");
-                     sprintf(OUTPUT,"(uid = %d,gid = %d)\n",statbuf->st_uid,statbuf->st_gid);
+                     snprintf(OUTPUT,bufsize*2,"(uid = %d,gid = %d)\n",statbuf->st_uid,statbuf->st_gid);
 		     CfLog(cferror,OUTPUT,"");
                      break;
                      }
 
                   if ((gp = getgrgid(statbuf->st_gid)) == NULL)
                      {
-                     sprintf(OUTPUT,"File %s is not owned by any group in group database\n",file);
+                     snprintf(OUTPUT,bufsize*2,"File %s is not owned by any group in group database\n",file);
 		     CfLog(cferror,OUTPUT,"");
                      break;
                      }
 
-                  sprintf(OUTPUT,"File %s is owned by [%s], group [%s]\n",file,pw->pw_name,gp->gr_name);
+                  snprintf(OUTPUT,bufsize*2,"File %s is owned by [%s], group [%s]\n",file,pw->pw_name,gp->gr_name);
 		  CfLog(cferror,OUTPUT,"");
                   break;
       }
@@ -992,7 +988,7 @@ int CheckHomeSubDir(testpath,tidypath,recurse)
 char *testpath, *tidypath;
 int recurse;
 
-{ char *sp, *subdirstart, *sp1, *sp2;
+{ char *subdirstart, *sp1, *sp2;
   char buffer[bufsize];
   int homelen;
 
@@ -1154,14 +1150,14 @@ if (strlen(COMPRESSCOMMAND) == 0)
    return;
    }
  
-sprintf(comm,"%.512s %.512s",COMPRESSCOMMAND,file); 
+snprintf(comm,bufsize,"%s %s",COMPRESSCOMMAND,file); 
 
-sprintf(OUTPUT,"Compressing file %s",file); 
+snprintf(OUTPUT,bufsize*2,"Compressing file %s",file); 
 CfLog(cfinform,OUTPUT,"");
  
 if ((pp=cfpopen(comm,"r")) == NULL)
    {
-   sprintf(OUTPUT,"Compression command failed on %s",file);
+   snprintf(OUTPUT,bufsize*2,"Compression command failed on %s",file);
    CfLog(cfverbose,OUTPUT,"");
    }
 
