@@ -180,7 +180,7 @@ else
 		   {
 		   }
 		
-		sscanf(sp+1,"%.256[^)]",VBUFF);
+		sscanf(sp+1,"%256[^)]",VBUFF);
 		if (!RegexOK(VBUFF))
 		   {
 		   yyerror("Regular expression error");
@@ -230,7 +230,7 @@ for (fp = VFILTERLIST; fp != NULL; fp = fp->next)
    {
    if (fp->criteria[filterresult] == NULL)
        {
-       sprintf(OUTPUT,"No result specified for filter %s",fp->alias);
+       snprintf(OUTPUT,bufsize*2,"No result specified for filter %s",fp->alias);
        CfLog(cferror,OUTPUT,"");
        FatalError("Consistency error");
        }
@@ -238,7 +238,7 @@ for (fp = VFILTERLIST; fp != NULL; fp = fp->next)
    if ((fp->criteria[filterfromctime]!=NULL && fp->criteria[filtertoctime]==NULL) ||
        (fp->criteria[filterfromctime]==NULL && fp->criteria[filtertoctime]!=NULL))
       {
-      sprintf(OUTPUT,"Incomplete ctime limit specification of filter %s",fp->alias);
+      snprintf(OUTPUT,bufsize*2,"Incomplete ctime limit specification of filter %s",fp->alias);
       CfLog(cferror,OUTPUT,"");
       FatalError("Consistency errors");
       }
@@ -246,7 +246,7 @@ for (fp = VFILTERLIST; fp != NULL; fp = fp->next)
    if ((fp->criteria[filterfromatime]!=NULL && fp->criteria[filtertoatime]==NULL) ||
        (fp->criteria[filterfromatime]==NULL && fp->criteria[filtertoatime]!=NULL))
       {
-      sprintf(OUTPUT,"Incomplete atime limit specification of filter %s",fp->alias);
+      snprintf(OUTPUT,bufsize*2,"Incomplete atime limit specification of filter %s",fp->alias);
       CfLog(cferror,OUTPUT,"");
       FatalError("Consistency errors");
       }
@@ -254,7 +254,7 @@ for (fp = VFILTERLIST; fp != NULL; fp = fp->next)
    if ((fp->criteria[filterfrommtime]!=NULL && fp->criteria[filtertomtime]==NULL) ||
        (fp->criteria[filterfrommtime]==NULL && fp->criteria[filtertomtime]!=NULL))
       {
-      sprintf(OUTPUT,"Incomplete mtime limit specification of filter %s",fp->alias);
+      snprintf(OUTPUT,bufsize*2,"Incomplete mtime limit specification of filter %s",fp->alias);
       CfLog(cferror,OUTPUT,"");
       FatalError("Consistency errors");
       }
@@ -262,7 +262,7 @@ for (fp = VFILTERLIST; fp != NULL; fp = fp->next)
    if ((fp->criteria[filterfromstime]!=NULL && fp->criteria[filtertostime]==NULL) ||
        (fp->criteria[filterfromstime]==NULL && fp->criteria[filtertostime]!=NULL))
       {
-      sprintf(OUTPUT,"Incomplete stime limit specification of filter %s",fp->alias);
+      snprintf(OUTPUT,bufsize*2,"Incomplete stime limit specification of filter %s",fp->alias);
       CfLog(cferror,OUTPUT,"");
       FatalError("Consistency errors");
       }
@@ -270,7 +270,7 @@ for (fp = VFILTERLIST; fp != NULL; fp = fp->next)
    if ((fp->criteria[filterfromttime]!=NULL && fp->criteria[filtertottime]==NULL) ||
        (fp->criteria[filterfromttime]==NULL && fp->criteria[filtertottime]!=NULL))
       {
-      sprintf(OUTPUT,"Incomplete ttime limit specification of filter %s",fp->alias);
+      snprintf(OUTPUT,bufsize*2,"Incomplete ttime limit specification of filter %s",fp->alias);
       CfLog(cferror,OUTPUT,"");
       FatalError("Consistency errors");
       }
@@ -279,7 +279,7 @@ for (fp = VFILTERLIST; fp != NULL; fp = fp->next)
    if ((fp->criteria[filterfromsize]!=NULL && fp->criteria[filtertosize]==NULL) ||
        (fp->criteria[filterfromsize]==NULL && fp->criteria[filtertosize]!=NULL))
       {
-      sprintf(OUTPUT,"Incomplete size limit specification of filter %s",fp->alias);
+      snprintf(OUTPUT,bufsize*2,"Incomplete size limit specification of filter %s",fp->alias);
       CfLog(cferror,OUTPUT,"");
       FatalError("Consistency errors");
       }
@@ -288,7 +288,7 @@ for (fp = VFILTERLIST; fp != NULL; fp = fp->next)
       {
       if (Date2Number(fp->criteria[filterfromatime],t) > Date2Number(fp->criteria[filtertoatime],t))
 	 {
-	 sprintf(OUTPUT,"To/From atimes silly in filter %s (from > to)",fp->alias);
+	 snprintf(OUTPUT,bufsize*2,"To/From atimes silly in filter %s (from > to)",fp->alias);
 	 CfLog(cferror,OUTPUT,"");
 	 FatalError("Consistency errors");
 	 }
@@ -298,7 +298,7 @@ for (fp = VFILTERLIST; fp != NULL; fp = fp->next)
       {
       if (Date2Number(fp->criteria[filterfromctime],t) > Date2Number(fp->criteria[filtertoctime],t))
 	 {
-	 sprintf(OUTPUT,"To/From ctimes silly in filter %s (from > to)",fp->alias);
+	 snprintf(OUTPUT,bufsize*2,"To/From ctimes silly in filter %s (from > to)",fp->alias);
 	 CfLog(cferror,OUTPUT,"");
 	 FatalError("Consistency errors");
 	 }
@@ -308,7 +308,7 @@ for (fp = VFILTERLIST; fp != NULL; fp = fp->next)
       {
       if (Date2Number(fp->criteria[filterfrommtime],t) > Date2Number(fp->criteria[filtertomtime],t))
 	 {
-	 sprintf(OUTPUT,"To/From mtimes silly in filter %s (from > to)",fp->alias);
+	 snprintf(OUTPUT,bufsize*2,"To/From mtimes silly in filter %s (from > to)",fp->alias);
 	 CfLog(cferror,OUTPUT,"");
 	 FatalError("Consistency errors");
 	 }
@@ -318,7 +318,7 @@ for (fp = VFILTERLIST; fp != NULL; fp = fp->next)
       {
       if (Date2Number(fp->criteria[filterfromstime],t) > Date2Number(fp->criteria[filtertostime],t))
 	 {
-	 sprintf(OUTPUT,"To/From stimes silly in filter %s (from > to)",fp->alias);
+	 snprintf(OUTPUT,bufsize*2,"To/From stimes silly in filter %s (from > to)",fp->alias);
 	 CfLog(cferror,OUTPUT,"");
 	 FatalError("Consistency errors");
 	 }
@@ -328,7 +328,7 @@ for (fp = VFILTERLIST; fp != NULL; fp = fp->next)
       {
       if (Date2Number(fp->criteria[filterfromttime],t) > Date2Number(fp->criteria[filtertottime],t))
 	 {
-	 sprintf(OUTPUT,"To/From ttimes silly in filter %s (from > to)",fp->alias);
+	 snprintf(OUTPUT,bufsize*2,"To/From ttimes silly in filter %s (from > to)",fp->alias);
 	 CfLog(cferror,OUTPUT,"");
 	 FatalError("Consistency errors");
 	 }
@@ -346,7 +346,7 @@ for (fp = VFILTERLIST; fp != NULL; fp = fp->next)
       {
       if (strcmp(fp->criteria[filterfromsize],fp->criteria[filtertosize]) > 0)
 	 {
-	 sprintf(OUTPUT,"To/From size is silly in filter %s (from > to)",fp->alias);
+	 snprintf(OUTPUT,bufsize*2,"To/From size is silly in filter %s (from > to)",fp->alias);
 	 CfLog(cferror,OUTPUT,"");
 	 FatalError("Consistency errors");
 	 }
@@ -416,9 +416,8 @@ time_t now;
     
   */
 
-{ char type[16],datestr[63],monthtxt[4];
+{ char type[16],datestr[63];
   int year=-1,month=-1,day=-1,hr=-1,min=-1,sec=-1; 
-  int nyear=-1,nmonth=-1,nday=-1,nhr=-1,nmin=-1,nsec=-1;
   time_t time;
   struct tm tmv;
   
@@ -569,7 +568,7 @@ struct Item *filterlist;
 { struct Item *tests = NULL;
   struct Filter *fp;
   int result = true, tmpres, i;
-  char *line[noproccols],buffer[bufsize];
+  char *line[noproccols];
   
 Debug("ProcessFilter(%.6s...)\n",proc);
 
@@ -578,6 +577,11 @@ if (filterlist == NULL)
    return true;
    }
 
+if (strlen(proc) == 0)
+   {
+   return false;
+   }
+ 
 SplitLine(proc,filterlist,names,start,end,line);
 
 for (fp = VFILTERLIST; fp != NULL; fp=fp->next)
@@ -588,7 +592,7 @@ for (fp = VFILTERLIST; fp != NULL; fp=fp->next)
 
       if (fp->criteria[filterresult] == NULL)
 	 {
-	 fp->criteria[filterresult] = strdup("PID.PPID.PGID.RSize.VSize.Status.Command.TTime.STime.TTY.Priority.Threads");
+	 fp->criteria[filterresult] = strdup("Owner.PID.PPID.PGID.RSize.VSize.Status.Command.TTime.STime.TTY.Priority.Threads");
 	 }
       
       DoProc(&tests,fp->criteria,names,line);
@@ -628,8 +632,7 @@ struct Item *filterlist;
 enum actions context;
 
 { struct Item *tests = NULL;
-  struct Filter *fp;
-  int result = true, tmpres;
+  struct Filter *fp;  int result = true, tmpres;
   
 Debug("FileObjectFilter(%s)\n",file);
  
@@ -775,7 +778,19 @@ struct Item **attr;
 char **crit,**names,**line;
 
 {
-if (crit[filterpid] != NULL)
+if (crit[filterowner] != NULL)
+   {
+   if (FilterProcMatch("UID","UID",crit[filterowner],names,line))
+      {
+      PrependItem(attr,"Owner","");
+      }
+   }
+else
+   {
+   PrependItem(attr,"Owner","");
+   }
+
+ if (crit[filterpid] != NULL)
    {
    if (FilterProcMatch("PID","PID",crit[filterpid],names,line))
       {
@@ -1053,12 +1068,17 @@ for (i = 0; names[i] != NULL; i++)
       if (strstr(line[i],":")) /* Hr:Min:Sec */
 	 {
          sscanf(line[i],"%2[^:]:%2[^:]:",hr,min);
-	 sprintf(timestr,"date(%s,%d,%s,%s,%s,0)",year,Month2Number(month),day,hr,min);
+	 snprintf(timestr,256,"date(%s,%d,%s,%s,%s,0)",year,Month2Number(month),day,hr,min);
 	 }
       else                     /* date Month */
 	 {
          sscanf(line[i],"%3[a-zA-Z] %2[0-9]",month,day);
-	 sprintf(timestr,"date(%s,%d,%s,%s,%s,0)",year,Month2Number(month),day,hr,min);
+	 snprintf(timestr,256,"date(%s,%d,%s,%s,%s,0)",year,Month2Number(month),day,hr,min);
+	 }
+      
+      if (Month2Number(month) < 0)
+	 {
+	 continue;
 	 }
       
       pstime = Date2Number(timestr,now);
@@ -1079,7 +1099,7 @@ char *name1,*name2,*fromexpr,*toexpr;
 char **names,**line;
 
 { int i,hr=0,min=-1,sec=-1;
-  time_t fromtime,totime,now = CFSTARTTIME,pstime;
+  time_t fromtime,totime,now = CFSTARTTIME,pstime = CFSTARTTIME;
   char timestr[256];
 
 fromtime = Date2Number(fromexpr,now);
@@ -1099,7 +1119,7 @@ for (i = 0; names[i] != NULL; i++)
 	    return false;
 	    }
 	 
-	 sprintf(timestr,"accumulated(0,0,0,%d,%d,%d)",hr,min,sec);
+	 snprintf(timestr,256,"accumulated(0,0,0,%d,%d,%d)",hr,min,sec);
 	 pstime = Date2Number(timestr,now);
 	 }
 
@@ -1186,8 +1206,7 @@ int FilterModeMatch(lstatptr,crit)
 char *crit;
 struct stat *lstatptr;
 
-{ struct Item *attrib = NULL;
-  mode_t plusmask,minusmask;
+{ mode_t plusmask,minusmask;
 
 ParseModeString(crit,&plusmask,&minusmask);
 
@@ -1201,8 +1220,7 @@ int FilterTimeMatch(lstatptr,from,to)
 char *from,*to;
 struct stat *lstatptr;
 
-{ struct Item *attrib = NULL;
- time_t fromtime,totime, now = CFSTARTTIME;
+{ time_t fromtime,totime, now = CFSTARTTIME;
 
 fromtime = Date2Number(from,now);
 totime = Date2Number(to,now);
@@ -1261,7 +1279,7 @@ for (sp = VBUFF+strlen(VBUFF)-1; (*sp != '(') && (sp > VBUFF); sp--)
    {
    }
  
-sscanf(sp+1,"%.256[^)]",expr);
+sscanf(sp+1,"%256[^)]",expr);
 strncpy(buffer,VBUFF,(int)(sp-VBUFF));
  
 if (CfRegcomp(&rx,expr,REG_EXTENDED) != 0)
@@ -1271,7 +1289,7 @@ if (CfRegcomp(&rx,expr,REG_EXTENDED) != 0)
 
 if ((pp = cfpopen(buffer,"r")) == NULL)
    {
-   sprintf(OUTPUT,"Couldn't open pipe to command %s\n",buffer);
+   snprintf(OUTPUT,bufsize*2,"Couldn't open pipe to command %s\n",buffer);
    CfLog(cferror,OUTPUT,"cfpopen");
    return false;
    }
@@ -1312,7 +1330,7 @@ bzero(buffer,bufsize);
  
 if (readlink(filename,buffer,bufsize-1) == -1)
    {
-   sprintf(OUTPUT,"Unable to read link %s in filter",filename);
+   snprintf(OUTPUT,bufsize*2,"Unable to read link %s in filter",filename);
    CfLog(cferror,OUTPUT,"readlink");
    regfree(&rx);
    return false;      
@@ -1365,7 +1383,7 @@ char *proc, **names;
 int *start,*end;
 
 { char *sp,title[16];
-  int col,offset;
+  int col,offset = 0;
   
 for (col = 0; col < noproccols; col++)
    {
@@ -1379,10 +1397,11 @@ for (sp = proc; *sp != '\0'; sp++)
    {
    offset = sp - proc;
 
-   if (isspace(*sp))
+   if (isspace((int)*sp))
       {
       if (start[col] != -1)
 	 {
+	 Debug("End of %s is %d\n",title,offset-1);
 	 end[col++] = offset - 1;
 	 if (col > noproccols - 1)
 	    {
@@ -1393,9 +1412,10 @@ for (sp = proc; *sp != '\0'; sp++)
       continue;
       }
 
-   if (start[col] == -1)
+   else if (start[col] == -1)
       {
       start[col] = offset;
+      Debug("Start of %s is %d\n",title,offset);
       sscanf(sp,"%15s",title);
       names[col] = strdup(title);
       Debug("Col[%d]=%s\n",col,names[col]);
@@ -1404,6 +1424,7 @@ for (sp = proc; *sp != '\0'; sp++)
 
 if (end[col] == -1)
    {
+   Debug("End of %s is %d\n",title,offset);
    end[col] = offset;
    }
 }
@@ -1416,7 +1437,7 @@ char *proc, **names, **line;
 int *start,*end;
 struct Item *filterlist;
 
-{ int i,s,e,c;
+{ int i,s,e;
 
 for (i = 0; i < noproccols; i++)
    {
@@ -1428,7 +1449,7 @@ for (i = 0; names[i] != NULL; i++)
    s = start[i];
    e = end[i];
 
-   for (s = start[i]; (s >= 0) && !isspace(*(proc+s)); s--)
+   for (s = start[i]; (s >= 0) && !isspace((int)*(proc+s)); s--)
       {
       }
 
@@ -1437,7 +1458,7 @@ for (i = 0; names[i] != NULL; i++)
       s = 0;
       }
    
-   if (isspace(proc[s]))
+   if (isspace((int)proc[s]))
       {
       s++;
       }
@@ -1448,13 +1469,16 @@ for (i = 0; names[i] != NULL; i++)
       }
    else
       {
-      for (e = end[i]; (e <= end[i]+10) && !isspace(*(proc+e)); e++)
+      for (e = end[i]; (e <= end[i]+10) && !isspace((int)*(proc+e)); e++)
 	 {
 	 }
       
-      if (isspace(proc[e]))
+      if (isspace((int)proc[e]))
 	 {
-	 e--;
+	 if (e > 0)
+	    {
+	    e--;
+	    }
 	 }
       }
 
