@@ -34,6 +34,7 @@
 /*                                                                 */
 /*******************************************************************/
 
+#include "../pub/getopt.h"
 #include "cf.defs.h"
 
 #define PUBLIC     /* Just for fun */
@@ -55,15 +56,11 @@ int CHECKSUMUPDATES = false;
 char *CHECKSUMDB;
 char PADCHAR = ' ';
 
-/*******************************************************************/
-/*                                                                 */
-/* TCP wrapper object. This is not part of cfengine, but these     */
-/* variables have to be defined anyway!                            */
-/*                                                                 */
-/*******************************************************************/
+char CFPUBKEYFILE[bufsize];
+char CFPRIVKEYFILE[bufsize];
+char AVDB[1024];
 
-  PRIVATE int allow_severity  = 0;
-  PRIVATE int deny_severity   = 0;
+RSA *PRIVKEY = NULL, *PUBKEY = NULL;
 
 /*******************************************************************/
 /*                                                                 */
@@ -84,13 +81,14 @@ char PADCHAR = ' ';
 /*                                                                 */
 /*******************************************************************/
 
-  PUBLIC char *COPYRIGHT = "Free Software Foundation 1995-2000\nDonated by Mark Burgess, Faculty of Engineering,\nOslo University College, 0254 Oslo, Norway";
+  PUBLIC char *COPYRIGHT = "Free Software Foundation 1994-2001\nDonated by Mark Burgess, Faculty of Engineering,\nOslo University College, 0254 Oslo, Norway";
 
   PRIVATE char *VPRECONFIG = "cf.preconf";
   PRIVATE char *VRCFILE = "cfrc";
 
   PUBLIC char *VSETUIDLOG = NULL;
   PUBLIC char *VARCH = NULL;
+  PUBLIC char *VARCH2 = NULL;
   PUBLIC char *VREPOSITORY = NULL;
   PUBLIC char *COMPRESSCOMMAND = NULL;
 
@@ -100,8 +98,8 @@ char PADCHAR = ' ';
   PUBLIC char VCURRENTFILE[bufsize];
   PUBLIC char VLOGFILE[bufsize];
   PUBLIC char ALLCLASSBUFFER[bufsize];
-  PUBLIC char CHROOT[bufsize];
   PUBLIC char ELSECLASSBUFFER[bufsize];
+  PUBLIC char CHROOT[bufsize];
 
   PUBLIC short DEBUG = false;
   PUBLIC short D1 = false;
@@ -109,11 +107,14 @@ char PADCHAR = ' ';
   PUBLIC short D3 = false;
   PUBLIC short VERBOSE = false;
   PUBLIC short INFORM = false;
+  PUBLIC short EXCLAIM = true;
+  PUBLIC short COMPATIBILITY_MODE = false;
   PUBLIC short LOGGING = false;
   PUBLIC short INFORM_save;
   PUBLIC short LOGGING_save;
   PUBLIC short CFPARANOID = false;
   PUBLIC short SHOWACTIONS = false;
+  PUBLIC short LOGTIDYHOMEFILES = true;
 
   PUBLIC char FORK = 'n';
 
@@ -122,11 +123,14 @@ char PADCHAR = ' ';
   PROTECTED int SENSIBLEFSSIZE = 1000;
 
   PUBLIC time_t CFSTARTTIME;
+  PUBLIC time_t CFINITSTARTTIME;
 
   PUBLIC enum classes VSYSTEMHARDCLASS;
 
   PUBLIC struct Item VDEFAULTBINSERVER =      /* see GetNameInfo(), main.c */
       {
+      'n',
+      NULL,
       NULL,
       NULL
       };
@@ -136,12 +140,12 @@ char PADCHAR = ' ';
   PUBLIC mode_t DEFAULTMODE = (mode_t) 0755;
   PUBLIC mode_t DEFAULTSYSTEMMODE = (mode_t) 0644;
 
-  PROTECTED int VIFELAPSED = 0;
+  PROTECTED int VIFELAPSED = 1;
   PROTECTED int VEXPIREAFTER = 120;
-  PROTECTED int VDEFAULTIFELAPSED = 0;    /* minutes */
-  PROTECTED int VDEFAULTEXPIREAFTER = 120;
+  PROTECTED int VDEFAULTIFELAPSED = 1;     
+  PROTECTED int VDEFAULTEXPIREAFTER = 120; /* minutes */
 
-  PUBLIC int SD = cf_not_connected; /* socket */
+  PUBLIC struct cfagent_connection *CONN = NULL;
 
 /*******************************************************************/
  /* Data structures - root pointers                                 */
@@ -194,8 +198,12 @@ char PADCHAR = ' ';
   PROTECTED  struct Item *VHOMEPATLIST = NULL;
   PROTECTED  struct Item *EXTENSIONLIST = NULL;
   PROTECTED  struct Item *SUSPICIOUSLIST = NULL;
+  PROTECTED  struct Item *SCHEDULE = NULL;
+  PROTECTED  struct Item *SPOOLDIRLIST = NULL;
   PROTECTED  struct Item *NONATTACKERLIST = NULL;
   PROTECTED  struct Item *MULTICONNLIST = NULL;
+  PROTECTED  struct Item *TRUSTKEYLIST = NULL;
+  PROTECTED  struct Item *ALLOWUSERLIST = NULL;
   PROTECTED  struct Item *SKIPVERIFY = NULL;
   PROTECTED  struct Item *ATTACKERLIST = NULL;
   PROTECTED  struct Item *MOUNTOPTLIST = NULL;
@@ -213,6 +221,9 @@ char PADCHAR = ' ';
   PROTECTED  struct Filter *VFILTERLISTTOP=NULL;
   PROTECTED  struct CFACL  *VACLLIST=NULL;
   PROTECTED  struct CFACL  *VACLLISTTOP=NULL;
+  PROTECTED  struct Strategy *VSTRATEGYLIST=NULL;
+  PROTECTED  struct Strategy *VSTRATEGYLISTTOP=NULL;
+
   PROTECTED  struct Item *VCLASSDEFINE=NULL;
   PROTECTED  struct Process *VPROCLIST=NULL;
   PROTECTED  struct Process *VPROCTOP=NULL;
@@ -270,7 +281,6 @@ char PADCHAR = ' ';
 
   /* GNU STUFF FOR LATER #include "getopt.h" */
  
-#include "../pub/getopt.h"
  
  PRIVATE struct option OPTIONS[] =
       {
@@ -314,6 +324,7 @@ char PADCHAR = ' ';
       { "no-modules",no_argument,0,'M'},
       { "force-net-copy",no_argument,0,'b'},
       { "secure-input",no_argument,0,'Y'},
+      { "zone-info",no_argument,0,'z'},
       { NULL,0,0,0 }
       };
 
@@ -337,6 +348,7 @@ char PADCHAR = ' ';
       "Binary Servers:",
       "Mail Server:",
       "Required Filesystems",
+      "Disks (Required)",
       "Reading Mountables",
       "Links:",
       "Import files:",
@@ -352,8 +364,9 @@ char PADCHAR = ' ';
       "Admit network access:",
       "Deny network access:",
       "Access control lists:",
-      "Additional network interfaces",
-      "Search filter objects",
+      "Additional network interfaces:",
+      "Search filter objects:",
+      "Strategies:",
       NULL
       };
 
@@ -372,12 +385,13 @@ char PADCHAR = ' ';
       "binservers",
       "mailserver",
       "required",
+      "disks",
       "mountables",
       "links",
       "import",
       "shellcommands",
       "disable",
-      "makepath",
+      "directories",
       "ignore",
       "broadcast",
       "defaultroute",
@@ -389,6 +403,23 @@ char PADCHAR = ' ';
       "acl",
       "interfaces",
       "filters",
+      "strategies",
+      NULL
+      };
+
+ PRIVATE char *BUILTINS[] =    /* The actions which may be specified as indexed */
+      {
+      "",
+      "randomint",
+      "isnewerthan",
+      "accessedbefore",
+      "changedbefore",
+      "fileexists",
+      "isdir",
+      "islink",
+      "isplain",
+      "execresult",
+      "returnszero",
       NULL
       };
 
@@ -418,10 +449,9 @@ char PADCHAR = ' ';
       {
       "directories",
       "links",
-      "simplelinks",
-      "childlinks",
       "mailcheck",
       "required",
+      "disks",
       "tidy",
       "shellcommands",
       "files",
@@ -465,6 +495,7 @@ char PADCHAR = ' ';
   PRIVATE short NOTIDY = false;
   PRIVATE short NOSCRIPTS = false;
   PRIVATE short PRSYSADM = false;
+  PRIVATE short PRMAILSERVER = false;
   PRIVATE short MOUNTCHECK = false;
   PRIVATE short NOPROCS = false;
   PRIVATE short NOEDITS = false;
@@ -476,14 +507,12 @@ char PADCHAR = ' ';
   PRIVATE short MINUSF = false;
   PRIVATE short NOLINKS = false;
   PRIVATE short ENFORCELINKS = false;
-  PRIVATE short FORCELINK;
   PRIVATE short NOCOPY = false;
   PRIVATE short FORCENETCOPY = false;
   PRIVATE short SILENT=false;
   PRIVATE short EDITVERBOSE=false;
   PRIVATE short LINKSILENT;
-  PRIVATE short FORCECOPY=false;
-  PRIVATE short IMAGEBACKUP=true;
+
   PRIVATE short ROTATE=0;
   PRIVATE short USEENVIRON=false;
   PRIVATE short PROMATCHES=-1;
@@ -494,9 +523,12 @@ char PADCHAR = ' ';
 
   PROTECTED  struct Item *VACLBUILD = NULL;
   PROTECTED  struct Item *VFILTERBUILD = NULL;
+  PROTECTED  struct Item *VSTRATEGYBUILD = NULL;
 
-  PRIVATE char PRESERVETIMES = 'f';
-  PRIVATE char TYPECHECK = 't';
+  PRIVATE char IMAGEBACKUP='y';
+  PRIVATE char TRUSTKEY = 'n';
+  PRIVATE char PRESERVETIMES = 'n';
+  PRIVATE char TYPECHECK = 'y';
   PRIVATE char LINKTYPE = 's';
   PRIVATE char AGETYPE = 'a';
   PRIVATE char COPYTYPE = 't';
@@ -505,39 +537,55 @@ char PADCHAR = ' ';
   PRIVATE char LISTSEPARATOR = ':';
   PRIVATE char LINKDIRS = 'k';
   PRIVATE char DISCOMP = '=';
-  PRIVATE char USESHELL = 'y';  /* yes or no */
+  PRIVATE char USESHELL = 'y';  /* yes or no or dumb */
   PRIVATE char PREVIEW = 'n';  /* yes or no */
   PRIVATE char PURGE = 'n';
   PRIVATE char LOGP = 'd';  /* y,n,d=default*/
   PRIVATE char INFORMP = 'd';
   PRIVATE char MOUNTMODE = 'w';   /* o or w for rw/ro*/
-  PRIVATE char DELETEDIR = 't';   /* t=true */
-  PRIVATE char DELETEFSTAB = 't';
-  PRIVATE char FORCE = 'f';
-  PRIVATE char STEALTH = 'f';
-  PRIVATE char CHECKSUM = 'n';
-  PRIVATE char COMPRESS = 'f';
+  PRIVATE char DELETEDIR = 'y';   /* t=true */
+  PRIVATE char DELETEFSTAB = 'y';
+  PRIVATE char FORCE = 'n';
+  PRIVATE char FORCEIPV4 = 'n';
+  PRIVATE char FORCELINK = 'n';
+  PRIVATE char FORCEDIRS = 'n';
+  PRIVATE char STEALTH = 'n';
+  PRIVATE char CHECKSUM = 'n'; /* n,m,s */
+  PRIVATE char COMPRESS = 'n';
 
-  PRIVATE char VUIDNAME[bufsize];
-  PRIVATE char VGIDNAME[bufsize];
-  PRIVATE char FILTERNAME[bufsize];
-  PRIVATE char CURRENTITEM[bufsize];  /* Many of these are recycled during run/parsing */
-  PRIVATE char GROUPBUFF[bufsize];
-  PRIVATE char ACTIONBUFF[bufsize];
-  PRIVATE char CURRENTPATH[bufsize];
-  PRIVATE char CLASSBUFF[bufsize];
-  PRIVATE char LINKFROM[bufsize];
-  PRIVATE char LINKTO[bufsize];
-  PRIVATE char ERROR[bufsize];
-  PRIVATE char MOUNTFROM[bufsize];
-  PRIVATE char MOUNTONTO[bufsize];
-  PRIVATE char MOUNTOPTS[bufsize];
-  PRIVATE char DESTINATION[bufsize];
-  PRIVATE char IMAGEACTION[bufsize];  /* end recyclable (see cf.defs.h) */
-  PRIVATE char VIFNAME[16];
-  PRIVATE char VIFNAMEOVERRIDE[16];
-  PRIVATE char CHDIR[bufsize];
-  PRIVATE char LOCALREPOS[bufsize];
+  PRIVATE char *VUIDNAME;
+  PRIVATE char *VGIDNAME;
+  PRIVATE char *FILTERNAME;
+  PRIVATE char *STRATEGYNAME;
+  PRIVATE char *CURRENTITEM;
+  PRIVATE char *GROUPBUFF;
+  PRIVATE char *ACTIONBUFF;
+  PRIVATE char *CURRENTPATH;
+  PRIVATE char *CLASSBUFF;
+  PRIVATE char *LINKFROM;
+  PRIVATE char *LINKTO;
+  PRIVATE char *ERROR;
+  PRIVATE char *MOUNTFROM;
+  PRIVATE char *MOUNTONTO;
+  PRIVATE char *MOUNTOPTS;
+  PRIVATE char *DESTINATION;
+  PRIVATE char *IMAGEACTION;
+  PRIVATE char *VIFNAME[16];
+  PRIVATE char *VIFNAMEOVERRIDE[16];
+  PRIVATE char *CHDIR;
+  PRIVATE char *LOCALREPOS;
+  PRIVATE char *EXPR;
+  PRIVATE char *CURRENTAUTHPATH;
+  PRIVATE char *RESTART;
+  PRIVATE char *FILTERDATA;
+  PRIVATE char *STRATEGYDATA;
+
+  PRIVATE short PROSIGNAL;
+  PRIVATE char  PROACTION;
+
+  PRIVATE char PROCOMP;
+  PRIVATE char IMGCOMP;
+  PRIVATE int IMGSIZE;
 
   PUBLIC int ERRORCOUNT = 0;
   PUBLIC int LINENUMBER = 1;
@@ -569,7 +617,9 @@ char PADCHAR = ' ';
   PRIVATE flag HAVE_RESTART = false;
   PRIVATE flag ACTIONPENDING = false;
   PRIVATE flag HOMECOPY = false;
-  PRIVATE short SECURE = false;
+  PRIVATE char ENCRYPT = 'n';
+  PRIVATE char VERIFY = 'n';
+  PRIVATE char COMPATIBILITY = 'n';
 
   PRIVATE char *COMMATTRIBUTES[] =
      {
@@ -584,6 +634,8 @@ char PADCHAR = ' ';
      "type",
      "destination",
      "force",
+     "forcedirs",
+     "forceipv4",
      "backup",
      "rotate",
      "size",
@@ -616,7 +668,8 @@ char PADCHAR = ' ';
      "stealth",
      "checksum",
      "flags",
-     "secure",
+     "encrypt",
+     "verify",
      "root",
      "typecheck",
      "umask",
@@ -628,6 +681,8 @@ char PADCHAR = ' ';
      "preview",
      "repository",
      "timestamps",
+     "trustkey",
+     "oldserver",
      NULL
      };
 
@@ -681,7 +736,7 @@ char PADCHAR = ' ';
   PRIVATE char VEDITABORT[maxlinksize];
 
   PUBLIC int EDITFILESIZE = 10000;
-  PUBLIC int EDITBINFILESIZE = 1000000;
+  PUBLIC int EDITBINFILESIZE = 10000000;
 
   PRIVATE int NUMBEROFEDITS = 0;
   PRIVATE int CURRENTLINENUMBER = 1;           /* current line number in file */
@@ -769,6 +824,8 @@ char PADCHAR = ' ';
      "BeginGroupIfFileIsNewer",
      "BeginGroupIfFileExists",
      "BeginGroupIfNoLineContaining",
+     "BeginGroupIfDefined",
+     "BeginGroupIfNotDefined",
      "AutoCreate",
      "ForEachLineIn",
      "EndLoop",
@@ -791,6 +848,8 @@ char PADCHAR = ' ';
      "Include",
      "Repository",
      "Umask",
+     "UseShell",
+     "Filter",
      NULL
      };
 
@@ -852,11 +911,28 @@ char PADCHAR = ' ';
   PRIVATE char VIPADDRESS[18];
   PRIVATE int  CF_TIMEOUT = 10;
 
-  PRIVATE int  CFSIGNATURE;
+  PRIVATE int  CFSIGNATURE = 0;
   PRIVATE char CFDES1[9];
   PRIVATE char CFDES2[9];
   PRIVATE char CFDES3[9];
 
+  PUBLIC char *PROTOCOL[] =
+   {
+   "EXEC",
+   "AUTH",  /* old protocol */
+   "GET",
+   "OPENDIR",
+   "SYNCH",
+   "CLASSES",
+   "MD5",
+   "SMD5",
+   "CAUTH",
+   "SAUTH",
+   "SSYNCH",
+   "SGET",
+   NULL
+   };
+
 /*******************************************************************/
 /*                                                                 */
 /* Adaptive lock object : Lock                                     */
@@ -869,7 +945,7 @@ char PADCHAR = ' ';
 
   PUBLIC char *VCANONICALFILE;
 
-  PUBLIC FILE *VLOGFP;
+  PUBLIC FILE *VLOGFP = NULL;
 
 /* EOF */
 
