@@ -65,7 +65,6 @@
 /*                                                                 */
 /*******************************************************************/
 
-#define INET
 #include "cf.defs.h"
 #include "cf.extern.h"
 
@@ -231,7 +230,7 @@ GetBroadcastAddr(inet_ntoa(inaddr),vifdev,vnetmask,vbroadcast);
 
 if (strcmp(VBUFF,VNUMBROADCAST) != 0)
    {
-   sprintf(OUTPUT,"Broadcast address was %s not %s (should be bit-type %s)\n",VBUFF,VNUMBROADCAST,vbroadcast);
+   snprintf(OUTPUT,bufsize*2,"Broadcast address was %s not %s (should be bit-type %s)\n",VBUFF,VNUMBROADCAST,vbroadcast);
    CfLog(cferror,OUTPUT,"");
    insane = true;
    }
@@ -279,7 +278,7 @@ if (ioctl(sk,SIOCSIFNETMASK, (caddr_t) &IFR) < 0)
    }
 else
    {
-   sprintf(OUTPUT,"Set Netmask to: %s\n",inet_ntoa(netmask.sin_addr));
+   snprintf(OUTPUT,bufsize*2,"Set Netmask to: %s\n",inet_ntoa(netmask.sin_addr));
    CfLog(cfinform,OUTPUT,"");
    }
 
@@ -304,7 +303,7 @@ if ((char *) sin->sin_addr.s_addr == NULL)
    }
 else
    {
-   sprintf(OUTPUT,"Set Broadcast address to: %s\n",inet_ntoa(sin->sin_addr));
+   snprintf(OUTPUT,bufsize*2,"Set Broadcast address to: %s\n",inet_ntoa(sin->sin_addr));
    CfLog(cfinform,OUTPUT,"");
    }
 }
@@ -376,7 +375,7 @@ Verbose("Looking at default route...\n");
   
 if (!IsPrivileged())                            
    {
-   sprintf(OUTPUT,"Only root can set a default route.");
+   snprintf(OUTPUT,bufsize*2,"Only root can set a default route.");
    CfLog(cfinform,OUTPUT,"");
    return;
    }
@@ -389,7 +388,7 @@ if (VDEFAULTROUTE[0] == '\0')
 
 if ((pp = cfpopen(VNETSTAT[VSYSTEMHARDCLASS],"r")) == NULL)
    {
-   sprintf(OUTPUT,"Failed to open pipe from %s\n",VNETSTAT[VSYSTEMHARDCLASS]);
+   snprintf(OUTPUT,bufsize*2,"Failed to open pipe from %s\n",VNETSTAT[VSYSTEMHARDCLASS]);
    CfLog(cferror,OUTPUT,"popen");
    return;
    }
@@ -457,7 +456,7 @@ if (! DONTDO)
       }
 
    CfLog(cferror,"The routing table did not contain a default route.\n","");
-   sprintf(OUTPUT,"I'm setting it to %s\n",VDEFAULTROUTE);
+   snprintf(OUTPUT,bufsize*2,"I'm setting it to %s\n",VDEFAULTROUTE);
    CfLog(cferror,OUTPUT,"");
    }
 
