@@ -1,6 +1,6 @@
 /* 
 
-        Copyright (C) 1995
+        Copyright (C) 1995-
         Free Software Foundation, Inc.
 
    This file is part of GNU cfengine - written and maintained 
@@ -30,8 +30,6 @@
 /*                                                                 */
 /*******************************************************************/
 
-#define INET 1
-
 #include "cf.defs.h"
 #include "cf.extern.h"
 
@@ -77,16 +75,16 @@ if (strlen(from) == 0)     /* Check for root dir */
 strcpy(newto,to);
 AddSlash(newto);
 
-if (! MakeDirectoriesFor(newto))
+if (! MakeDirectoriesFor(newto,ip->forcedirs))
    {
-   sprintf(OUTPUT,"Unable to make directory for %s in copy: %s to %s\n",newto,ip->path,ip->destination);
+   snprintf(OUTPUT,bufsize*2,"Unable to make directory for %s in copy: %s to %s\n",newto,ip->path,ip->destination);
    CfLog(cferror,OUTPUT,"");
    return;
    }
 
 if ((dirh = cfopendir(from,ip)) == NULL)
    {
-   sprintf(OUTPUT,"copy can't open directory [%s]\n",from);
+   snprintf(OUTPUT,bufsize*2,"copy can't open directory [%s]\n",from);
    CfLog(cferror,OUTPUT,"");
    return;
    }
@@ -133,8 +131,6 @@ for (dirp = cfreaddir(dirh,ip); dirp != NULL; dirp = cfreaddir(dirh,ip))
 
    if (TRAVLINKS || ip->linktype == 'n')
       {
-      printf("DEBUG: TRAVLINKS=%d,ip->linktype=%c\n",TRAVLINKS,ip->linktype);
-
       if (cfstat(newfrom,&statbuf,ip) == -1)
          {
          Verbose("%s: (Can't stat %s)\n",VPREFIX,newfrom);
@@ -164,7 +160,7 @@ for (dirp = cfreaddir(dirh,ip); dirp != NULL; dirp = cfreaddir(dirh,ip))
       {
       if (TRAVLINKS || ip->linktype == 'n')
 	 {
-	 CfLog(cfinform,"Traversing directory links during copy is too dangerous, pruned","");
+	 CfLog(cfverbose,"Traversing directory links during copy is too dangerous, pruned","");
 	 continue;
 	 }
       
@@ -221,7 +217,7 @@ struct Image *ip;
 
 { DIR *dirh, *dirh2;
   struct dirent *dirp, *dirp2;
-  char *ReadLastNode(), username[maxvarsize];
+  char username[maxvarsize];
   char homedir[bufsize],dest[bufsize];
   struct passwd *pw;
   struct group *gr;
@@ -238,7 +234,7 @@ if (!MountPathDefined())
 
 if (cfstat(ip->path,&statbuf,ip) == -1)
    {
-   sprintf(OUTPUT,"Master file %s doesn't exist for copying\n",ip->path);
+   snprintf(OUTPUT,bufsize*2,"Master file %s doesn't exist for copying\n",ip->path);
    CfLog(cferror,OUTPUT,"");
    return;
    }
@@ -252,7 +248,7 @@ for (itp = VMOUNTLIST; itp != NULL; itp=itp->next)
    
    if ((dirh = opendir(itp->name)) == NULL)
       {
-      sprintf(OUTPUT,"Can't open directory %s\n",itp->name);
+      snprintf(OUTPUT,bufsize*2,"Can't open directory %s\n",itp->name);
       CfLog(cferror,OUTPUT,"opendir");
       return;
       }
@@ -275,7 +271,7 @@ for (itp = VMOUNTLIST; itp != NULL; itp=itp->next)
 
       if ((dirh2 = opendir(homedir)) == NULL)
          {
-	 sprintf(OUTPUT,"Can't open directory %s\n",homedir);
+	 snprintf(OUTPUT,bufsize*2,"Can't open directory %s\n",homedir);
 	 CfLog(cferror,OUTPUT,"opendir");
          return;
          }
@@ -297,7 +293,7 @@ for (itp = VMOUNTLIST; itp != NULL; itp=itp->next)
             AddSlash(dest);
 	    if (strlen(ip->destination) < 6)
 	       {
-	       sprintf(OUTPUT,"Error in home/copy to %s",ip->destination);
+	       snprintf(OUTPUT,bufsize*2,"Error in home/copy to %s",ip->destination);
 	       CfLog(cferror,OUTPUT,"");
 	       return;
 	       }
@@ -383,7 +379,7 @@ else
  
 if (found == -1)
    {
-   sprintf(OUTPUT,"Can't stat %s\n",source);
+   snprintf(OUTPUT,bufsize*2,"Can't stat %s\n",source);
    CfLog(cferror,OUTPUT,"");
    FlushClientCache(ip);
    return;
@@ -416,7 +412,7 @@ if (S_ISDIR(sourcestatbuf.st_mode))
 
    if ((dirh = cfopendir(sourcedir,ip)) == NULL)
       {
-      sprintf(OUTPUT,"%s: Can't open directory %s\n",VPREFIX,sourcedir);
+      snprintf(OUTPUT,bufsize*2,"%s: Can't open directory %s\n",VPREFIX,sourcedir);
       CfLog(cferror,OUTPUT,"opendir");
       FlushClientCache(ip);
       (ip->uid)->uid = save_uid;
@@ -495,6 +491,13 @@ char *directory;
 Debug("PurgeFiles(%s)\n",directory);
 
  /* If we purge with no authentication we wipe out EVERYTHING */ 
+
+ if (strlen(directory) < 2)
+    {
+    snprintf(OUTPUT,bufsize,"Purge of %s denied -- too dangerous!",directory);
+    CfLog(cferror,OUTPUT,"");
+    return;
+    }
  
  if (!AUTHENTICATED)
     {
@@ -504,7 +507,7 @@ Debug("PurgeFiles(%s)\n",directory);
 
  if ((dirh = opendir(directory)) == NULL)
     {
-    sprintf(OUTPUT,"Can't open directory %s\n",directory);
+    snprintf(OUTPUT,bufsize*2,"Can't open directory %s\n",directory);
     CfLog(cferror,OUTPUT,"cfopendir");
     return;
     }
@@ -521,6 +524,8 @@ Debug("PurgeFiles(%s)\n",directory);
        strcpy(filename,directory);
        AddSlash(filename);
        strcat(filename,dirp->d_name);
+
+       Debug("Checking purge %s..\n",filename);
        
        if (DONTDO)
 	  {
@@ -528,15 +533,12 @@ Debug("PurgeFiles(%s)\n",directory);
 	  }
        else
 	  {
-	  sprintf(OUTPUT,"Purging %s in copy dest directory\n",filename);
+	  snprintf(OUTPUT,bufsize*2,"Purging %s in copy dest directory\n",filename);
 	  CfLog(cfsilent,OUTPUT,"");
 	  
-          /*
-           * HvB: Use lstat instead of stat else the link will be followed!!
-           */
-	  if (lstat(filename,&statbuf) == -1)
+	  if (stat(filename,&statbuf) == -1)
 	     {
-	     sprintf(OUTPUT,"Couldn't stat %s while purging\n",filename);
+	     snprintf(OUTPUT,bufsize*2,"Couldn't stat %s while purging\n",filename);
 	     CfLog(cfverbose,OUTPUT,"stat");
 	     }
 
@@ -545,7 +547,7 @@ Debug("PurgeFiles(%s)\n",directory);
 	     struct Tidy tp;
 	     struct TidyPattern tpat;
 	     
-	     tp.recurse;
+	     tp.maxrecurse = 2;
 	     tp.tidylist = &tpat;
 	     tp.next = NULL;
 	     tp.path = filename;
@@ -557,20 +559,20 @@ Debug("PurgeFiles(%s)\n",directory);
 	     tpat.classes = strdup("any");
 	     tpat.defines = NULL;
 	     tpat.elsedef = NULL;
-	     tpat.dirlinks = 't';
-	     tpat.travlinks = 'f';
-	     tpat.rmdirs = 't';
+	     tpat.dirlinks = 'y';
+	     tpat.travlinks = 'n';
+	     tpat.rmdirs = 'y';
 	     tpat.searchtype = 'a';
 	     tpat.log = 'd';
 	     tpat.inform = 'd';
 	     tpat.next = NULL;
-	     RecursiveTidySpecialArea(filename,&tp,INFINITERECURSE,&statbuf);
+	     RecursiveTidySpecialArea(filename,&tp,INFINITERECURSE);
 	     free(tpat.pattern);
 	     free(tpat.classes);
 	     
 	     if (rmdir(filename) == -1)
 		{
-		sprintf(OUTPUT,"Couldn't remove directory %s while purging\n",filename);
+		snprintf(OUTPUT,bufsize*2,"Couldn't remove directory %s while purging\n",filename);
 		CfLog(cfverbose,OUTPUT,"rmdir");
 		}
 	     
@@ -578,7 +580,7 @@ Debug("PurgeFiles(%s)\n",directory);
 	     }
 	  else if (unlink(filename) == -1)
 	     {
-	     sprintf(OUTPUT,"Couldn't unlink %s while purging\n",filename);
+	     snprintf(OUTPUT,bufsize*2,"Couldn't unlink %s while purging\n",filename);
 	     CfLog(cfverbose,OUTPUT,"");
 	     }
 	  }
@@ -603,15 +605,15 @@ struct Image *ip;
 { char linkbuf[bufsize], *lastnode;
   struct stat deststatbuf;
   struct Link empty;
-  int succeed, i, silent = false, enforcelinks;
+  int succeed, silent = false, enforcelinks;
   mode_t srcmode = sourcestatbuf.st_mode;
-  int ok_to_copy, found;
+  int ok_to_copy = false, found;
   
 Debug2("ImageCopy(%s,%s,+%o,-%o)\n",sourcefile,destfile,ip->plus,ip->minus);
 
 if ((strcmp(sourcefile,destfile) == 0) && (strcmp(ip->server,"localhost") == 0))
    {
-   sprintf(OUTPUT,"Image loop: file/dir %s copies to itself",sourcefile);
+   snprintf(OUTPUT,bufsize*2,"Image loop: file/dir %s copies to itself",sourcefile);
    CfLog(cfinform,OUTPUT,"");
    return;
    }
@@ -626,6 +628,8 @@ if (IgnoredOrExcluded(image,sourcefile,ip->inclusions,ip->exclusions))
 
 if (ip->linktype != 'n')
    {
+   lastnode=ReadLastNode(sourcefile);
+   
    if (IsWildItemIn(VLINKCOPIES,lastnode) || IsWildItemIn(ip->symlink,lastnode))
       {
       Verbose("cfengine: copy item %s marked for linking instead\n",sourcefile);
@@ -674,7 +678,7 @@ if (found != -1)
       {
       if (unlink(destfile) == -1)
 	 {
-	 sprintf(OUTPUT,"Couldn't remove link %s",destfile);
+	 snprintf(OUTPUT,bufsize*2,"Couldn't remove link %s",destfile);
 	 CfLog(cferror,OUTPUT,"unlink");
 	 return;
 	 }
@@ -689,7 +693,7 @@ if (ip->size != cfnosize)
       {
       case '<':  if (sourcestatbuf.st_size > ip->size)
 	 {
-	 sprintf(OUTPUT,"Source file %s is > %d bytes in copy (omitting)\n",sourcefile,ip->size);
+	 snprintf(OUTPUT,bufsize*2,"Source file %s is > %d bytes in copy (omitting)\n",sourcefile,ip->size);
 	 CfLog(cfinform,OUTPUT,"");
 	 return;
 	 }
@@ -698,7 +702,7 @@ if (ip->size != cfnosize)
       case '=':
 	  if (sourcestatbuf.st_size != ip->size)
 	     {
-	     sprintf(OUTPUT,"Source file %s is not %d bytes in copy (omitting)\n",sourcefile,ip->size);
+	     snprintf(OUTPUT,bufsize*2,"Source file %s is not %d bytes in copy (omitting)\n",sourcefile,ip->size);
 	     CfLog(cfinform,OUTPUT,"");
 	     return;
 	     }
@@ -720,18 +724,24 @@ if (found == -1)
    {
    if (strcmp(ip->action,"warn") == 0)
       {
-      sprintf(OUTPUT,"Image file %s is non-existent\n",destfile);
+      snprintf(OUTPUT,bufsize*2,"Image file %s is non-existent\n",destfile);
       CfLog(cfinform,OUTPUT,"");
-      sprintf(OUTPUT,"(should be copy of %s)\n",sourcefile);
+      snprintf(OUTPUT,bufsize*2,"(should be copy of %s)\n",sourcefile);
       CfLog(cfinform,OUTPUT,"");
       return;
       }
 
    if (S_ISREG(srcmode))
       {
-      sprintf(OUTPUT,"%s wasn't at destination (copying)",destfile);
+      snprintf(OUTPUT,bufsize*2,"%s wasn't at destination (copying)",destfile);
+      if (DONTDO)
+	 {
+	 printf("Need this: %s\n",OUTPUT);
+	 return;
+	 }
+
       CfLog(cfverbose,OUTPUT,"");
-      sprintf(OUTPUT,"Copying from %s:%s\n",ip->server,sourcefile);
+      snprintf(OUTPUT,bufsize*2,"Copying from %s:%s\n",ip->server,sourcefile);
       CfLog(cfinform,OUTPUT,"");
 
       if (CopyReg(sourcefile,destfile,sourcestatbuf,deststatbuf,ip))
@@ -754,7 +764,7 @@ if (found == -1)
          }
       else if (mkfifo (destfile,srcmode))
          {
-         sprintf(OUTPUT,"Cannot create fifo `%s'", destfile);
+         snprintf(OUTPUT,bufsize*2,"Cannot create fifo `%s'", destfile);
 	 CfLog(cferror,OUTPUT,"mkfifo");
          return;
          }
@@ -772,7 +782,7 @@ if (found == -1)
             }
          else if (mknod (destfile, srcmode, sourcestatbuf.st_rdev))
             {
-            sprintf(OUTPUT,"Cannot create special file `%s'",destfile);
+            snprintf(OUTPUT,bufsize*2,"Cannot create special file `%s'",destfile);
 	    CfLog(cferror,OUTPUT,"mknod");
             return;
             }
@@ -785,13 +795,13 @@ if (found == -1)
       {
       if (cfreadlink(sourcefile,linkbuf,bufsize,ip) == -1)
          {
-         sprintf(OUTPUT,"Can't readlink %s\n",sourcefile);
+         snprintf(OUTPUT,bufsize*2,"Can't readlink %s\n",sourcefile);
 	 CfLog(cferror,OUTPUT,"");
 	 Debug2("Leaving ImageCopy\n");
          return;
          }
 
-      sprintf(OUTPUT,"Checking link from %s to %s\n",destfile,linkbuf);
+      snprintf(OUTPUT,bufsize*2,"Checking link from %s to %s\n",destfile,linkbuf);
       CfLog(cfverbose,OUTPUT,"");
 
       if (ip->linktype == 'a' && linkbuf[0] != '/')      /* Not absolute path - must fix */
@@ -838,26 +848,26 @@ else
    {
    Debug("Destination file %s exists\n",destfile);
    
-   if (! ip->force)
+   if (ip->force == 'n')
       {
       switch (ip->type)
          {
          case 'c': if (S_ISREG(deststatbuf.st_mode) && S_ISREG(srcmode))
 		      {
-	              ok_to_copy = CompareMD5CheckSums(sourcefile,destfile,ip,&sourcestatbuf,&deststatbuf);
+	              ok_to_copy = CompareCheckSums(sourcefile,destfile,ip,&sourcestatbuf,&deststatbuf);
 		      }	     
 	           else
 	              {
-		      CfLog(cfinform,"Checksum comparison replaced by ctime: files not regular\n","");
-		      sprintf(OUTPUT,"%s -> %s\n",sourcefile,destfile);
+		      CfLog(cfverbose,"Checksum comparison replaced by ctime: files not regular\n","");
+		      snprintf(OUTPUT,bufsize*2,"%s -> %s\n",sourcefile,destfile);
 		      CfLog(cfinform,OUTPUT,"");
 		      ok_to_copy = (deststatbuf.st_ctime < sourcestatbuf.st_ctime)||(deststatbuf.st_mtime < sourcestatbuf.st_mtime);
 	              }
 
                    if (ok_to_copy && strcmp(ip->action,"warn") == 0)
                       { 
-                      sprintf(OUTPUT,"Image file %s has a wrong MD5 checksum (should be copy of %s)\n",destfile,sourcefile);
-		      CfLog(cfinform,OUTPUT,"");
+                      snprintf(OUTPUT,bufsize*2,"Image file %s has a wrong MD5 checksum (should be copy of %s)\n",destfile,sourcefile);
+		      CfLog(cferror,OUTPUT,"");
                       return;
                       }
 	           break;
@@ -869,15 +879,15 @@ else
 	           else
 	              {
 		      CfLog(cfinform,"Byte comparison replaced by ctime: files not regular\n","");
-		      sprintf(OUTPUT,"%s -> %s\n",sourcefile,destfile);
-		      CfLog(cfinform,OUTPUT,"");
+		      snprintf(OUTPUT,bufsize*2,"%s -> %s\n",sourcefile,destfile);
+		      CfLog(cfverbose,OUTPUT,"");
 		      ok_to_copy = (deststatbuf.st_ctime < sourcestatbuf.st_ctime)||(deststatbuf.st_mtime < sourcestatbuf.st_mtime);
 	              }
 
                    if (ok_to_copy && strcmp(ip->action,"warn") == 0)
                       { 
-                      sprintf(OUTPUT,"Image file %s has a wrong binary checksum (should be copy of %s)\n",destfile,sourcefile);
-		      CfLog(cfinform,OUTPUT,"");
+                      snprintf(OUTPUT,bufsize*2,"Image file %s has a wrong binary checksum (should be copy of %s)\n",destfile,sourcefile);
+		      CfLog(cferror,OUTPUT,"");
                       return;
                       }
 	           break;
@@ -886,8 +896,8 @@ else
 	     
                    if (ok_to_copy && strcmp(ip->action,"warn") == 0)
                       { 
-                      sprintf(OUTPUT,"Image file %s out of date (should be copy of %s)\n",destfile,sourcefile);
-                      CfLog(cfinform,OUTPUT,"");
+                      snprintf(OUTPUT,bufsize*2,"Image file %s out of date (should be copy of %s)\n",destfile,sourcefile);
+                      CfLog(cferror,OUTPUT,"");
                       return;
                       }
 	           break;
@@ -896,8 +906,8 @@ else
 	     
                    if (ok_to_copy && strcmp(ip->action,"warn") == 0)
                       { 
-                      sprintf(OUTPUT,"Image file %s out of date (should be copy of %s)\n",destfile,sourcefile);
-                      CfLog(cfinform,OUTPUT,"");
+                      snprintf(OUTPUT,bufsize*2,"Image file %s out of date (should be copy of %s)\n",destfile,sourcefile);
+                      CfLog(cferror,OUTPUT,"");
                       return;
                       }
 	           break;
@@ -905,7 +915,7 @@ else
       }
 
 
-   if (ip->typecheck == 't')
+   if (ip->typecheck == 'y')
       {
       if ((S_ISDIR(deststatbuf.st_mode)  && ! S_ISDIR(sourcestatbuf.st_mode))  ||
 	  (S_ISREG(deststatbuf.st_mode)  && ! S_ISREG(sourcestatbuf.st_mode))  ||
@@ -922,11 +932,18 @@ else
 	 }
       }
 
-   if ((ip->force) || ok_to_copy || S_ISLNK(sourcestatbuf.st_mode))  /* Always check links */
+   if ((ip->force == 'y') || ok_to_copy || S_ISLNK(sourcestatbuf.st_mode))  /* Always check links */
       {
       if (S_ISREG(srcmode))
          {
-         sprintf(OUTPUT,"Updating image %s from master %s on %s\n",destfile,sourcefile,ip->server);
+         snprintf(OUTPUT,bufsize*2,"Update of image %s from master %s on %s",destfile,sourcefile,ip->server);
+
+	 if (DONTDO)
+	    {
+	    printf("Need this: %s\n",OUTPUT);
+	    return;
+	    }
+	 
 	 CfLog(cfinform,OUTPUT,"");
 
 	 AddMultipleClasses(ip->defines);
@@ -944,12 +961,12 @@ else
          {
          if (cfreadlink(sourcefile,linkbuf,bufsize,ip) == -1)
             {
-            sprintf(OUTPUT,"Can't readlink %s\n",sourcefile);
+            snprintf(OUTPUT,bufsize*2,"Can't readlink %s\n",sourcefile);
 	    CfLog(cferror,OUTPUT,"");
             return;
             }
 
-         sprintf(OUTPUT,"Checking link from %s to %s\n",destfile,linkbuf);
+         snprintf(OUTPUT,bufsize*2,"Checking link from %s to %s\n",destfile,linkbuf);
 	 CfLog(cfverbose,OUTPUT,"");
 
 	 enforcelinks = ENFORCELINKS;
@@ -989,7 +1006,7 @@ else
       {
       CheckCopiedFile(destfile,ip->plus,ip->minus,fixall,ip->uid,ip->gid,&deststatbuf,&sourcestatbuf,NULL,ip->acl_aliases);
 
-      Debug2("cfengine: image file is up to date: %s\n",destfile);
+      Debug("cfengine: image file is up to date: %s\n",destfile);
       AddMultipleClasses(ip->elsedef);
       }
    }
@@ -1009,9 +1026,7 @@ struct Image *ip;
 
 if (ip == NULL)
    {
-   res = stat(file,buf);
-   /* CheckForHoles(buf,ip); Don't need this if ip is NULL */
-   return res;
+   return stat(file,buf);
    }
 
 if (strcmp(ip->server,"localhost") == 0)
@@ -1040,9 +1055,7 @@ struct Image *ip;
 
 if (ip == NULL)
    {
-   res = lstat(file,buf);
-   /* CheckForHoles(buf,ip); Don't need this if ip is NULL */
-   return res;
+   return lstat(file,buf);
    }
 
 if (strcmp(ip->server,"localhost") == 0)
@@ -1200,7 +1213,7 @@ free((char *)dirh);
 /* Level 2                                                           */
 /*********************************************************************/
 
-int CopyReg (source,dest,sstat,dstat,ip)
+int CopyReg(source,dest,sstat,dstat,ip)
 
 char *source, *dest;
 struct stat sstat, dstat;
@@ -1208,7 +1221,8 @@ struct Image *ip;
 
 { char backup[bufsize];
   char new[bufsize], *linkable;
-  int remote = false, silent;
+  int remote = false, silent, backupisdir=false;
+  struct stat s;
 #ifdef HAVE_UTIME_H
   struct utimbuf timebuf;
 #endif  
@@ -1252,6 +1266,11 @@ strcat(new,CF_NEW);
 
 if (remote)
    {
+   if (CONN->error)
+      {
+      return false;
+      }
+   
    if (!CopyRegNet(source,new,ip,sstat.st_size))
       {
       return false;
@@ -1264,7 +1283,7 @@ else
       return false;
       }
 
-   if (ip->stealth == 't')
+   if (ip->stealth == 'y')
       {
 #ifdef HAVE_UTIME_H
       timebuf.actime = sstat.st_atime;
@@ -1276,7 +1295,7 @@ else
 
 Debug("CopyReg succeeded in copying to %s to %s\n",source,new);
 
-if (IMAGEBACKUP)
+if (IMAGEBACKUP == 'y')
    {
    strcpy(backup,dest);
    strcat(backup,CF_SAVED);
@@ -1286,17 +1305,44 @@ if (IMAGEBACKUP)
       return true;
       }
 
+   /* Mainly important if there is a dir in the way */
+
+   if (lstat(backup,&s) != -1)
+      {
+      if (S_ISDIR(s.st_mode))
+	 {
+	 backupisdir = true;
+	 PurgeFiles(NULL,backup);
+	 rmdir(backup);
+	 }
+
+      unlink(backup);
+      }
+   
    if (rename(dest,backup) == -1)
       {
       /* ignore */
       }
    }
+else
+   {
+   /* Mainly important if there is a dir in the way */
+
+   if (stat(dest,&s) != -1)
+      {
+      if (S_ISDIR(s.st_mode))
+	 {
+	 PurgeFiles(NULL,dest);
+	 rmdir(dest);
+	 }
+      }
+   }
 
 stat(new,&dstat);
 
 if (dstat.st_size != sstat.st_size)
    {
-   sprintf(OUTPUT,"WARNING: new file %s seems to have been corrupted in transit (sizes %d and %d), aborting!\n",new, (int) dstat.st_size, (int) sstat.st_size);
+   snprintf(OUTPUT,bufsize*2,"WARNING: new file %s seems to have been corrupted in transit (sizes %d and %d), aborting!\n",new, (int) dstat.st_size, (int) sstat.st_size);
    CfLog(cfverbose,OUTPUT,"");
    if (rename(backup,dest) == -1)
       {
@@ -1305,16 +1351,15 @@ if (dstat.st_size != sstat.st_size)
    return false;
    }
 
-if (ip->secure)
+if (ip->verify == 'y')
    {
-   Debug("Final verification of transmission...\n");
-   if (CompareMD5CheckSums(source,new,ip,&sstat,&dstat))
+   Verbose("Final verification of transmission.\n");
+   if (CompareCheckSums(source,new,ip,&sstat,&dstat))
       {
-      sprintf(OUTPUT,"WARNING: new file %s seems to have been corrupted in transit, aborting!\n",new);
+      snprintf(OUTPUT,bufsize*2,"WARNING: new file %s seems to have been corrupted in transit, aborting!\n",new);
       CfLog(cfverbose,OUTPUT,"");
       if (rename(backup,dest) == -1)
 	 {
-	 /* ignore */
 	 }
       return false;
       }
@@ -1322,18 +1367,23 @@ if (ip->secure)
 
 if (rename(new,dest) == -1)
    {
-   sprintf(OUTPUT,"Problem: could not install copy file as %s, directory in the way?\n",dest);
+   snprintf(OUTPUT,bufsize*2,"Problem: could not install copy file as %s, directory in the way?\n",dest);
    CfLog(cferror,OUTPUT,"rename");
    rename(backup,dest);
    return false;
    }
 
-if (IMAGEBACKUP && Repository(backup,ip->repository))
+if ((IMAGEBACKUP == 'y') && backupisdir)
+   {
+   snprintf(OUTPUT,bufsize,"Cannot move a directory to repository, leaving at %s",backup);
+   CfLog(cfinform,OUTPUT,"");
+   }
+else if ((IMAGEBACKUP == 'y') && Repository(backup,ip->repository))
    {
    unlink(backup);
    }
 
-if (ip->preservetimes == 't')
+if (ip->preservetimes == 'y')
    {
 #ifdef HAVE_UTIME_H
    timebuf.actime = sstat.st_atime;
