@@ -61,6 +61,7 @@ for (ptr = list; ptr != NULL; ptr=ptr->next)
 return(false);
 }
 
+
 /*********************************************************************/
 
 int IsFuzzyItemIn(list,item)
@@ -70,6 +71,8 @@ char *item;
 
 { struct Item *ptr; 
 
+Debug("FuzzyItemIn(%s)\n",item);
+ 
 if ((item == NULL) || (strlen(item) == 0))
    {
    return true;
@@ -94,7 +97,7 @@ struct Item **liststart;
 char *itemstring,*classes;
 
 { struct Item *ip;
-  char *sp,*spe;
+  char *sp,*spe = NULL;
 
 EditVerbose("Prepending %s\n",itemstring);
 
@@ -142,7 +145,7 @@ struct Item **liststart;
 char *itemstring,*classes;
 
 { struct Item *ip, *lp;
-  char *sp,*spe;
+  char *sp,*spe = NULL;
 
 EditVerbose("Appending [%s]\n",itemstring);
 
