@@ -55,7 +55,7 @@ if (stat(to,&statbuf) == -1)
 
 if ((dirh = opendir(to)) == NULL)
    {
-   sprintf(OUTPUT,"Can't open directory %s\n",to);
+   snprintf(OUTPUT,bufsize*2,"Can't open directory %s\n",to);
    CfLog(cferror,OUTPUT,"opendir");
    return false;
    }
@@ -150,7 +150,7 @@ Debug("LinkChildren(%s)\n",path);
   
 if (! S_ISDIR(rootstat->st_mode))
    {
-   sprintf(OUTPUT,"File %s is not a directory: it has no children to link!\n",path);
+   snprintf(OUTPUT,bufsize*2,"File %s is not a directory: it has no children to link!\n",path);
    CfLog(cferror,OUTPUT,"");
    return;
    }
@@ -176,7 +176,7 @@ for (sp = path+strlen(path); sp != path-1; sp--)
 
          if ((dirh = opendir(odir)) == NULL)
             {
-            sprintf(OUTPUT,"Can't open directory %s\n",path);
+            snprintf(OUTPUT,bufsize*2,"Can't open directory %s\n",path);
 	    CfLog(cferror,OUTPUT,"opendir");
             return;
             }
@@ -231,7 +231,7 @@ for (sp = path+strlen(path); sp != path-1; sp--)
                             linkfiles = HardLinkFiles;
                             break;
                   default:
-                            sprintf(OUTPUT,"Internal error, link type was [%c]\n",type);
+                            snprintf(OUTPUT,bufsize*2,"Internal error, link type was [%c]\n",type);
 			    CfLog(cferror,OUTPUT,"");
                             continue;
                   }
@@ -250,9 +250,9 @@ for (sp = path+strlen(path); sp != path-1; sp--)
       }
    }
 
-sprintf(OUTPUT,"Couldn't link the children of %s to anything because no\n",path);
+snprintf(OUTPUT,bufsize*2,"Couldn't link the children of %s to anything because no\n",path);
 CfLog(cferror,OUTPUT,""); 
-sprintf(OUTPUT,"file system was found to mirror it in the defined binservers list.\n");
+snprintf(OUTPUT,bufsize*2,"file system was found to mirror it in the defined binservers list.\n");
 CfLog(cferror,OUTPUT,""); 
 }
 
@@ -293,7 +293,7 @@ bug_check = lp->next;
 
 if ((dirh = opendir(to)) == NULL)
    {
-   sprintf(OUTPUT,"Can't open directory [%s]\n",to);
+   snprintf(OUTPUT,bufsize*2,"Can't open directory [%s]\n",to);
    CfLog(cferror,OUTPUT,"opendir");
    return false;
    }
@@ -342,7 +342,7 @@ for (dirp = readdir(dirh); dirp != NULL; dirp = readdir(dirh))
       {
       if (stat(newto,&statbuf) == -1)
          {
-         sprintf(OUTPUT,"Can't stat %s\n",newto);
+         snprintf(OUTPUT,bufsize*2,"Can't stat %s\n",newto);
 	 CfLog(cfverbose,OUTPUT,"");
          continue;
          }
@@ -351,7 +351,7 @@ for (dirp = readdir(dirh); dirp != NULL; dirp = readdir(dirh))
       {
       if (lstat(newto,&statbuf) == -1)
          {
-         sprintf(OUTPUT,"Can't stat %s\n",newto);
+         snprintf(OUTPUT,bufsize*2,"Can't stat %s\n",newto);
 	 CfLog(cfverbose,OUTPUT,"");
 	 
 	 bzero(VBUFF,bufsize);
@@ -363,6 +363,12 @@ for (dirp = readdir(dirh); dirp != NULL; dirp = readdir(dirh))
          }
       }
 
+   if (!FileObjectFilter(newto,&statbuf,lp->filters,links))
+      {
+      Debug("Skipping filtered file %s\n",newto);
+      continue;
+      }
+
    if (S_ISDIR(statbuf.st_mode))
       {
       RecursiveLink(lp,newfrom,newto,maxrecurse-1);
@@ -398,7 +404,9 @@ return true;
 
 /*********************************************************************/
 
-int LinkFiles(from,to_tmp,inclusions,exclusions,copy,nofile,ptr)  /* should return true if 'to' found */
+int LinkFiles(from,to_tmp,inclusions,exclusions,copy,nofile,ptr)
+
+/* should return true if 'to' found */
 
 char *from, *to_tmp;
 struct Item *inclusions, *exclusions, *copy;
@@ -406,7 +414,7 @@ short nofile;
 struct Link *ptr;
 
 { struct stat buf,savebuf;
-  char to[bufsize],linkbuf[bufsize], saved[bufsize], absto[bufsize], *sp, *lastnode;
+  char to[bufsize],linkbuf[bufsize],saved[bufsize],absto[bufsize],*lastnode;
   struct UidList fakeuid;
   struct Image ip;
 
@@ -461,7 +469,7 @@ if (IsWildItemIn(VCOPYLINKS,lastnode) || IsWildItemIn(copy,lastnode))
    ip.minus_flags = 0;
    ip.server = strdup("localhost");
    Verbose("%s: Link item %s marked for copying instead\n",VPREFIX,from);
-   MakeDirectoriesFor(to);
+   MakeDirectoriesFor(to,'n');
    CheckImage(to,from,&ip);
    free(ip.server);
    return true;
@@ -496,16 +504,16 @@ if (lstat(from,&buf) == 0)
    {
    if (! S_ISLNK(buf.st_mode) && ! ENFORCELINKS)
       {
-      sprintf(OUTPUT,"Error linking %s -> %s\n",from,to);
+      snprintf(OUTPUT,bufsize*2,"Error linking %s -> %s\n",from,to);
       CfLog(cfsilent,OUTPUT,"");
-      sprintf(OUTPUT,"Cannot make link: %s exists and is not a link! (uid %d)\n",from,buf.st_uid);
+      snprintf(OUTPUT,bufsize*2,"Cannot make link: %s exists and is not a link! (uid %d)\n",from,buf.st_uid);
       CfLog(cfsilent,OUTPUT,"");
       return(true);
       }
 
    if (S_ISREG(buf.st_mode) && ENFORCELINKS)
       {
-      sprintf(OUTPUT,"Moving %s to %s%s\n",from,from,CF_SAVED);
+      snprintf(OUTPUT,bufsize*2,"Moving %s to %s%s\n",from,from,CF_SAVED);
       CfLog(cfsilent,OUTPUT,"");
 
       if (DONTDO)
@@ -519,7 +527,7 @@ if (lstat(from,&buf) == 0)
 
       if (rename(from,saved) == -1)
          {
-         sprintf(OUTPUT,"Can't rename %s to %s\n",from,saved);
+         snprintf(OUTPUT,bufsize*2,"Can't rename %s to %s\n",from,saved);
 	 CfLog(cferror,OUTPUT,"rename");
          return(true);
          }
@@ -532,7 +540,7 @@ if (lstat(from,&buf) == 0)
 
    if (S_ISDIR(buf.st_mode) && ENFORCELINKS)
       {
-      sprintf(OUTPUT,"Moving directory %s to %s%s.dir\n",from,from,CF_SAVED);
+      snprintf(OUTPUT,bufsize*2,"Moving directory %s to %s%s.dir\n",from,from,CF_SAVED);
       CfLog(cfsilent,OUTPUT,"");
 
       if (DONTDO)
@@ -547,16 +555,16 @@ if (lstat(from,&buf) == 0)
 
       if (stat(saved,&savebuf) != -1)
 	 {
-	 sprintf(OUTPUT,"Couldn't save directory %s, since %s exists already\n",from,saved);
+	 snprintf(OUTPUT,bufsize*2,"Couldn't save directory %s, since %s exists already\n",from,saved);
 	 CfLog(cferror,OUTPUT,"");
-	 sprintf(OUTPUT,"Unable to force link to existing directory %s\n",from);
+	 snprintf(OUTPUT,bufsize*2,"Unable to force link to existing directory %s\n",from);
 	 CfLog(cferror,OUTPUT,"");
 	 return true;
 	 }
 
       if (rename(from,saved) == -1)
          {
-         sprintf(OUTPUT,"Can't rename %s to %s\n",from,saved);
+         snprintf(OUTPUT,bufsize*2,"Can't rename %s to %s\n",from,saved);
 	 CfLog(cferror,OUTPUT,"rename");
          return(true);
          }
@@ -567,11 +575,11 @@ bzero(linkbuf,bufsize);
 
 if (readlink(from,linkbuf,bufsize-1) == -1)
    {
-   if (! MakeDirectoriesFor(from))                  /* link doesn't exist */
+   if (! MakeDirectoriesFor(from,'n'))                  /* link doesn't exist */
       {
-      sprintf(OUTPUT,"Couldn't build directory tree up to %s!\n",from);
+      snprintf(OUTPUT,bufsize*2,"Couldn't build directory tree up to %s!\n",from);
       CfLog(cfsilent,OUTPUT,"");
-      sprintf(OUTPUT,"One element was a plain file, not a directory!\n");
+      snprintf(OUTPUT,bufsize*2,"One element was a plain file, not a directory!\n");
       CfLog(cfsilent,OUTPUT,"");      
       return(true);
       }
@@ -595,7 +603,7 @@ else
       {
       if (ENFORCELINKS)
          {
-         sprintf(OUTPUT,"Removing link %s\n",from);
+         snprintf(OUTPUT,bufsize*2,"Removing link %s\n",from);
 	 CfLog(cfinform,OUTPUT,"");
 
          if (!DONTDO)
@@ -611,22 +619,21 @@ else
          }
       else
          {
-         sprintf(OUTPUT,"Old link %s points somewhere else. Doing nothing!\n",from);
+         snprintf(OUTPUT,bufsize*2,"Old link %s points somewhere else. Doing nothing!\n",from);
 	 CfLog(cfsilent,OUTPUT,"");
-         sprintf(OUTPUT,"(Link points to %s not %s)\n\n",linkbuf,to);
+         snprintf(OUTPUT,bufsize*2,"(Link points to %s not %s)\n\n",linkbuf,to);
 	 CfLog(cfsilent,OUTPUT,"");	 
          return(true);
          }
       }
    else
       {
-      sprintf(OUTPUT,"Link (%s->%s) exists.\n",from,to_tmp);
+      snprintf(OUTPUT,bufsize*2,"Link (%s->%s) exists.\n",from,to_tmp);
       CfLog(cfverbose,OUTPUT,"");
 
       if (!nofile)
 	 {
-	 KillOldLink(from);             /* Check whether link points somewhere */
-	 AddMultipleClasses(ptr->defines);
+	 KillOldLink(from,ptr->defines);      /* Check whether link points somewhere */
 	 return true;
 	 }
 
@@ -648,7 +655,7 @@ short nofile;
 struct Link *ptr;
 /* global char LINKTO[] */
 
-{ char *sp, *commonto, *commonfrom, *lastnode;
+{ char *sp, *commonto, *commonfrom;
   char buff[bufsize];
   int levels=0;
   
@@ -661,7 +668,7 @@ if (*to == '.')
 
 if (!CompressPath(LINKTO,to))
    {
-   sprintf(OUTPUT,"Failed to link %s to %s\n",from,to);
+   snprintf(OUTPUT,bufsize*2,"Failed to link %s to %s\n",from,to);
    CfLog(cferror,OUTPUT,"");
    return false;
    }
@@ -672,7 +679,7 @@ commonfrom = from;
 if (strcmp(commonto,commonfrom) == 0)
    {
    CfLog(cferror,"Can't link file to itself!\n","");
-   sprintf(OUTPUT,"(%s -> %s)\n",from,to);
+   snprintf(OUTPUT,bufsize*2,"(%s -> %s)\n",from,to);
    CfLog(cferror,OUTPUT,"");
    return false;
    }
@@ -758,16 +765,23 @@ CompressPath(absto,LINKTO);
 
 expand[0] = '\0';
 
-if (!ExpandLinks(expand,absto,0))  /* begin at level 1 and beam out at 15 */
-   {
-   CfLog(cferror,"Failed to make absolute link in\n","");
-   sprintf("%s: %s -> %s\n",from,to);
-   CfLog(cferror,OUTPUT,"");
-   return false;
+if (!nofile)
+   {  
+   if (!ExpandLinks(expand,absto,0))  /* begin at level 1 and beam out at 15 */
+      {
+      CfLog(cferror,"Failed to make absolute link in\n","");
+      snprintf(OUTPUT,bufsize*2,"%s -> %s\n",from,to);
+      CfLog(cferror,OUTPUT,"");
+      return false;
+      }
+   else
+      {
+      Debug2("ExpandLinks returned %s\n",expand);
+      }
    }
 else
    {
-   Debug2("ExpandLinks returned %s\n",expand);
+   strcpy(expand,absto);
    }
 
 CompressPath(LINKTO,expand);
@@ -788,12 +802,12 @@ if (DONTDO)
    }
 else
    {
-   sprintf(OUTPUT,"Linking files %s -> %s\n",from,to);
+   snprintf(OUTPUT,bufsize*2,"Linking files %s -> %s\n",from,to);
    CfLog(cfinform,OUTPUT,"");
 
    if (symlink(to,from) == -1)
       {
-      sprintf(OUTPUT,"Couldn't link %s to %s\n",to,from);
+      snprintf(OUTPUT,bufsize*2,"Couldn't link %s to %s\n",to,from);
       CfLog(cferror,OUTPUT,"symlink");
       return false;
       }
@@ -803,19 +817,18 @@ else
       return true;
       }
    }
-   return true;
+ return true;
 }
 
 /*********************************************************************/
 
-void KillOldLink(name)
+void KillOldLink(name,defs)
 
-char *name;
+char *name,*defs;
 
 { char linkbuf[bufsize];
   char linkpath[bufsize],*sp;
   struct stat statbuf;
-  short i;
 
 Debug("KillOldLink(%s)\n",name);
 bzero(linkbuf,bufsize);
@@ -823,7 +836,7 @@ bzero(linkpath,bufsize);
 
 if (readlink(name,linkbuf,bufsize-1) == -1)
    {
-   sprintf(OUTPUT,"(Can't read link %s while checking for deadlinks)\n",name);
+   snprintf(OUTPUT,bufsize*2,"(Can't read link %s while checking for deadlinks)\n",name);
    CfLog(cfverbose,OUTPUT,"");
    return;
    }
@@ -845,18 +858,19 @@ if (stat(VBUFF,&statbuf) == -1)               /* link points nowhere */
    {
    if (KILLOLDLINKS || DEBUG || D2)
       {
-      sprintf(OUTPUT,"%s is a link which points to %s, but that file doesn't seem to exist\n",name,VBUFF);
+      snprintf(OUTPUT,bufsize*2,"%s is a link which points to %s, but that file doesn't seem to exist\n",name,VBUFF);
       CfLog(cfsilent,OUTPUT,"");
       }
 
    if (KILLOLDLINKS)
       {
-      sprintf(OUTPUT,"Removing dead link %s\n",name);
+      snprintf(OUTPUT,bufsize*2,"Removing dead link %s\n",name);
       CfLog(cfinform,OUTPUT,"");
 
       if (! DONTDO)
          {
          unlink(name);  /* May not work on a client-mounted system ! */
+	 AddMultipleClasses(defs);
          }
       }
    }
@@ -922,7 +936,7 @@ if (stat(to,&tobuf) == -1)
 
 if (! S_ISREG(tobuf.st_mode))
    {
-   sprintf(OUTPUT,"%s: will only hard link regular files and %s is not regular\n",VPREFIX,to);
+   snprintf(OUTPUT,bufsize*2,"%s: will only hard link regular files and %s is not regular\n",VPREFIX,to);
    CfLog(cfsilent,OUTPUT,"");
    return true;
    }
@@ -946,7 +960,7 @@ if (frombuf.st_ino != tobuf.st_ino && frombuf.st_dev != frombuf.st_dev)
 
    if (frombuf.st_mode == tobuf.st_mode && frombuf.st_size == tobuf.st_size)
       {
-      sprintf(OUTPUT,"Hard link (%s->%s) on different device APPEARS okay\n",from,to);
+      snprintf(OUTPUT,bufsize*2,"Hard link (%s->%s) on different device APPEARS okay\n",from,to);
       CfLog(cfverbose,OUTPUT,"");
       AddMultipleClasses(ptr->elsedef);
       return true;
@@ -955,18 +969,18 @@ if (frombuf.st_ino != tobuf.st_ino && frombuf.st_dev != frombuf.st_dev)
 
 if (frombuf.st_ino == tobuf.st_ino && frombuf.st_dev == frombuf.st_dev)
    {
-   sprintf(OUTPUT,"Hard link (%s->%s) exists and is okay.\n",from,to);
+   snprintf(OUTPUT,bufsize*2,"Hard link (%s->%s) exists and is okay.\n",from,to);
    CfLog(cfverbose,OUTPUT,"");
    AddMultipleClasses(ptr->elsedef);
    return true;
    }
 
-sprintf(OUTPUT,"%s does not appear to be a hard link to %s\n",from,to);
+snprintf(OUTPUT,bufsize*2,"%s does not appear to be a hard link to %s\n",from,to);
 CfLog(cfinform,OUTPUT,""); 
 
 if (ENFORCELINKS)
    {
-   sprintf(OUTPUT,"Moving %s to %s.%s\n",from,from,CF_SAVED);
+   snprintf(OUTPUT,bufsize*2,"Moving %s to %s.%s\n",from,from,CF_SAVED);
    CfLog(cfinform,OUTPUT,"");
 
    if (DONTDO)
@@ -1003,7 +1017,7 @@ if (DONTDO)
    }
 else
    {
-   sprintf(OUTPUT,"Hardlinking files %s -> %s\n",from,to);
+   snprintf(OUTPUT,bufsize*2,"Hardlinking files %s -> %s\n",from,to);
    CfLog(cfinform,OUTPUT,"");
 
    if (link(to,from) == -1)
@@ -1066,7 +1080,7 @@ for (sp = from; *sp != '\0'; sp++)
       {
       if (! ChopLastNode(LINKTO))
 	 {
-	 Debug("cfengine: used .. beyond top of filesystem!\n","");
+	 Debug("cfengine: used .. beyond top of filesystem!\n");
 	 return false;
 	 }
       continue;
@@ -1080,7 +1094,7 @@ for (sp = from; *sp != '\0'; sp++)
 
    if (lstat(dest,&statbuf) == -1)  /* File doesn't exist so we can stop here */
       {
-      sprintf(OUTPUT,"Can't stat %s in ExpandLinks\n",dest);
+      snprintf(OUTPUT,bufsize*2,"Can't stat %s in ExpandLinks\n",dest);
       CfLog(cferror,OUTPUT,"stat");
       return false;
       }
@@ -1091,7 +1105,7 @@ for (sp = from; *sp != '\0'; sp++)
       
       if (readlink(dest,buff,bufsize-1) == -1)
 	 {
-	 sprintf(OUTPUT,"Expand links can't stat %s\n",dest);
+	 snprintf(OUTPUT,bufsize*2,"Expand links can't stat %s\n",dest);
 	 CfLog(cferror,OUTPUT,"readlink");
 	 return false;
 	 }
