@@ -59,7 +59,7 @@ for (sp = buffer; *sp != '\0'; sp++)
    {
    if (*sp == '%')
       {
-      *sp = '*';
+      *sp = '?';
       }
    }
 
@@ -101,7 +101,7 @@ switch(level)
 			if ((errstr == NULL) || (strlen(errstr) > 0))
 			   {
 			   ShowAction();
-			   printf("%s: %s",VPREFIX,buffer);
+			   printf("%s: %s\n",VPREFIX,buffer);
 			   printf("%s: %s",VPREFIX,errstr);
 			   endl = true;
 			   }
@@ -125,11 +125,11 @@ switch(level)
    case cflogonly:
                      if (LOGGING && IsPrivileged())
 			{
-			syslog(LOG_ERR,"%s",buffer);    
+			syslog(LOG_ERR,"cfagent: %s",buffer);    
 			
 			if ((errstr == NULL) || (strlen(errstr) > 0))
 			   {
-			   syslog(LOG_ERR,"%s",errstr);  
+			   syslog(LOG_ERR,"cfagent: %s",errstr);  
 			   }
 			}
 		     
@@ -140,7 +140,7 @@ switch(level)
 
 		     if (LOGGING && IsPrivileged())
 			{
-			syslog(LOG_ERR,"%s",buffer);    
+			syslog(LOG_ERR,"cfagent: %s",buffer);    
 			}
  
 		     if (buffer[strlen(buffer)-1] != '\n')
@@ -156,7 +156,7 @@ switch(level)
 			
 			if (LOGGING && IsPrivileged())
 			   {
-			   syslog(LOG_ERR, "%s: %s",errstr,strerror(errno));   
+			   syslog(LOG_ERR, "cfagent: %s: %s",errstr,strerror(errno));   
 			   }
                         }
 		     return;
@@ -164,9 +164,9 @@ switch(level)
 
  
 if (endl && (buffer[strlen(buffer)-1] != '\n'))
-    {
-    printf("\n");
-    }
+   {
+   printf("\n");
+   }
 }
 
 /*****************************************************************************/
@@ -178,24 +178,24 @@ char log, inform;
   /* t = true, f = false, d = default */
 
 {
-if ((log == 't') || (log == 'f') || (inform == 't') || (inform == 'f'))
+if ((log == 'y') || (log == 'n') || (inform == 'y') || (inform == 'n'))
    {
    INFORM_save = INFORM;
    LOGGING_save = LOGGING;
    
    switch (log)
       {
-      case 't': LOGGING = true;
+      case 'y': LOGGING = true;
 	        break;
-      case 'f': LOGGING = false;
+      case 'n': LOGGING = false;
 	        break;
       }
 
    switch (inform)
       {
-      case 't': INFORM = true;
+      case 'y': INFORM = true;
 	        break;
-      case 'f': INFORM = false;
+      case 'n': INFORM = false;
 	        break;
       }
    }
