@@ -102,7 +102,6 @@ while (*key)
 return (h % hashtablesize);
 }
 
-
 /*******************************************************************/
 
 void AddMacroValue(name,value)                           /* Like putenv */
@@ -128,7 +127,7 @@ if (strlen(name) > maxvarsize)
 
 ExpandVarstring(value,exp,NULL);
 
-sprintf(buffer,"%s=%s",name,exp);
+snprintf(buffer,bufsize,"%s=%s",name,exp);
 
 if ((sp = malloc(strlen(buffer)+1)) == NULL)
    {
@@ -145,8 +144,11 @@ if (HASH[slot] != 0)
    Debug("Macro Collision!\n");
    if (CompareMacro(name,HASH[slot]) == 0)
       {
-      sprintf(VBUFF,"Redefinition of macro %s=%s",name,exp);
-      Warning(VBUFF);
+      if (PARSING)
+	 {
+	 snprintf(VBUFF,bufsize,"Redefinition of macro %s=%s",name,exp);
+	 Warning(VBUFF);
+	 }
       free(HASH[slot]);
       HASH[slot] = sp;
       return;
@@ -162,6 +164,15 @@ if (HASH[slot] != 0)
          {
          FatalError("AddMacroValue - internal error #1");
          }
+
+      if (CompareMacro(name,HASH[slot]) == 0)
+	 {
+	 snprintf(VBUFF,bufsize,"Redefinition of macro %s=%s",name,exp);
+	 Warning(VBUFF);
+	 free(HASH[slot]);
+	 HASH[slot] = sp;
+	 return;
+	 }
       }
    }
 
@@ -223,8 +234,7 @@ void DeleteMacro(id)
 
 char *id;
 
-{ char *sp;
-  int slot,i;
+{ int slot,i;
 
 i = slot = Hash(id);
  
@@ -253,21 +263,6 @@ else
    }   
 }
 
-/*******************************************************************/
-
-void RecordMacroId(name)
-
-char *name;
-
-{
-Debug("RecordMacroId(%s)\n",name);
-strcpy(CURRENTITEM,name);
-
-if (strcmp(name,"this") == 0)
-   {
-   yyerror("$(this) is a reserved variable");
-   }
-}
 
 /*******************************************************************/
 
