@@ -37,10 +37,13 @@ void ParseModeString(modestring,plusmask,minusmask)
 char *modestring;
 mode_t *plusmask, *minusmask;
 
-{ char modebuffer[maxvarsize], *sp; 
+{ char *sp;
  int affected = 0, value = 0, gotaction;
-  char action = '=';
-  enum modestate state = who;
+ char action = '=';
+ enum modestate state = unknown;
+ enum modesort found_sort = unknown; /* Already found "sort" of mode */
+ enum modesort sort = unknown; /* Sort of started but not yet finished mode */
+
 
 if (modestring == NULL)
    {
@@ -56,20 +59,24 @@ for (sp = modestring; true ; *sp++)
    {
    switch (*sp)
       {
-      case 'a': CheckModeState(who,state,*sp);
+      case 'a': CheckModeState(who,state,symbolic,sort,*sp);
                 affected |= 07777;
+		sort = symbolic;
                 break;
 
-      case 'u': CheckModeState(who,state,*sp);
+      case 'u': CheckModeState(who,state,symbolic,sort,*sp);
                 affected |= 04700;
+		sort = symbolic;
                 break;
 
-      case 'g': CheckModeState(who,state,*sp);
+      case 'g': CheckModeState(who,state,symbolic,sort,*sp);
                 affected |= 02070;
+		sort = symbolic;
                 break;
 
-      case 'o': CheckModeState(who,state,*sp);
+      case 'o': CheckModeState(who,state,symbolic,sort,*sp);
                 affected |= 00007;
+		sort = symbolic;
                 break;
 
       case '+':
@@ -78,29 +85,37 @@ for (sp = modestring; true ; *sp++)
                    {
                    yyerror("Too many +-= in mode string");
                    }
+
+      		CheckModeState(who,state,symbolic,sort,*sp);
                 action = *sp;
                 state = which;
                 gotaction = true;
+		sort = unknown;
                 break;
 
-      case 'r': CheckModeState(which,state,*sp);
+      case 'r': CheckModeState(which,state,symbolic,sort,*sp);
                 value |= 0444 & affected;
+		sort = symbolic;
                 break;
 
-      case 'w': CheckModeState(which,state,*sp);
+      case 'w': CheckModeState(which,state,symbolic,sort,*sp);
                 value |= 0222 & affected;
+		sort = symbolic;
                 break;
 
-      case 'x': CheckModeState(which,state,*sp);
+      case 'x': CheckModeState(which,state,symbolic,sort,*sp);
                 value |= 0111 & affected;
+		sort = symbolic;
                 break;
 
-      case 's': CheckModeState(which,state,*sp);
+      case 's': CheckModeState(which,state,symbolic,sort,*sp);
                 value |= 06000 & affected;
+		sort = symbolic;
                 break;
 
-      case 't': CheckModeState(which,state,*sp);
+      case 't': CheckModeState(which,state,symbolic,sort,*sp);
                 value |= 01000;
+		sort = symbolic;
                 break;
 
       case '0':
@@ -110,9 +125,21 @@ for (sp = modestring; true ; *sp++)
       case '4':
       case '5':
       case '6':
-      case '7': state = which;
+      case '7': CheckModeState(which,state,numeric,sort,*sp);
+		sort = numeric;
+		gotaction = true;
+		state = which;
+	        affected = 07777; /* TODO: Hard-coded; see below */
                 sscanf(sp,"%o",&value);
-                while (isdigit(*sp) && (*sp != '\0'))
+		if (value > 07777) /* TODO: Hardcoded !
+				      Is this correct for all sorts of Unix ?
+				      What about NT ?
+				      Any (POSIX)-constants ??
+				   */
+                   {
+		   yyerror("Mode-Value too big !\n");
+                   }
+                while (isdigit((int)*sp) && (*sp != '\0'))
 		   {
                    sp++;
                    }
@@ -120,7 +147,13 @@ for (sp = modestring; true ; *sp++)
                 break;
 
       case ',':
-                SetMask(action,value,plusmask,minusmask);
+                SetMask(action,value,affected,plusmask,minusmask);
+		if (found_sort != unknown && found_sort != sort)
+		   {
+		   Warning("Symbolic and numeric form for modes mixed");
+		   }
+		found_sort = sort;
+		sort = unknown;
                 action = '=';
                 affected = 0;
                 value = 0;
@@ -129,51 +162,60 @@ for (sp = modestring; true ; *sp++)
                 break;
 
       case '\0':
-                if (state == who || value == 0)
-                   {
+	        if (state == who || value == 0)
+		   {
 		   if (strcmp(modestring,"0000") != 0 && strcmp(modestring,"000") != 0)
-                         {
-                         Warning("mode string is incomplete");
-                         }
+		      {
+		      Warning("mode string is incomplete");
+		      }
                    }
-
-                SetMask(action,value,plusmask,minusmask);
+		
+                SetMask(action,value,affected,plusmask,minusmask);
+		if (found_sort != unknown && found_sort != sort)
+		   {
+		   Warning("Symbolic and numeric form for modes mixed");
+		   }
                 Debug1("[PLUS=%o][MINUS=%o]\n",*plusmask,*minusmask);
                 return;
 
-      default:  yyerror ("Invalid mode string");
-                break;
+      default: snprintf(OUTPUT,bufsize,"Invalid mode string (%s)",modestring);  
+ 	       yyerror (OUTPUT);
+               break;
       }
    }
 }
 
 /*********************************************************/
 
-void CheckModeState(stateA,stateB,ch)
+void CheckModeState(stateA,stateB,sortA,sortB,ch)
 
-enum modestate stateA;
-int stateB;
+enum modestate stateA, stateB;
+enum modesort sortA, sortB;
 char ch;
 
 {
-if ((int)stateA != stateB)
+if ((stateA != wild) && (stateB != wild) && (stateA != stateB))
    {
    sprintf(VBUFF,"Mode string constant (%c) used out of context",ch);
    yyerror(VBUFF);
    }
-return;
+
+if ((sortA != unknown) && (sortB != unknown) && (sortA != sortB))
+   {
+   yyerror("Symbolic and numeric filemodes mixed within expression");
+   }
 }
 
 /*********************************************************/
 
-void SetMask(action,value,p,m)
+void SetMask(action,value,affected,p,m)
 
 char action;
-int value;
+int value, affected;
 mode_t *p,*m;
 
 {
-Debug1("SetMask(%c%o)\n",action,value);
+Debug1("SetMask(%c%o,%o)\n",action,value,affected);
 
 switch(action)
    {
@@ -187,7 +229,7 @@ switch(action)
              return;
    case '=':
              *p |= value;
-             *m |= (~value) & 07777;
+             *m |= ((~value) & 07777 & affected);
              return;
    default:
              sprintf(VBUFF,"Mode directive %c is unknown",action);
