@@ -322,6 +322,29 @@ return -1;
 
 #endif
 
+/*******************************************************************/
+
+#ifndef HAVE_DRAND48  
+
+double drand48(void)
+
+{
+return (double)random();
+}
+
+#endif
+
+#ifndef HAVE_DRAND48  
+
+void srand48(seed)
+
+long seed;
+
+{
+srandom((unsigned int)seed);
+}
+
+#endif
 
 
 /*******************************************************************/
