@@ -54,11 +54,6 @@
 
 int cf_full_write ARGLIST((int desc, char *ptr, size_t len));
 
-/* ../pub/md5.c */
-
-void cfMDFile  ARGLIST((char *filename, unsigned char digest[16]));
-char *cfMDPrint ARGLIST((unsigned char digest[16]));
-
 /* 2Dlist.c */
 
 void Set2DList ARGLIST((struct TwoDimList *list));
@@ -119,16 +114,27 @@ int yyparse ARGLIST((void));
 
 /* checksums.c */
 
-int CompareMD5CheckSums ARGLIST((char *file1, char *file2, struct Image *ip, struct stat *sstat, struct stat *dstat));
+int CompareCheckSums ARGLIST((char *file1, char *file2, struct Image *ip, struct stat *sstat, struct stat *dstat));
 int CompareBinarySums ARGLIST((char *file1, char *file2, struct Image *ip, struct stat *sstat, struct stat *dstat));
 
+/* functions.c */
+
+char *EvaluateFunction ARGLIST((char *function, char *value));
+enum builtin FunctionStringToCode ARGLIST((char *str));
+int IsBuiltinFunction  ARGLIST((char *function));
+
+/* granules.c  */
+
+char *ConvTimeKey ARGLIST((char *str));
+char *GenTimeKey ARGLIST((time_t now));
+
 /* chflags.c */
 
 void ParseFlagString ARGLIST((char *flagstring, u_long *plusmask, u_long *minusmask));
 
 /* client.c */
 
-int OpenServerConnection ARGLIST((struct Item *serverlist, struct Image *ip));
+int OpenServerConnection ARGLIST((struct Image *ip));
 void CloseServerConnection ARGLIST((void));
 int cf_rstat ARGLIST((char *file, struct stat *buf, struct Image *ip, char *stattype));
 CFDIR *cf_ropendir ARGLIST((char *dirname, struct Image *ip));
@@ -139,6 +145,9 @@ int GetCachedStatData ARGLIST((char *file, struct stat *statbuf, struct Image *i
 void CacheData ARGLIST((struct cfstat *data, struct Image *ip));
 void FlushToEnd ARGLIST((int sd, int toget));
 int cfprintf(char *out, int len2, char *in1, char *in2, char *in3);
+struct cfagent_connection *NewAgentConn ARGLIST((void));
+void DeleteAgentConn ARGLIST((struct cfagent_connection *ap));
+int RemoteConnect ARGLIST((char *host,int ipv4));
 
 /* comparray.c */
 
@@ -151,6 +160,7 @@ char *CompressedArrayValue ARGLIST((struct CompressedArray *start, int key));
 
 void CheckForHoles ARGLIST((struct stat *sstat, struct Image *ip));
 int CopyRegDisk ARGLIST((char *source, char *new, struct Image *ip));
+int EmbeddedWrite ARGLIST((char *new,int dd,char *buf,struct Image *ip,int towrite,int *last_write_made_hole,int n_read));
 
 /* dce_acl.c */
 
@@ -169,10 +179,12 @@ void MakePaths ARGLIST((void));
 void MakeChildLinks ARGLIST((void));
 void MakeLinks ARGLIST((void));
 void MailCheck ARGLIST((void));
-void MailCheckMore ARGLIST((char *rmailpath));
+void ExpiredUserCheck ARGLIST((char *spooldir, int always));
 void MountFileSystems ARGLIST((void));
 void CheckRequired ARGLIST((void));
 void TidyFiles ARGLIST((void));
+void TidyHome ARGLIST((void));
+void TidyHomeDir ARGLIST((struct TidyPattern *ptr, char *subdir));
 void Scripts ARGLIST((void));
 void GetSetuidLog ARGLIST((void));
 void CheckFiles ARGLIST((void));
@@ -201,7 +213,7 @@ int MatchStringInFstab ARGLIST((char *str));
 
 /* edittools.c */
 
-int DoRecursiveEditFiles ARGLIST((char *name, int level, struct Edit *ptr,struct stat *sb));
+void DoRecursiveEditFiles ARGLIST((char *name, int level, struct Edit *ptr));
 void DoEditHomeFiles ARGLIST((struct Edit *ptr));
 void WrapDoEditFile ARGLIST((struct Edit *ptr, char *filename));
 void DoEditFile ARGLIST((struct Edit *ptr, char *filename));
@@ -222,12 +234,15 @@ void WarnIfContainsRegex ARGLIST((void *memseg, off_t size, char *data, char *fi
 void WarnIfContainsFilePattern ARGLIST((void *memseg, off_t size, char *data, char *filename));
 int BinaryReplaceRegex ARGLIST((void *memseg, off_t size, char *search, char *replace, char *filename));
 
-/* encrypt.c */
+/* crypto.c */
 
 void LoadSecretKeys ARGLIST((void));
-void DESMD5Random ARGLIST((char digest[16]));
-void cfencrypt ARGLIST((char *in, char *out, char key1[8], char key2[8], char key3[8], int len));
-void cfdecrypt ARGLIST((char *in, char *out, char key1[8], char key2[8], char key3[8], int len));
+void MD5Random ARGLIST((unsigned char digest[EVP_MAX_MD_SIZE+1]));
+int EncryptString ARGLIST((char *in, char *out, unsigned char *key, int len));
+int DecryptString ARGLIST((char *in, char *out, unsigned char *key, int len));
+RSA *HavePublicKey ARGLIST((char *ipaddress));
+void SavePublicKey ARGLIST((char *ipaddress, RSA *key));
+void GenerateRandomSessionKey ARGLIST((void));
 
 /* errors.c */
 
@@ -237,6 +252,7 @@ void ResetLine ARGLIST((char *s));
 
 /* eval.c */
 
+int Day2Number ARGLIST((char *s));
 int Month2Number ARGLIST((char *s));
 void AddInstallable ARGLIST((char *classlist));
 void AddMultipleClasses ARGLIST((char *classlist));
@@ -262,7 +278,7 @@ int IsBracketed ARGLIST((char *s));
 
 int IsHomeDir ARGLIST((char *name));
 int EmptyDir ARGLIST((char *path));
-int RecursiveCheck ARGLIST((char *name, mode_t plus, mode_t minus, enum fileactions action, struct UidList *uidlist, struct GidList *gidlist, int recurse, int rlevel, struct File *ptr,struct stat *sb));
+void RecursiveCheck ARGLIST((char *name, mode_t plus, mode_t minus, enum fileactions action, struct UidList *uidlist, struct GidList *gidlist, int recurse, int rlevel, struct File *ptr));
 void CheckExistingFile ARGLIST((char *file, mode_t plus, mode_t minus, enum fileactions action, struct UidList *uidlist, struct GidList *gidlist, struct stat *dstat, struct File *ptr, struct Item *acl_aliases));
 void CheckCopiedFile ARGLIST((char *file, mode_t plus, mode_t minus, enum fileactions action, struct UidList *uidlist, struct GidList *gidlist, struct stat *dstat, struct stat *sstat, struct File *ptr, struct Item *acl_aliases));
 int CheckOwner ARGLIST((char *file, enum fileactions action, struct UidList *uidlist, struct GidList *gidlist, struct stat *statbuf));
@@ -274,15 +290,17 @@ void CompressFile ARGLIST((char *file));
 /* filenames.c */
 
 int IsAbsoluteFileName ARGLIST((char *f));
+int RootDirLength ARGLIST((char *f));
 void AddSlash ARGLIST((char *str));
 void DeleteSlash ARGLIST((char *str));
 void DeleteNewline ARGLIST((char *str));
+char *LastFileSeparator ARGLIST((char *str));
 int ChopLastNode ARGLIST((char *str));
 char *CanonifyName ARGLIST((char *str));
 char *Space2Score ARGLIST((char *str));
 char *ASUniqueName ARGLIST((char *str));
 char *ReadLastNode ARGLIST((char *str));
-int MakeDirectoriesFor ARGLIST((char *file));
+int MakeDirectoriesFor ARGLIST((char *file, char force));
 int BufferOverflow ARGLIST((char *str1, char *str2));
 void Chop ARGLIST((char *str));
 int CompressPath ARGLIST((char *dest, char *src));
@@ -344,6 +362,11 @@ void cfclosedir ARGLIST((CFDIR *dirh));
 int CopyReg  ARGLIST((char *source, char *dest, struct stat sstat, struct stat dstat, struct Image *ip));
 void RegisterHardLink ARGLIST((int i, char *value, struct Image *ip));
 
+/* init.c */
+
+void CheckWorkDirectories ARGLIST((void));
+void RandomSeed ARGLIST((void));
+
 /* install.c */
 
 void InstallLocalInfo  ARGLIST((char *varvalue));
@@ -395,28 +418,20 @@ void HandleTidySize ARGLIST((char *value));
 void HandleUmask ARGLIST((char *value));
 void HandleDisableSize ARGLIST((char *value));
 void HandleCopySize ARGLIST((char *value));
-void HandlePurge ARGLIST((char *value));
 void HandleRequiredSize ARGLIST((char *value));
 void HandleTidyType ARGLIST((char *value));
 void HandleTidyLinkDirs ARGLIST((char *value));
 void HandleTidyRmdirs ARGLIST((char *value));
-void HandleForceCopy ARGLIST((char *value));
 void HandleCopyBackup ARGLIST((char *value));
 void HandleTimeOut ARGLIST((char *value));
 void HandleUseShell ARGLIST((char *value));
-void HandleSetLog ARGLIST((char *value));
 void HandleFork ARGLIST((char *value));
-void HandleSetInform ARGLIST((char *value));
-void HandleCompress ARGLIST((char *value));
-void HandleSetSecure ARGLIST((char *value));
 void HandleChecksum ARGLIST((char *value));
-void HandleStealth ARGLIST((char *value));
-void HandleTypecheck ARGLIST((char *value));
 void HandleTimeStamps ARGLIST((char *value));
 int GetFileAction ARGLIST((char *action));
 void InstallFileListItem ARGLIST((char *path, mode_t plus, mode_t minus, enum fileactions action, char *uidnames, char *gidnames, int recurse, char travlinks, char chksum));
 void InstallProcessItem ARGLIST((char *expr, char *restart, short int matches, char comp, short int signal, char action, char *classes, char useshell, char *uidname, char *gidname));
-void InstallImageItem ARGLIST((char *path, mode_t plus, mode_t minus, char *destination, char *action, char *uidnames, char *gidnames, short int force, int size, char comp, short int backup, int rec, char type, char lntype, char *server, char purge, char stealth, short int secure));
+void InstallImageItem ARGLIST((char *path, mode_t plus, mode_t minus, char *destination, char *action, char *uidnames, char *gidnames, int size, char comp, int rec, char type, char lntype, char *server));
 void InstallAuthItem ARGLIST((char *path, char *attribute, struct Auth **list, struct Auth **listtop, char *classes));
 int GetCommAttribute ARGLIST((char *s));
 void HandleRecurse ARGLIST((char *value));
@@ -448,10 +463,18 @@ void AddAuthHostItem ARGLIST((char *path, char *attribute, char *classes, struct
 int AuthPathExists ARGLIST((char *path, struct Auth *list));
 int HandleAdmitAttribute ARGLIST((struct Auth *ptr, char *attribute));
 void PrependTidy ARGLIST((struct TidyPattern **list, char *wild, int rec, short int age, char travlinks, int tidysize, char type, char ldirs, short int tidydirs, char *classes));
-void HandlePreview ARGLIST((char *value));
+void HandleShortSwitch ARGLIST((char *name,char *value,short *flag));
+void HandleCharSwitch ARGLIST((char *name,char *value,char *flag));
+
+/* ip.c */
+
+char *sockaddr_ntop ARGLIST((struct sockaddr *sa));
+short CfenginePort ARGLIST((void));
+int IsIPV6Address ARGLIST((char *name));
 
 /* item-ext.c */
 
+int OrderedListsMatch ARGLIST((struct Item *list1, struct Item *list2));
 int RegexOK ARGLIST((char *string));
 int IsWildItemIn ARGLIST((struct Item *list, char *item));
 void InsertItemAfter  ARGLIST((struct Item **filestart, struct Item *ptr, char *string));
@@ -491,6 +514,7 @@ int CompareToFile ARGLIST((struct Item *liststart, char *file));
 /* item.c */
 
 int IsItemIn ARGLIST((struct Item *list, char *item));
+int IsClassedItemIn ARGLIST((struct Item *list, char *item));
 int IsFuzzyItemIn ARGLIST((struct Item *list, char *item));
 void PrependItem  ARGLIST((struct Item **liststart, char *itemstring, char *classes));
 void AppendItem  ARGLIST((struct Item **liststart, char *itemstring, char *classes));
@@ -511,7 +535,7 @@ int LinkFiles ARGLIST((char *from, char *to, struct Item *inclusions, struct Ite
 int RelativeLink ARGLIST((char *from, char *to, struct Item *inclusions, struct Item *exclusions, struct Item *copy, short int nofile, struct Link *ptr));
 int AbsoluteLink ARGLIST((char *from, char *to, struct Item *inclusions, struct Item *exclusions, struct Item *copy, short int nofile, struct Link *ptr));
 int DoLink  ARGLIST((char *from, char *to, char *defines));
-void KillOldLink ARGLIST((char *name));
+void KillOldLink ARGLIST((char *name, char *defines));
 int HardLinkFiles ARGLIST((char *from, char *to, struct Item *inclusions, struct Item *exclusions, struct Item *copy, short int nofile, struct Link *ptr));
 void DoHardLink  ARGLIST((char *from, char *to, char *defines));
 int ExpandLinks ARGLIST((char *dest, char *from, int level));
@@ -519,15 +543,21 @@ char *AbsLinkPath ARGLIST((char *from, char *relto));
 
 /* locks.c */
 
+void PreLockState ARGLIST((void));
+void InitializeLocks ARGLIST((void));
+void CloseLocks ARGLIST((void));
 void HandleSignal ARGLIST((int signum));
 int GetLock ARGLIST((char *operator, char *operand, int ifelapsed, int expireafter, char *host, time_t now));
 void ReleaseCurrentLock ARGLIST((void));
 int CountActiveLocks ARGLIST((void));
 time_t GetLastLock ARGLIST((void));
 time_t CheckOldLock ARGLIST((void));
-int GetLockPid ARGLIST((void));
 void SetLock ARGLIST((void));
 void LockLog ARGLIST((int pid, char *str, char *operator, char *operand));
+int PutLock ARGLIST((char *name));
+int DeleteLock ARGLIST((char *name));
+time_t GetLockTime ARGLIST((char *name));
+pid_t GetLockPid ARGLIST((char *name));
 
 /* log.c */
 
@@ -548,27 +578,31 @@ int CompareMacro ARGLIST((char *name, char *macro));
 void DeleteMacros ARGLIST((void));
 void DeleteMacro  ARGLIST((char *name));
 
+
 /* misc.c */
 
-int DirPush ARGLIST((char *name,struct stat *sb));
-void DirPop ARGLIST((int goback,char *name,struct stat *sb));
-void CheckLinkSecurity ARGLIST((struct stat *sb, char *name));
 void GetNameInfo ARGLIST((void));
 void AddNetworkClass ARGLIST((char *netmask));
 void TruncateFile ARGLIST((char *name));
 int FileSecure  ARGLIST((char *name));
-int ChecksumChanged ARGLIST((char *filename, char *digest, int warnlevel, int refresh));
+int ChecksumChanged ARGLIST((char *filename, unsigned char digest[EVP_MAX_MD_SIZE+1], int warnlevel, int refresh, char type));
+char *ChecksumPrint   ARGLIST((char type,unsigned char digest[EVP_MAX_MD_SIZE+1]));
+void ChecksumFile  ARGLIST((char *filename,unsigned char digest[EVP_MAX_MD_SIZE+1],char type));
+int ChecksumsMatch ARGLIST((unsigned char digest1[EVP_MAX_MD_SIZE+1],unsigned char digest2[EVP_MAX_MD_SIZE+1],char type));
+void ChecksumString  ARGLIST((char *buffer,int len,unsigned char digest[EVP_MAX_MD_SIZE+1],char type));
 int IgnoredOrExcluded ARGLIST((enum actions action, char *file, struct Item *inclusions, struct Item *exclusions));
 void Banner ARGLIST((char *string));
 void SetDomainName ARGLIST((char *sp));
 void GetInterfaceInfo ARGLIST((void));
-
-
+void GetV6InterfaceInfo ARGLIST((void));
+void DebugBinOut ARGLIST((char *string, int len));
+int ShellCommandReturnsZero ARGLIST((char *comm));
+void SetClassesOnScript ARGLIST((char *comm, char *classes, char *elseclasses, int useshell));
 /* modes.c */
 
 void ParseModeString ARGLIST((char *modestring, mode_t *plusmask, mode_t *minusmask));
-void CheckModeState ARGLIST((enum modestate stateA, int stateB, char ch));
-void SetMask ARGLIST((char action, int value, mode_t *p, mode_t *m));
+void CheckModeState ARGLIST((enum modestate stateA, enum modestate stateB,enum modesort modeA, enum modesort modeB, char ch));
+void SetMask ARGLIST((char action, int value, int affected, mode_t *p, mode_t *m));
 
 /* mount.c */
 
@@ -579,11 +613,18 @@ int IsMountedFileSystem  ARGLIST((struct stat *childstat, char *dir, int rlevel)
 /* net.c */
 
 void TimeOut ARGLIST((void));
+int SendTransaction ARGLIST((int sd, char *buffer,int len, char status));
+int ReceiveTransaction ARGLIST((int sd, char *buffer,int *more));
 int RecvSocketStream ARGLIST((int sd, char *buffer, int toget, int nothing));
+int SendSocketStream ARGLIST((int sd, char *buffer, int toget, int flags));
 
 /* parse.c */
 
 void ParseInputFiles ARGLIST((void));
+int ParseBootFiles ARGLIST((void));
+void ParseFile ARGLIST((char *f,char *env));
+void NewParser ARGLIST((void));
+void DeleteParser ARGLIST((void));
 void SetAction  ARGLIST((enum actions action));
 void HandleId ARGLIST((char *id));
 void HandleClass  ARGLIST((char *id));
@@ -592,11 +633,17 @@ void HandlePath  ARGLIST((char *path));
 void HandleVarpath  ARGLIST((char *varpath));
 void HandleWildcard ARGLIST((char *wildcard));
 int CompoundId ARGLIST((char *id));
-int ShellCommandReturnsZero ARGLIST((char *comm));
 void InitializeAction ARGLIST((void));
 void SetMountPath  ARGLIST((char *value));
 void SetRepository  ARGLIST((char *value));
 
+/* strategies.c */
+
+void InstallStrategy ARGLIST((char *value, char *classes));
+void AddClassToStrategy ARGLIST((char *alias,char *class,char *value));
+void SetStrategies ARGLIST((void));
+void GetNonMarkov ARGLIST((void));
+
 /* patches.c */
 
 #ifndef HAVE_GETNETGRENT
@@ -658,7 +705,11 @@ int FindMatches ARGLIST((struct Process *pp, struct Item *procdata, struct Item
 void DoSignals ARGLIST((struct Process *pp,struct Item *list));
 
 /* proto.c */
-int IdentifyForVerification ARGLIST((int sd, char *fqname));
+int IdentifyForVerification ARGLIST((int sd,char *localip, int family));
+int KeyAuthentication ARGLIST((struct Image *ip));
+int BadProtoReply  ARGLIST((char *buf));
+int OKProtoReply  ARGLIST((char *buf));
+int FailedProtoReply  ARGLIST((char *buf));
 
 /* read.c */
 int ReadLine ARGLIST((char *buff, int size, FILE *fp));
@@ -666,6 +717,7 @@ int ReadLine ARGLIST((char *buff, int size, FILE *fp));
 /* report.c */
 
 void ListDefinedClasses ARGLIST((void));
+void ListDefinedStrategies ARGLIST((void));
 void ListDefinedInterfaces ARGLIST((void));
 void ListDefinedHomePatterns ARGLIST((void));
 void ListDefinedBinservers ARGLIST((void));
@@ -705,9 +757,9 @@ int SensibleFile ARGLIST((char *nodename, char *path, struct Image *ip));
 
 /* tidy.c */
 
-int RecursiveHomeTidy ARGLIST((char *name, int level, struct Tidy *tp,struct stat *sb));
-void TidyHomeFile ARGLIST((char *path, char *name, struct stat *statbuf));
-int RecursiveTidySpecialArea ARGLIST((char *name, struct Tidy *tp, int maxrecurse,struct stat *sb));
+void RecursiveHomeTidy ARGLIST((char *name, int level));
+int TidyHomeFile ARGLIST((char *path, char *name,struct stat *statbuf, int level));
+void RecursiveTidySpecialArea ARGLIST((char *name, struct Tidy *tp, int maxrecurse));
 void TidyParticularFile ARGLIST((char *path, char *name, struct Tidy *tp, struct stat *statbuf, int is_dir, int level));
 void DoTidyFile ARGLIST((char *path, char *name, struct TidyPattern *tlp, struct stat *statbuf, short int logging_this));
 void DeleteTidyList ARGLIST((struct TidyPattern *list));
