@@ -40,7 +40,7 @@
 
 /*********************************************************************/
 
-ReadLine(buff,size,fp)
+int ReadLine(buff,size,fp)
 
 char *buff;
 int size;
