@@ -158,6 +158,11 @@ printf ("\nDefined Binservers = ( ");
 for (ptr = VBINSERVERS; ptr != NULL; ptr=ptr->next)
    {
    printf("%s ",ptr->name);
+   
+   if (ptr->classes)
+      {
+      printf("(if defined %s), ",ptr->classes);
+      }
    }
 
 printf (")\n");
@@ -174,7 +179,7 @@ printf ("\nDEFINED LINKS\n\n");
 
 for (ptr = VLINK; ptr != NULL; ptr=ptr->next)
    {
-   printf("\nFrom %s -> %s force=%d, attr=%d type=%c nofile=%d\n",ptr->from,ptr->to,ptr->force,ptr->silent,ptr->type, ptr->nofile);
+   printf("\nFrom %s -> %s force=%c, attr=%d type=%c nofile=%d\n",ptr->from,ptr->to,ptr->force,ptr->silent,ptr->type, ptr->nofile);
    for (ip = ptr->copy; ip != NULL; ip = ip->next)
       {
       printf(" Copy %s\n",ip->name);
@@ -195,6 +200,11 @@ for (ptr = VLINK; ptr != NULL; ptr=ptr->next)
       printf(" Ignore %s\n",ip->name);
       }
 
+   for (ip = ptr->filters; ip != NULL; ip = ip->next)
+      {
+      printf(" Filters %s\n",ip->name);
+      }
+
    if (ptr->defines)
       {
       printf(" Define %s\n",ptr->defines);
@@ -218,7 +228,7 @@ printf ("\nDEFINED CHILD LINKS\n\n");
 
 for (ptr = VCHLINK; ptr != NULL; ptr=ptr->next)
    {
-   printf("\n%s->%s force=%d attr=%d, rec=%d\n",ptr->from,ptr->to,
+   printf("\n%s->%s force=%c attr=%d, rec=%d\n",ptr->from,ptr->to,
 	  ptr->force,ptr->silent,ptr->recurse);
    
    for (ip = ptr->copy; ip != NULL; ip = ip->next)
@@ -263,7 +273,7 @@ printf ("\nDEFINED NAMESERVERS\n\n");
 
 for (ptr = VRESOLVE; ptr != NULL; ptr=ptr->next)
    {
-   printf("%s\n",ptr->name);
+   printf("%s (%s)\n",ptr->name,ptr->classes);
    }
 }
 
@@ -278,7 +288,7 @@ printf ("\nDEFINED SHELLCOMMANDS\n\n");
 for (ptr = VSCRIPT; ptr != NULL; ptr=ptr->next)
    {
    printf("\n%s\n timeout=%d\n uid=%d,gid=%d\n",ptr->name,ptr->timeout,ptr->uid,ptr->gid);
-   printf(" umask = %c, background = %c\n",ptr->umask,ptr->fork);
+   printf(" umask = %o, background = %c\n",ptr->umask,ptr->fork);
    printf (" ChDir=%s, ChRoot=%s\n",ptr->chdir,ptr->chroot);
    
    if (ptr->defines)
@@ -315,13 +325,13 @@ for (svp = VSERVERLIST; svp != NULL; svp=svp->next) /* order servers */
    for (ptr = VIMAGE; ptr != NULL; ptr=ptr->next)
       {
       if (strcmp(svp->name,ptr->server) != 0)  /* group together similar hosts so */
-	 {                                  /* can can do multiple transactions */
-	 continue;                          /* on one connection */
+	 {                                     /* can can do multiple transactions */
+	 continue;                             /* on one connection */
 	 } 
       
       printf("\n%s\n +%o\n -%o\n dest: %s\n action: %s\n",ptr->path,ptr->plus,ptr->minus,
 	     ptr->destination,ptr->action);
-      
+
       printf(" Size %c %d\n",ptr->comp,ptr->size);
       
       if (ptr->recurse == INFINITERECURSE)
@@ -388,7 +398,8 @@ for (svp = VSERVERLIST; svp != NULL; svp=svp->next) /* order servers */
       
       printf(" method = %c (time/checksum)\n",ptr->type);
       
-      printf(" server = %s (secure=%d)\n",ptr->server,ptr->secure);
+      printf(" server = %s (encrypt=%c,verified=%c)\n",ptr->server,ptr->encrypt,ptr->verify);
+      printf(" accept the server's public key on trust? %c\n",ptr->trustkey);
       
       printf(" purging = %c\n",ptr->purge);
       
@@ -408,7 +419,7 @@ for (svp = VSERVERLIST; svp != NULL; svp=svp->next) /* order servers */
 	 printf(" ElseDefine %s\n",ptr->elsedef);
 	 }
       
-      if (! ptr->backup)
+      if (ptr->backup == 'n')
 	 {
 	 printf(" NOT BACKED UP\n");
 	 }
@@ -418,15 +429,21 @@ for (svp = VSERVERLIST; svp != NULL; svp=svp->next) /* order servers */
 	 printf(" Local repository = %s\n",ptr->repository);
 	 }
       
-      if (ptr->stealth == 't')
+      if (ptr->stealth == 'y')
 	 {
 	 printf(" Stealth copy\n");
 	 }
 
-      if (ptr->preservetimes == 't')
+      if (ptr->preservetimes == 'y')
 	 {
 	 printf(" File times preserved\n");
 	 }
+
+      if (ptr->forcedirs == 'y')
+	 {
+	 printf(" Forcible movement of obstructing files in recursion\n");
+	 }
+
       }
    }
 }
@@ -443,7 +460,14 @@ printf ("\nDEFINED TIDY MASKS\n\n");
 
 for (ptr = VTIDY; ptr != NULL; ptr=ptr->next)
    {
-   printf("%s\n",ptr->path);
+   if (ptr->maxrecurse == INFINITERECURSE)
+      {
+      printf("%s (maxrecurse = inf)\n",ptr->path);
+      }
+   else
+      {
+      printf("%s (maxrecurse = %d)\n",ptr->path,ptr->maxrecurse);
+      }
    
    for(tp = ptr->tidylist; tp != NULL; tp=tp->next)
       {
@@ -530,6 +554,12 @@ printf ("\nDefined home servers = ( ");
 for (ptr = VHOMESERVERS; ptr != NULL; ptr=ptr->next)
    {
    printf("%s ",ptr->name);
+      
+   if (ptr->classes)
+      {
+      printf("(if defined %s), ",ptr->classes);
+      }
+
    }
 
 printf (")\n");
@@ -545,8 +575,8 @@ printf ("\nDEFINED DISABLE\n\n");
 
 for (ptr = VDISABLELIST; ptr != NULL; ptr=ptr->next)
    {
-   printf("\n %s:\n rotate=%d, type=%s, size%c%d\n",
-	  ptr->name,ptr->rotate,ptr->type,ptr->comp,ptr->size);
+   printf("\n %s:\n rotate=%d, type=%s, size%c%d action=%c\n",
+	  ptr->name,ptr->rotate,ptr->type,ptr->comp,ptr->size,ptr->action);
 
    if (ptr->repository)
       {
@@ -838,20 +868,44 @@ for (ptr = VACLLIST; ptr != NULL; ptr=ptr->next)
 
 }
 
+
+/*******************************************************************/
+
+void ListDefinedStrategies()
+
+{ struct Strategy *ptr;
+  struct Item *ip;
+ 
+printf("\nDEFINED STRATEGIES\n\n");
+
+for (ptr = VSTRATEGYLIST; ptr != NULL; ptr=ptr->next)
+   {
+   printf("%s (type=%c)\n",ptr->name,ptr->type);
+   if (ptr->strategies)
+      {
+      for (ip = ptr->strategies; ip !=NULL; ip=ip->next)
+	 {
+	 printf("  %s - weight %s\n",ip->name,ip->classes);
+	 }
+      }
+   printf("\n");
+   }
+
+}
+
 /*******************************************************************/
 
 void ListFileEdits()
 
 { struct Edit *ptr;
   struct Edlist *ep;
-  struct Item *ip;
 
 printf("\nDEFINED FILE EDITS\n\n");
 
 for (ptr=VEDITLIST; ptr != NULL; ptr=ptr->next)
    {
-   printf("%s (r=%d)\n",ptr->fname,ptr->recurse);
-
+   printf("%s (%c)(r=%d)\n",ptr->fname,ptr->done,ptr->recurse);
+   
    if (ptr->repository)
       {
       printf(" Local repository = %s\n",ptr->repository);
@@ -877,7 +931,6 @@ for (ptr=VEDITLIST; ptr != NULL; ptr=ptr->next)
 void ListFilters()
 
 { struct Filter *ptr;
-  struct FilterList *ep;
   int i;
 
 printf("\nDEFINED FILTERS\n");
