@@ -44,20 +44,26 @@ char *startpath;
 void *vp;
 
 { struct Tidy *tp;
-  struct stat sb;
- 
+
 tp = (struct Tidy *) vp;
 
 Debug2("TidyWrapper(%s)\n",startpath);
- 
-if (stat(startpath,&sb) == -1)
+
+if (tp->done == 'y')
+   {
+   return;
+   }
+
+tp->done = 'y'; 
+
+if (!GetLock(ASUniqueName("tidy"),CanonifyName(startpath),VIFELAPSED,VEXPIREAFTER,VUQNAME,CFSTARTTIME))
    {
-   snprintf(OUTPUT,bufsize,"Tidy directory %s cannot be accessed",startpath);
-   CfLog(cfinform,OUTPUT,"stat");
    return;
    }
  
-RecursiveTidySpecialArea(startpath,tp,tp->recurse,&sb);
+RecursiveTidySpecialArea(startpath,tp,tp->maxrecurse);
+
+ReleaseCurrentLock(); 
 }
 
 /*******************************************************************/
@@ -67,18 +73,17 @@ void RecHomeTidyWrapper(startpath,vp)
 char *startpath;
 void *vp;
 
-{ struct stat sb;
- 
-Verbose("Tidying %s...\n",startpath);
+{
+Verbose("Tidying Home partition %s...\n",startpath);
 
-if (stat(startpath,&sb) == -1)
+if (!GetLock(ASUniqueName("tidy"),CanonifyName(startpath),VIFELAPSED,VEXPIREAFTER,VUQNAME,CFSTARTTIME))
    {
-   snprintf(OUTPUT,bufsize,"Tidy directory %s cannot be accessed",startpath);
-   CfLog(cfinform,OUTPUT,"stat");
    return;
    }
+
+RecursiveHomeTidy(startpath,1);
  
-RecursiveHomeTidy(startpath,1,vp,&sb);
+ReleaseCurrentLock();
 }
 
 /*******************************************************************/
@@ -94,6 +99,11 @@ void *vp;
    int fd;
    struct File *ptr;
 
+if (!GetLock(ASUniqueName("files"),CanonifyName(startpath),VIFELAPSED,VEXPIREAFTER,VUQNAME,CFSTARTTIME))
+   {
+   return;
+   }
+
 ptr = (struct File *)vp;
 
 if ((strlen(startpath) == 0) || (startpath == NULL))
@@ -129,20 +139,22 @@ if (ptr->inclusions != NULL && !IsWildItemIn(ptr->inclusions,lastnode))
 if (IsWildItemIn(ptr->exclusions,lastnode))
    {
    Debug2("Skipping excluded pattern file %s\n",lastnode);
+   ReleaseCurrentLock();
    return;
    }
     
 if (stat(startpath,&statbuf) == -1)
    {
-   sprintf(OUTPUT,"Cannot access file/directory %s\n",ptr->path);
+   snprintf(OUTPUT,bufsize*2,"Cannot access file/directory %s\n",ptr->path);
    CfLog(cfinform,OUTPUT,"stat");
 
    if (TouchDirectory(ptr))                /* files ending in /. */
       {
-      MakeDirectoriesFor(startpath);
+      MakeDirectoriesFor(startpath,'n');
       ptr->action = fixall;
       *(startpath+strlen(ptr->path)-2) = '\0';       /* trunc /. */
       CheckExistingFile(startpath,ptr->plus,ptr->minus,ptr->action,ptr->uid,ptr->gid,&statbuf,ptr,ptr->acl_aliases);
+      ReleaseCurrentLock();
       return;
       }
 
@@ -155,7 +167,7 @@ if (stat(startpath,&statbuf) == -1)
       case create:
       case touch:   if (! DONTDO)
                        {
-                       MakeDirectoriesFor(startpath);
+                       MakeDirectoriesFor(startpath,'n');
 		       
                        if ((fd = creat(ptr->path,filemode)) == -1)
 			  { 
@@ -170,17 +182,17 @@ if (stat(startpath,&statbuf) == -1)
 		       CheckExistingFile(startpath,ptr->plus,ptr->minus,fixall,ptr->uid,ptr->gid,&statbuf,ptr,ptr->acl_aliases);
                        }
 
-                    sprintf(OUTPUT,"Creating file %s, mode = %o\n",ptr->path,filemode);
+                    snprintf(OUTPUT,bufsize*2,"Creating file %s, mode = %o\n",ptr->path,filemode);
 		    CfLog(cfinform,OUTPUT,"");
                     break;
-
+      case alert:
       case linkchildren: 
       case warnall:
       case warnplain:
       case warndirs:
       case fixplain:
       case fixdirs:
-      case fixall:  sprintf(OUTPUT,"File/Dir %s did not exist and was marked (%s)\n",ptr->path,FILEACTIONTEXT[ptr->action]);
+      case fixall:  snprintf(OUTPUT,bufsize*2,"File/Dir %s did not exist and was marked (%s)\n",ptr->path,FILEACTIONTEXT[ptr->action]);
 	            CfLog(cfinform,OUTPUT,"");
                     break;
       case compress:
@@ -193,11 +205,13 @@ else
    {
    if (TouchDirectory(ptr))  /* Don't check just touch */
       {
+      ReleaseCurrentLock();
       return;
       }
 
    if (ptr->action == create)
       {
+      ReleaseCurrentLock();
       return;
       }
 
@@ -212,6 +226,7 @@ else
 		   NULL,
 		   0,
 		   NULL);
+      ReleaseCurrentLock();
       return;
       }
 
@@ -219,16 +234,20 @@ else
       {
       if (!IgnoreFile(startpath,ReadLastNode(startpath),ptr->ignores))
 	 {
-	 Verbose("%s: Skipping ignored root-search directory %s\n",VPREFIX,startpath);
+	 Verbose("%s: Skipping ignored directory %s\n",VPREFIX,startpath);
 	 CheckExistingFile(startpath,ptr->plus,ptr->minus,ptr->action,ptr->uid,ptr->gid,&statbuf,ptr,ptr->acl_aliases);
 	 }
-      RecursiveCheck(startpath,ptr->plus,ptr->minus,ptr->action,ptr->uid,ptr->gid,ptr->recurse,0,ptr,&statbuf); 
+
+          
+      RecursiveCheck(startpath,ptr->plus,ptr->minus,ptr->action,ptr->uid,ptr->gid,ptr->recurse,0,ptr);
       }
    else
       {
       CheckExistingFile(startpath,ptr->plus,ptr->minus,ptr->action,ptr->uid,ptr->gid,&statbuf,ptr,ptr->acl_aliases);
       }
    }
+
+ReleaseCurrentLock();
 }
 
 /*******************************************************************/
@@ -250,15 +269,21 @@ ExpandVarstring(dir,directory,"");
 AddSlash(directory);
 strcat(directory,".");
  
-MakeDirectoriesFor(directory);
+MakeDirectoriesFor(directory,'n');
  
 if (stat(directory,&statbuf) == -1)
    {
-   sprintf(OUTPUT,"Cannot stat %s\n",directory);
+   snprintf(OUTPUT,bufsize*2,"Cannot stat %s\n",directory);
    CfLog(cfinform,OUTPUT,"stat");
    return;
    }
- 
+
+if (!GetLock(ASUniqueName("directories"),CanonifyName(directory),VIFELAPSED,VEXPIREAFTER,VUQNAME,CFSTARTTIME))
+   {
+   return;
+   }
+
 CheckExistingFile(dir,ptr->plus,ptr->minus,ptr->action,ptr->uid,ptr->gid,&statbuf,ptr,ptr->acl_aliases);
+ReleaseCurrentLock(); 
 }
 
